<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\File;

class MakeService extends Command
{
    protected $signature = 'make:service {name}';

    protected $description = 'Create a new service class';

    public function __construct()
    {
        parent::__construct();
    }

    public function handle()
    {
        $name = $this->argument('name');
        $fileName = app_path('Services/' . $name . 'Service.php');

        if (File::exists($fileName)) {
            $this->error('Service already exists!');
            return;
        }

        File::put($fileName, $this->generateServiceStub($name));

        $this->info('Service created successfully!');
    }

    protected function generateServiceStub($name)
    {
        $stub = "<?php\n\n";
        $stub .= "namespace App\Services;\n\n";
        $stub .= "class {$name}Service\n";
        $stub .= "{\n";
        $stub .= "    // Your service logic goes here\n";
        $stub .= "}\n";

        return $stub;
    }
}
