<?php

namespace App\Exports;

use App\Ingreso;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Concerns\FromQuery;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithCustomCsvSettings;

class CompraProveedorExport implements FromQuery, WithCustomCsvSettings
{
    protected $startDate;
    protected $endDate;

    public function __construct($startDate, $endDate)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
    }

    public function query()
    {
        $consulta = DB::table('cp_orden_compra')
            ->join('proveedores', 'cp_orden_compra.idproveedor', '=', 'proveedores.id')
            ->select(
                'cp_orden_compra.fecha_liva AS fecha_liva',
                'cp_orden_compra.clase_documento AS tipodocanx',
                DB::raw("CASE WHEN cp_orden_compra.tipo_documento = 1 THEN '03' END AS clasedocanx"),
                'cp_orden_compra.num_documento AS num_documento',
                DB::raw("CASE WHEN cp_orden_compra.tipo_compra IN (1, 2) THEN proveedores.nit END AS nrc"),
                'proveedores.nombre AS nombre',
                DB::raw("ROUND(cp_orden_compra.cext_locales, 2) AS cext_locales"),
                DB::raw("'0.00' AS Hanx"),
                DB::raw("'0.00' AS Ianx"),
                DB::raw("ROUND(cp_orden_compra.sub_total_oc, 2) AS subinterna"),
                DB::raw("'0.00' AS kanx"),
                DB::raw("'0.00' AS lanx"),
                DB::raw("'0.00' AS manx"),
                DB::raw("ROUND(cp_orden_compra.iva_oc, 2) AS iva_oc"),
                DB::raw("ROUND(cp_orden_compra.total_oc, 2) AS total_oc"),
                DB::raw("' ' AS panx"),
                DB::raw("'0' AS coperacion"),
                DB::raw("'0' AS codclasi"),
                DB::raw("'0' AS codsec"),
                DB::raw("'0' AS codgascos"),
                DB::raw("'3' AS uanx")
            )
            ->where('cp_orden_compra.ver_libro', 1)
            ->where('cp_orden_compra.fecha_liva', '>=', '2024-09-01');

        $unionQuery1 = DB::table('cp_compras_cc')
            ->join('mhcp_tipo_operacion', 'cp_compras_cc.tipo_operacion', '=', 'mhcp_tipo_operacion.codigo')
            ->join('mhcp_clasificacion', 'cp_compras_cc.clasificacion', '=', 'mhcp_clasificacion.codigo')
            ->join('mhcp_sector', 'cp_compras_cc.sector', '=', 'mhcp_sector.codigo')
            ->join('mhcp_tipo_gastocosto', 'cp_compras_cc.tipo_costo_gasto', '=', 'mhcp_tipo_gastocosto.codigo')
            ->join('conta_cuentas_contables', 'cp_compras_cc.idCuenta', '=', 'conta_cuentas_contables.id')
            ->select(
                'cp_compras_cc.fechalibroiva AS fechalibroiva',
                'cp_compras_cc.clasedoc AS tipodocanx',
                'cp_compras_cc.tipodoc AS clasedocanx',
                'cp_compras_cc.numdoc AS numdoc',
                DB::raw("CASE WHEN cp_compras_cc.tipodoc IN ('03', '05') THEN cp_compras_cc.nrcnit END AS nrc"),
                'cp_compras_cc.proveedor AS proveedor',
                DB::raw("ROUND(cp_compras_cc.fovial + cp_compras_cc.contrans + cp_compras_cc.cesc, 2) AS cext_locales"),
                DB::raw("'0.00' AS Hanx"),
                DB::raw("'0.00' AS Ianx"),
                DB::raw("ROUND(
            CASE 
                WHEN cp_compras_cc.tipodoc IN ('03', 4) THEN cp_compras_cc.subtotal 
                WHEN cp_compras_cc.tipodoc = '05' THEN cp_compras_cc.subtotal 
                ELSE 0 
            END, 2
        ) AS subinterna"),
                DB::raw("'0.00' AS kanx"),
                DB::raw("'0.00' AS lanx"),
                DB::raw("'0.00' AS manx"),
                DB::raw("ROUND(
            CASE 
                WHEN cp_compras_cc.tipodoc IN ('03', 4) THEN cp_compras_cc.iva 
                ELSE 0 
            END, 2
        ) AS iva"),
                DB::raw("ROUND(
            CASE 
                WHEN cp_compras_cc.tipodoc IN ('03', 4) THEN cp_compras_cc.total 
                ELSE 0 
            END, 2
        ) AS total_pagar"),
                DB::raw("' ' AS panx"),
                'mhcp_tipo_operacion.codigo AS coperacion',
                'mhcp_clasificacion.codigo AS codclasi',
                'mhcp_sector.codigo AS codsec',
                'mhcp_tipo_gastocosto.codigo AS codgascos',
                DB::raw("'3' AS uanx")
            )
            ->where('cp_compras_cc.ver_liva', 1)
            ->where('cp_compras_cc.fechalibroiva', '>=', '2024-09-01')
            ->whereIn('cp_compras_cc.tipodoc', ['05', '03']);

        $unionQuery2 = DB::table('compra_json')
            ->join('mhcp_tipo_operacion', 'compra_json.tipo_operacion', '=', 'mhcp_tipo_operacion.codigo')
            ->join('mhcp_clasificacion', 'compra_json.clasificacion', '=', 'mhcp_clasificacion.codigo')
            ->join('mhcp_sector', 'compra_json.sector', '=', 'mhcp_sector.codigo')
            ->join('mhcp_tipo_gastocosto', 'compra_json.tipo_costo_gasto', '=', 'mhcp_tipo_gastocosto.codigo')
            ->join('conta_cuentas_contables', 'compra_json.idCuenta', '=', 'conta_cuentas_contables.id')
            ->select(
                'compra_json.fecha_libro_iva AS fechalibroiva',
                DB::raw("'4' AS tipodocanx"),
                'compra_json.tipoDte AS clasedocanx',
                'compra_json.codigoGeneracion AS numdoc',
                'compra_json.emi_nit AS nit',
                'compra_json.emi_nombre AS proveedor',
                DB::raw("ROUND(compra_json.resu_tribu_c8 + compra_json.resu_tribu_d1, 2) AS cext_locales"),
                DB::raw("'0.00' AS Hanx"),
                DB::raw("'0.00' AS Ianx"),
                DB::raw("CASE WHEN compra_json.tipoDte = '03' THEN compra_json.resu_subtotal END AS subinterna"),
                DB::raw("'0.00' AS kanx"),
                DB::raw("'0.00' AS lanx"),
                DB::raw("'0.00' AS manx"),
                DB::raw("CASE WHEN compra_json.tipoDte = '03' THEN compra_json.resu_tribu_20 END AS iva"),
                DB::raw("CASE WHEN compra_json.tipoDte = '03' THEN compra_json.resu_totalpagar END AS total_pagar"),
                DB::raw("' ' AS panx"),
                'mhcp_tipo_operacion.codigo AS coperacion',
                'mhcp_clasificacion.codigo AS codclasi',
                'mhcp_sector.codigo AS codsec',
                'mhcp_tipo_gastocosto.codigo AS codgascos',
                DB::raw("'3' AS uanx")
            )
            ->where('compra_json.fecha_libro_iva', '>=', '2024-09-01');

        $resultado = $consulta
            ->union($unionQuery1)
            ->union($unionQuery2)
            ->get();

        return $resultado;
    }

    public function getCsvSettings(): array
    {
        return [
            'delimiter' => ';',
            'enclosure' => '',
        ];
    }
}
