<?php

namespace App\Exports;

use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Concerns\FromQuery;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithCustomCsvSettings;

class VentaConsumidorFinal implements FromQuery,WithCustomCsvSettings
{
    protected $startDate;
    protected $endDate;
    protected $anulada;
    protected $idBodega;

    public function __construct($startDate, $endDate, $anulada, $idBodega)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->anulada = $anulada;
        $this->idBodega = $idBodega;
    }

    public function query()
    {
        return DB::table('fc_factura')
            ->join('cliente', 'fc_factura.idCliente', '=', 'cliente.id')
            ->select(
                DB::raw("DATE_FORMAT(fc_factura.fecha_factura, '%d/%m/%Y') AS fecha_factura"),
                DB::raw("'4' AS clasedoc"),
                'fc_factura.tipo_documento AS tipo_documento',
                DB::raw("REPLACE(fc_factura.codigoGeneracion, '-', '') AS NUMRESO"),
                'fc_factura.selloRecibido AS SERIE',
                DB::raw("'' AS NCONTROLDEL"),
                DB::raw("'' AS NCONTROLAL"),
                DB::raw("REPLACE(fc_factura.numeroControl, '-', '') AS NDOCDEL"),
                DB::raw("REPLACE(fc_factura.numeroControl, '-', '') AS NDOCAL"),
                DB::raw("'' AS NMAQUINA"),
                DB::raw("CAST(fc_factura.vta_exentas AS CHAR) AS vta_exentas"),
                DB::raw("'0.00' AS vtainternasexentas"),
                DB::raw("CAST(fc_factura.vta_nosuje AS CHAR) AS vta_nosuje"),
                DB::raw("(CASE WHEN (fc_factura.tipo_documento = '01') THEN fc_factura.sub_total_fc WHEN (fc_factura.tipo_documento <> '01') THEN CAST(fc_factura.fovial AS CHAR) END) AS sub_total_fc"),
                DB::raw("(CASE WHEN (fc_factura.tipo_documento = '11') THEN ((fc_factura.sub_total_fc + fc_factura.seguro) + fc_factura.flete) WHEN (fc_factura.tipo_documento <> '11') THEN CAST(fc_factura.fovial AS CHAR) END) AS exportadca"),
                DB::raw("'0.00' AS exportfuerca"),
                DB::raw("'0.00' AS exposerv"),
                DB::raw("'0.00' AS vtazfran"),
                DB::raw("'0.00' AS vtacuentater"),
                'fc_factura.total_fc AS total_fc',
                DB::raw("'2' AS nanexo")
            )
            ->whereBetween('fc_factura.fecha_factura', [$this->startDate, $this->endDate])
            ->where('fc_factura.idBodega', 4)
            ->whereNotNull('fc_factura.selloRecibido')
            ->whereIn('fc_factura.tipo_documento', ['01', '11'])
            ->where('fc_factura.anulada', 0)
            ->where('fc_factura.esPrefactura', 0)
            ->orderBy('fc_factura.fecha_factura');
    }

   


    public function getCsvSettings(): array
    {
        return [
            'delimiter' => ';',
            'enclosure' => '',
        ];
    }
}
