<?php

namespace App\Exports;

use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Concerns\FromQuery;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithCustomCsvSettings;

class VentaCreditoFiscalExport implements FromQuery,WithCustomCsvSettings
{
    protected $startDate;
    protected $endDate;
    protected $anulada;
    protected $idBodega;

    public function __construct($startDate, $endDate, $anulada, $idBodega)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->anulada = $anulada;
        $this->idBodega = $idBodega;
    }

    public function query()
    {
        return DB::table('fc_factura')
            ->join('cliente', 'fc_factura.idCliente', '=', 'cliente.id')
            ->select(
                DB::raw("DATE_FORMAT(fc_factura.fecha_factura, '%d/%m/%Y') AS fechaemision"),
                DB::raw("'4' AS clasedoc"),
                'fc_factura.tipo_documento AS tipodoc',
                DB::raw("REPLACE(fc_factura.numeroControl, '-', '') AS numreso"),
                'fc_factura.selloRecibido AS numserie',
                DB::raw("REPLACE(fc_factura.codigoGeneracion, '-', '') AS numdoc"),
                DB::raw("'' AS numcontrolinterno"),
                DB::raw("REPLACE(cliente.nit, '-', '') AS NIT"),
                'cliente.nombre AS nombre',
                DB::raw("FORMAT(fc_factura.vta_exentas, 2) AS exentas"),
                DB::raw("FORMAT(fc_factura.vta_nosuje, 2) AS nosuje"),
                DB::raw("REPLACE(FORMAT(fc_factura.sub_total_fc, 2),',','') AS subgrava"),
                DB::raw("REPLACE(FORMAT(fc_factura.iva_fc, 2),',','') AS iva"),
                DB::raw("FORMAT(fc_factura.vta_ctercero, 2) AS vtaterce"),
                DB::raw("'0.00' AS dfiscalctaterce"),
                DB::raw("REPLACE(FORMAT(fc_factura.total_fc, 2),',','') AS total"),
                DB::raw("'' AS dui"),
                DB::raw("1 AS anexo")
            )
            ->whereBetween('fc_factura.fecha_factura', [$this->startDate, $this->endDate])
            ->whereIn('fc_factura.tipo_documento', ['03', '05'])
            ->whereNotNull('fc_factura.selloRecibido')
            ->where('fc_factura.idBodega', 4)
            ->where('fc_factura.anulada', 0)
            ->where('fc_factura.esPrefactura', 0)
            ->orderBy('fc_factura.fecha_factura');
    }

    // public function headings(): array
    // {
    //     return [
    //         'fechaemision',
    //         'clasedoc',
    //         'tipodoc',
    //         'numreso',
    //         'numserie',
    //         'numdoc',
    //         'numcontrolinterno',
    //         'NIT',
    //         'nombre',
    //         'exentas',
    //         'nosuje',
    //         'subgrava',
    //         'iva',
    //         'vtaterce',
    //         'dfiscalctaterce',
    //         'total',
    //         'dui',
    //         'anexo'
    //     ];
    // }


    public function getCsvSettings(): array
    {
        return [
            'delimiter' => ';',
            'enclosure' => '',
        ];
    }
}
