<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Bodega;
use App\Persona;
use App\User;
use App\BodegaEmpleado;
use App\BodegaDocumento;
use App\BodegaSala;
use App\Http\Controllers\Controller;

class BodegaController extends Controller
{
     public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $bodegas = Bodega::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{
            $bodegas = Bodega::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $bodegas->total(),
                'current_page' => $bodegas->currentPage(),
                'per_page'     => $bodegas->perPage(),
                'last_page'    => $bodegas->lastPage(),
                'from'         => $bodegas->firstItem(),
                'to'           => $bodegas->lastItem(),
            ],
            'bodegas' => $bodegas
        ];
    }


    public function listarPdf(){
        $bodegas = Bodega::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('nombre', 'asc')->get();        
        $cont=Bodega::where('idEmpresa','=', Auth::user()->idEmpresa)->count();

        $pdf = \PDF::loadView('pdf.bodegaspdf',['bodegas'=>$bodegas,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('bodegas.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodega = new Bodega();
        $bodega->nombre = $request->nombre;
        $bodega->direccion = $request->direccion;
        $bodega->tipo_bodega = $request->tipo;
        $bodega->fact_directa = $request->factDirecta;
        $bodega->idEmpresa = Auth::user()->idEmpresa;
        $bodega->estado = "1";
        $bodega->valor = $request->valor;
        $bodega->idTipoPrecio = $request->tipoPrecio;
        $bodega->idBodegaPadre = $request->idBodegaPadre;
        $bodega->valMin = $request->valMin;
        $bodega->valMax = $request->valMax;
        $bodega->formato_impresion = $request->formatoImp;
        $bodega->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodega = Bodega::findOrFail($request->id);
        $bodega->nombre = $request->nombre;
        $bodega->direccion = $request->direccion;
        $bodega->tipo_bodega = $request->tipo;
        $bodega->fact_directa = $request->factDirecta;
        $bodega->valor = $request->valor;
        $bodega->idTipoPrecio = $request->tipoPrecio;
        $bodega->idBodegaPadre = $request->idBodegaPadre;
        $bodega->valMin = $request->valMin;
        $bodega->valMax = $request->valMax;
        $bodega->formato_impresion = $request->formatoImp;
        $bodega->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodega = Bodega::findOrFail($request->id);
        $bodega->estado = 0;
        $bodega->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodega = Bodega::findOrFail($request->id);
        $bodega->estado = 1;
        $bodega->save();
    }

     public function selectBodega(Request $request){
        if (!$request->ajax()) return redirect('/');
        $bodegas = Bodega::select('id','nombre')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('id', 'asc')->get();
        return ['bodegas' => $bodegas];
    }

    //METODOS PARA MODELO BODEGAEMPLEADO
    public function indexBodegaEmpleado(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $bodegas = Bodega::orderBy('id', 'desc')
            ->where('idEmpresa','=', Auth::user()
            ->idEmpresa)
            ->with('bodegaEmpleado')
            ->with('bodegaEmpleado.bodegaEmpleadoPersonas')
            ->paginate(10);
        }
        else{
            $bodegas = Bodega::where($criterio, 'like', '%'. $buscar . '%')
            ->where('idEmpresa','=', Auth::user()->idEmpresa)
            ->orderBy('id', 'desc')
            ->with('bodegaEmpleado')
            ->with('bodegaEmpleado.bodegaEmpleadoPersonas')
            ->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $bodegas->total(),
                'current_page' => $bodegas->currentPage(),
                'per_page'     => $bodegas->perPage(),
                'last_page'    => $bodegas->lastPage(),
                'from'         => $bodegas->firstItem(),
                'to'           => $bodegas->lastItem(),
            ],
            'bodegas' => $bodegas
        ];
    }

    public function autoCompleteUsuarios(Request $request){
        if (!$request->ajax()) return redirect('/');
        $personas = $personas = User::join('personas','users.id','=','personas.id')
        // ->join('roles','users.idrol','=','roles.id')
        ->select('personas.id','personas.nombre','personas.tipo_documento','personas.num_documento')
        // ->where('users.usuario','<>', 'root')
        //  ->where('users.id','<>', $idUsuario)
         ->where('users.idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('personas.id', 'desc')
        // ->where(DB::raw("CONCAT(codigo,nombre)"), 'like', '%'. $request->parametro . '%')
        ->where('personas.nombre', 'like', '%'. $request->parametro . '%')
        ->get();
        return ['personas' => $personas];
    }

    public function storeBodegaUsuario(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodegaEmpleado = BodegaEmpleado::where("id_bodega","=",$request->idBodega)
        ->where("id_empleado","=",$request->idEmpleado)
        ->count();
        if ($bodegaEmpleado == 0){
            $bodega = new BodegaEmpleado();
            $bodega->id_bodega = $request->idBodega;
            $bodega->id_empleado = $request->idEmpleado;
            $bodega->empleado_Activo = '0';
            $bodega->encargado = 0;
            $bodega->supervisa_cajas = 0;
            $bodega->supervisa_remesa_bod = 0;
            $bodega->save();
        }else{
            return "exist";
        }
       
    }

    public function deleteUsuarioBodega(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodegaEmpleado = BodegaEmpleado::findOrFail($request->id);
        $bodegaEmpleado->delete();
    }


    public function storeBodegaDocumento(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        // $bodegaEmpleado = BodegaDocumento::where("id_bodega","=",$request->idBodega)
        // ->where("id_empleado","=",$request->idEmpleado)
        // ->count();
        // if ($bodegaEmpleado == 0){
            $bodega = new BodegaDocumento();
            $bodega->id_bodega = $request->idBodega;
            $bodega->tipo_documento = $request->tipoDocumento;
            $bodega->clase_documento = $request->claseDocumento;
            $bodega->num_serie = $request->numSerie;
            $bodega->num_resolucion = $request->numReso;
            $bodega->num_actual = $request->numActual;
            $bodega->save();
        // }else{
        //     return "exist";
        // }
       
    }

    public function indexBodegaDocumento(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $idBodega = $request->idBodega;
        
        if ($buscar==''){
            $bodegaDocs = BodegaDocumento::orderBy('id', 'desc')
            ->where('id_bodega','=', $idBodega)
            ->paginate(10);
        }
        // else{
        //     $bodegas = Bodega::where($criterio, 'like', '%'. $buscar . '%')
        //     ->where('id_bodega','=', $idBodega)
        //     ->orderBy('id', 'desc')
        //     ->with('bodegaEmpleado')
        //     ->with('bodegaEmpleado.bodegaEmpleadoPersonas')
        //     ->paginate(10);
        // }
        

        return [
            'pagination' => [
                'total'        => $bodegaDocs->total(),
                'current_page' => $bodegaDocs->currentPage(),
                'per_page'     => $bodegaDocs->perPage(),
                'last_page'    => $bodegaDocs->lastPage(),
                'from'         => $bodegaDocs->firstItem(),
                'to'           => $bodegaDocs->lastItem(),
            ],
            'bodegaDocs' => $bodegaDocs
        ];
    }

    public function updateBodegaDocumento(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodega = BodegaDocumento::findOrFail($request->id);
        $bodega->tipo_documento = $request->tipoDocumento;
        $bodega->clase_documento = $request->claseDocumento;
        $bodega->num_serie = $request->numSerie;
        $bodega->num_resolucion = $request->numReso;
        $bodega->num_actual = $request->numActual;
        $bodega->save();
    }

    public function deleteBodegaDocumento(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = BodegaDocumento::findOrFail($request->id);
        $call->delete();
    }

    public function getNumerosDocumentos(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $idBodega = $request->idBodega;
        $tipoDoc = $request->tipoDoc;
        $claseDoc = $request->claseDoc;
        
        $bodegaDoc = BodegaDocumento::where('id_bodega','=', $idBodega)
        ->where('tipo_documento','=', $tipoDoc)
        ->where('clase_documento','=', $claseDoc)
        ->first();
       
        if ($bodegaDoc){
            return [
                'bodegaDoc' => $bodegaDoc
            ];
        }else{
            return "NF";
        }
        
    }

    public function indexBodegaSala(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $idBodega = $request->idBodega;
        
        if ($buscar==''){
            $bodegaSalas = BodegaSala::orderBy('id', 'desc')
            ->where('id_bodega','=', $idBodega)->with('bodegaSala')
            ->paginate(10);
        }
       
        return [
            'pagination' => [
                'total'        => $bodegaSalas->total(),
                'current_page' => $bodegaSalas->currentPage(),
                'per_page'     => $bodegaSalas->perPage(),
                'last_page'    => $bodegaSalas->lastPage(),
                'from'         => $bodegaSalas->firstItem(),
                'to'           => $bodegaSalas->lastItem(),
            ],
            'bodegaSalas' => $bodegaSalas
        ];
    }

    public function storeBodegaSala(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $salaBodega = BodegaSala::where("id_sala","=",$request->idSala)
        ->where("id_bodega","=",$request->idBodega)
        ->count();
        if ($salaBodega == 0){
            $bodega = new BodegaSala();
            $bodega->id_bodega = $request->idBodega;
            $bodega->id_sala = $request->idSala;
            $bodega->save();   
        }else{
            return 'exist';
        }
           
    }

    public function updateBodegaSala(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodega = BodegaSala::findOrFail($request->id);
        $bodega->id_sala = $request->idSala;
        $bodega->save();
    }

    public function deleteBodegaSala(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = BodegaSala::findOrFail($request->id);
        $call->delete();
    }
     
}
