<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
//use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Categoria;

class CategoriaController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
         if ($rolUsuario != $_ENV['ROOT']){
            if ($buscar==''){
                $categorias = Categoria::orderBy('id', 'desc')->where('idEmpresa','=', $idEmpresa)->paginate(10);
            }
            else{
                $categorias = Categoria::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', $idEmpresa)->orderBy('id', 'desc')->paginate(10);
            }
         }else{
            if ($buscar==''){
                $categorias = Categoria::orderBy('id', 'desc')->paginate(10);
            }
            else{
                $categorias = Categoria::where($criterio, 'like', '%'. $buscar . '%')->orderBy('id', 'desc')->paginate(10);
            }
         }
        
        

        return [
            'pagination' => [
                'total'        => $categorias->total(),
                'current_page' => $categorias->currentPage(),
                'per_page'     => $categorias->perPage(),
                'last_page'    => $categorias->lastPage(),
                'from'         => $categorias->firstItem(),
                'to'           => $categorias->lastItem(),
            ],
            'categorias' => $categorias
        ];
    }

    public function selectCategoria(Request $request){
        if (!$request->ajax()) return redirect('/');
        $idEmpresa = Auth::user()->idEmpresa;
        $categorias = Categoria::where('condicion','=','1')->where('idEmpresa','=',$idEmpresa)
        ->select('id','nombre')->orderBy('nombre', 'asc')->get();
        return ['categorias' => $categorias];
    }

    public function listarPdf(){
        $idEmpresa = Auth::user()->idEmpresa;
        $categorias = Categoria::select('nombre','descripcion','condicion')->where('idEmpresa','=', $idEmpresa)->orderBy('nombre', 'asc')->get();
        $cont=Categoria::count();

        $pdf = \PDF::loadView('pdf.categoriaspdf',['categorias'=>$categorias,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('categorias.pdf');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        $categoria = new Categoria();
        $categoria->nombre = $request->nombre;
        $categoria->descripcion = $request->descripcion;
        $categoria->condicion = '1';
        if ($rolUsuario == $_ENV['ROOT']) {
            $categoria->idEmpresa = $request->empresa;
        }else{
             $categoria->idEmpresa = $idEmpresa;
        }
        $categoria->save();
    }
  

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $categoria = Categoria::findOrFail($request->id);
        $categoria->nombre = $request->nombre;
        $categoria->descripcion = $request->descripcion;
        $categoria->condicion = '1';
        $categoria->save();
    }

    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $categoria = Categoria::findOrFail($request->id);
        $categoria->condicion = '0';
        $categoria->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $categoria = Categoria::findOrFail($request->id);
        $categoria->condicion = '1';
        $categoria->save();
    }

    
}
