<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use App\Persona;
use App\Cliente;
use App\Departamento;
use App\Municipio;
use App\Amortizador;
use App\HistoricoCobro;
use App\Venta;
use App\OperacionBancaria;

use Illuminate\Support\Facades\Storage;


class ClienteController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $personas = Cliente::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $personas = Cliente::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $personas->total(),
                'current_page' => $personas->currentPage(),
                'per_page'     => $personas->perPage(),
                'last_page'    => $personas->lastPage(),
                'from'         => $personas->firstItem(),
                'to'           => $personas->lastItem(),
            ],
            'personas' => $personas
        ];
    }

    public function selectCliente(Request $request){
        if (!$request->ajax()) return redirect('/');

        $filtro = $request->filtro;
        $clientes = Persona::where('nombre', 'like', '%'. $filtro . '%')
        ->orWhere('num_documento', 'like', '%'. $filtro . '%')
        ->select('id','nombre','num_documento')
        ->orderBy('nombre', 'asc')->get();

        return ['clientes' => $clientes];
    }

    public function listarPdf(){
        $clientes = Cliente::orderBy('nombre', 'asc')->where('idEmpresa','=', Auth::user()->idEmpresa)->get();        
        $cont=Cliente::count();

        $pdf = \PDF::loadView('pdf.clientespdf',['clientes'=>$clientes,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('clientes.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
         // Validar si el NRC, NIT o DUI ya existen
         $exists = Cliente::where(function($query) use ($request) {
            if ($request->nrc != '0' && $request->nrc != 0) {
                $query->where('nrc', $request->nrc);
            }
            if ($request->nit != '0' && $request->nit != 0) {
                $query->orWhere('nit', $request->nit);
            }
            if ($request->dui != '0'  && $request->dui != 0) {
                $query->orWhere('dui', $request->dui);
            }
        })
        ->exists();
        $existTipoDocEx = isset($request->tipoDocExtranjero) && $request->tipoDocExtranjero !== '' ? $request->tipoDocExtranjero : null;
        $existDocEx = isset($request->numDocExtranjero) && $request->numDocExtranjero !== '' ? $request->numDocExtranjero : null;        
        if ($exists && ($existTipoDocEx == null && $existDocEx == null)) {
            return response()->json(['error' => 'El NRC, NIT o DUI ya están registrados para otro cliente.'], 400);
        }
        $cortarFecha = explode('/',$request->fechaNacimiento);
        $cliente = new Cliente();
        $cliente->idEmpresa = Auth::user()->idEmpresa;
        //$cliente->fechaingreso = $request->tipo_documento;
        $cliente->nombre = $request->nombre;
        $cliente->apellido = $request->apellido;
        $cliente->codCliente = $request->codigoCliente;
        $cliente->fechanacimiento = '2010-01-01';
        $cliente->direccion = $request->direccion;
        $cliente->telcasa = $request->telCasa;
        $cliente->teltrabajo = $request->telTrabajo;
        $cliente->cel = $request->celular;
        $cliente->dui = $request->dui;
        $cliente->nit = $request->nit;
        $cliente->nrc = $request->nrc;
        $cliente->tipocontribuyente = $request->contribuyente;
        $cliente->departamento = $request->idDepartamento;
        $cliente->municipio = $request->idMunicipio;
        $cliente->idClienteVendedor = $request->idVendedor;
        $cliente->idSector = $request->idSector;
        $cliente->idTipo = $request->idTipo;
        $cliente->idClienteGiro = $request->idGiro;
        $cliente->idTerminoCobro = $request->idTermino;
        $cliente->creditodisponible = $request->creditoDisponible;
        $cliente->limitecredito = $request->limiteCredito;
        $cliente->montodisponible = $request->limiteCredito;
        //$cliente->montodisponible = $request->montoDisponible;
        $cliente->creditoutilizado = $request->creditoUtilizado;
        $cliente->ctacobrar = $request->cuentaCobrar;
        $cliente->genero = $request->genero;
        $cliente->tipoReferencia = $request->tipoReferencia;
        $cliente->textoReferencia = $request->textoReferencia;
        $cliente->email = $request->email;
        $cliente->responsable = $request->responsable;
        $cliente->telContacto = $request->telContacto;
        $cliente->diasCredito = $request->diasCredito;
        $cliente->tipo_operacion = $request->tipoOperacion;
        $cliente->clasificacion = $request->clasificacion;
        $cliente->sector = $request->sector;
        $cliente->tipoDocExtranjero = $request->tipoDocExtranjero;
        $cliente->numDocExtranjero = $request->numDocExtranjero;
    
        //$cliente->ctacobrar2 = $request->email;
        $cliente->estado = 1;
        $cliente->flagimage = 0;
        if ($request->imgBase64 != "empty") {
            $cliente->flagimage = 1;
        } 
        if ($cliente->save() && $request->imgBase64 != "empty") {
            self::saveFoto($request->imgBase64,$cliente->id);
        }
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $cortarFecha = explode('/',$request->fechaNacimiento);
        $cliente = Cliente::findOrFail($request->id);
        $cliente->nombre = $request->nombre;
        $cliente->apellido = $request->apellido;
        $cliente->codCliente = $request->codigoCliente;
        $cliente->fechanacimiento = $cortarFecha[2]."-".$cortarFecha[1].'-'.$cortarFecha[0];
        $cliente->direccion = $request->direccion;
        $cliente->telcasa = $request->telCasa;
        $cliente->teltrabajo = $request->telTrabajo;
        $cliente->cel = $request->celular;
        $cliente->dui = $request->dui;
        $cliente->nit = $request->nit;
        $cliente->nrc = $request->nrc;
        $cliente->tipocontribuyente = $request->contribuyente;
        $cliente->departamento = $request->idDepartamento;
        $cliente->municipio = $request->idMunicipio;
        $cliente->idClienteVendedor = $request->idVendedor;
        $cliente->idSector = $request->idSector;
        $cliente->idTipo = $request->idTipo;
        $cliente->idClienteGiro = $request->idGiro;
        $cliente->idTerminoCobro = $request->idTermino;
        $cliente->creditodisponible = $request->creditoDisponible;
        $cliente->limitecredito = $request->limiteCredito;
        $cliente->montodisponible = $request->limiteCredito;
        $cliente->ctacobrar = $request->cuentaCobrar;
        $cliente->montodisponible = $request->montoDisponible;
        $cliente->creditoutilizado = $request->creditoUtilizado;
        $cliente->genero = $request->genero;
        $cliente->tipoReferencia = $request->tipoReferencia;
        $cliente->textoReferencia = $request->textoReferencia;
        $cliente->email = $request->email;
        $cliente->responsable = $request->responsable;
        $cliente->telContacto = $request->telContacto;
        $cliente->diasCredito = $request->diasCredito;
        $cliente->tipo_operacion = $request->tipoOperacion;
        $cliente->clasificacion = $request->clasificacion;
        $cliente->sector = $request->sector;
        $cliente->tipo_costo_gasto = $request->tipoCostoGasto;
        $cliente->tipoDocExtranjero = $request->tipoDocExtranjero;
        $cliente->numDocExtranjero = $request->numDocExtranjero;
        // if ($request->imgBase64 != "empty") {
        //     $cliente->flagimage = 1;
        // } 
        // if ($cliente->save() && $request->imgBase64 != "empty") {
        //     self::saveFoto($request->imgBase64,$cliente->id);
        // }
        // $cliente->save();
        if ($request->imgBase64 != "empty") {
            $cliente->flagimage = 1;
        } 
        if ($cliente->save() && $request->imgBase64 != "empty") {
            self::saveFoto($request->imgBase64,$cliente->id);
        }
    }

    public function selectDepartamento(Request $request){
        if (!$request->ajax()) return redirect('/');
        $departamentos = Departamento::select('id','depto_nombre','depto_codigodte')->orderBy('depto_nombre', 'asc')->get();
        return ['departamentos' => $departamentos];
    }

    public function selectMunicipio(Request $request){
        if (!$request->ajax()) return redirect('/');
        $idDepartamento = $request->idDepartamento;
        $municipios = Municipio::select('id','muni_nombre','muni_codigo')
        ->where('depto_id','=',$idDepartamento)
        ->orderBy('muni_nombre', 'asc')->get();
        return ['municipios' => $municipios];
    }

    public function autoCompleteCliente(Request $request){
        if (!$request->ajax()) return redirect('/');
        $clientes = Cliente::select('id','nombre','nrc','nit','tipocontribuyente')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
         ->where('estado','=', 1)
        ->where(DB::raw("CONCAT(nrc,nombre,nit)"), 'like', '%'. $request->parametro . '%')
        ->distinct('id')
        ->orderBy('nrc', 'asc')->get();
        return ['clientes' => $clientes];
    }

    public function saveFoto($base64Img,$idPaciente){
        $extension = explode('/', explode(':', substr($base64Img, 0, strpos($base64Img, ';')))[1])[1];   // .jpg .png .pdf

        $replace = substr($base64Img, 0, strpos($base64Img, ',')+1); 

        // find substring fro replace here eg: data:image/png;base64,

        $image = str_replace($replace, '', $base64Img); 
        $image = str_replace(' ', '+', $image); 
        $imageName = $idPaciente.'.'."jpg";
        Storage::disk('fotosperfil')->put($imageName, base64_decode($image));
    }

    public function getFotoBase64(Request $request){
        $file = "" ;
        $base64img = "";
        //return $request->idPaciente;
        if (Storage::disk('fotosperfil')->exists($request->idPaciente.'.jpg')) {
           $file =  Storage::disk('fotosperfil')->get($request->idPaciente.'.jpg');
          return  "data:image/jpeg;base64,".base64_encode($file);
        }else{
            return 'empty';
        }
    }

    public function filterClientsCallCenter(Request $request){
        if (!$request->ajax()) return redirect('/');
            $registros = Cliente::join('fc_factura','cliente.id','=','fc_factura.idCliente')
            ->join('in_bodega','fc_factura.idBodega','=','in_bodega.id')
           ->select(
                'cliente.id as idCliente',
                'cliente.apellido AS apellido',
                'cliente.nombre AS nombre',
                'cliente.codCliente AS Max_codCliente',
                'cliente.telcasa AS telcasa',
                'cliente.cel AS cel',
                'cliente.teltrabajo AS teltrabajo',
                'cliente.email AS email',
                'cliente.textoReferencia AS textoReferencia',
                'cliente.responsable',
                'cliente.telContacto',
                DB::raw("max(fc_factura.fecha_factura) AS Max_fecha_factura"),
                'in_bodega.nombre AS sucursal',
                DB::raw("Max((CASE
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) =
                0) THEN 'sincolor'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN 1 AND 30)
                THEN 'azul'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                31 AND 60) THEN 'verde'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                61 AND 90) THEN 'amarillo'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                91 AND 120) THEN 'naranja'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                121 AND 999999) THEN 'rojo' END)) AS utimavisita"),
                )
                ->where(function ($query) use ($request) {
                    $query->where('cliente.nombre', 'like', $request->letra . '%')
                        ->orWhere('cliente.apellido', 'like', $request->letra . '%');
                })
                ->where('fc_factura.anulada','=', 0)
                ->where('in_bodega.id','=', $request->bodega)
                ->where('cliente.idEmpresa','=', Auth::user()->idEmpresa)
                ->groupBy(['cliente.id', 'cliente.apellido','cliente.codCliente','cliente.telcasa','cliente.cel','cliente.teltrabajo',
                'cliente.email','cliente.textoReferencia','cliente.responsable','cliente.telContacto','in_bodega.nombre'])
                ->orderBy('cliente.'.$request->ordenar, 'asc')
                ->paginate(10);
      return [
        'pagination' => [
            'total'        => $registros->total(),
            'current_page' => $registros->currentPage(),
            'per_page'     => $registros->perPage(),
            'last_page'    => $registros->lastPage(),
            'from'         => $registros->firstItem(),
            'to'           => $registros->lastItem(),
        ],
        'registros' => $registros
    ];
    }


    public function getAbonoClientesListado(Request $request){
        if (!$request->ajax()) return redirect('/');
            $idCliente = $request->idCliente;
            $registros = Venta::join('fc_factura_detalle','fc_factura_detalle.idFactura','=','fc_factura.id')
            ->join('cliente','cliente.id','=','fc_factura.idCliente')
            ->join('personas','fc_factura.idVendedor','=','personas.id')
            ->join('users','personas.id','=','users.id')
            ->join('articulos','fc_factura_detalle.idProducto','=','articulos.id')
           ->select(
                'personas.id',
                'personas.nombre',
                'cliente.id AS id1',
                'fc_factura.tipo_documento',
                DB::raw("concat(cliente.codCliente, '-', cliente.nombre, '-', cliente.dui) AS cliente"),
                'fc_factura.id AS id2',
                'fc_factura.codigoGeneracion',
                'fc_factura.fecha_factura',
                'fc_factura.fecha_vencimiento',
                DB::raw("Group_Concat(articulos.nombre) AS nombre1"),
                'fc_factura.total_fc AS vfact',
                'fc_factura.saldo_documento',
                'fc_factura.tipoVenta',
                DB::raw("CASE
                WHEN To_Days(Now()) - To_Days(fc_factura.fecha_vencimiento) <
                0 THEN 'Sin Vencer'
                WHEN To_Days(Now()) - To_Days(fc_factura.fecha_vencimiento) BETWEEN 0 AND 30
                THEN '0 a 30D'
                WHEN To_Days(Now()) - To_Days(fc_factura.fecha_vencimiento) BETWEEN 31 AND
                60 THEN '31 a 60D'
                WHEN To_Days(Now()) - To_Days(fc_factura.fecha_vencimiento) BETWEEN 61 AND
                90 THEN '61 a 90D'
                WHEN To_Days(Now()) - To_Days(fc_factura.fecha_vencimiento) BETWEEN 91 AND
                120 THEN '91 a 120D'
                WHEN To_Days(Now()) - To_Days(fc_factura.fecha_vencimiento) BETWEEN 121 AND
                9999999 THEN 'Mas de 121D' END AS rangov"),
                DB::raw("0 AS abono"),
                DB::raw("0 AS active"),
           )
                ->where('fc_factura.saldo_documento','>', 0)
                ->whereIn('fc_factura.tipo_documento', ['01', '03'])
                ->where('fc_factura.condicion_venta','=', 2)
                // ->orderBy('cliente.'.$request->ordenar, 'asc')
                ->where('fc_factura.anulada','=', 0);

                if ($idCliente > 0) {
                    $registros = $registros->where('cliente.id','=', $idCliente);
                }
                $registros = $registros
                ->groupBy(['personas.id', 'personas.nombre','cliente.id','fc_factura.tipo_documento','fc_factura.id','fc_factura.codigoGeneracion',
                'fc_factura.fecha_factura','fc_factura.fecha_vencimiento','fc_factura.total_fc','fc_factura.saldo_documento','fc_factura.tipoVenta'])
                ->paginate(1000);
      return [
        'pagination' => [
            'total'        => $registros->total(),
            'current_page' => $registros->currentPage(),
            'per_page'     => $registros->perPage(),
            'last_page'    => $registros->lastPage(),
            'from'         => $registros->firstItem(),
            'to'           => $registros->lastItem(),
        ],
        'registros' => $registros
    ];
    }

    public function storeAbonosClientes(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        try{
            $abonos =  $request->abonos;
            $idCuentaBancaria =  $request->idCuentaBancaria;
            DB::beginTransaction();
            //REGISTRAMOS ENCABEZADO
            $encabezado = new OperacionBancaria();
            $encabezado->idCuenta = $idCuentaBancaria;
            $encabezado->id_cliente = $request->idCliente;
            $encabezado->tipo_ingreso = "DP";
            $encabezado->tipo_movimiento = 0;
            $encabezado->num_movimiento = $request->numMovimiento;
            $encabezado->fecha_movimiento = $request->fechaAbono;
            $encabezado->desc_movimiento = "Abono clientes";
            $encabezado->valor_credito = $request->totalAbono;
            $encabezado->usuario = Auth::user()->id;
            $encabezado->anulado = 0;
            $conceptoAbono = isset($request->conceptoAbono) && !empty(trim($request->conceptoAbono)) ? $request->conceptoAbono  : '-';
            $encabezado->conceptoAbono = $conceptoAbono;
            $encabezado->save();
            
            foreach($abonos as $ep=>$ab){ 
                if ($ab["abono"] > 0){
                    $abono = new HistoricoCobro();
                    $abono->fecha_abono = $request->fechaAbono;
                    $abono->fecha_procesado = $request->fechaAbono;
                    $abono->ID_cliente = $ab["id1"];
                    $abono->ID_factura = $ab["id2"];
                    $abono->ID_vendedor = $encabezado->id;
                    $abono->abono = $ab["abono"];
                    $abono->rutacobro = $ab["tipo_documento"];
                    $abono->num_reporte = $ab["id"];
                    $abono->observado = 1;
                    $abono->idCuentaBancaria = $idCuentaBancaria;
                    $abono->idAhoAporta = $encabezado->id;
                    $abono->save();

                    //ACTUALIZAMOS SALDO DE DOCUMENTO EN FACTURA
                    $venta = Venta::findOrFail($abono->ID_factura);
                    $saldoDocumento =  $venta->saldo_documento;
                    $venta->saldo_documento = $saldoDocumento - $abono->abono;
                    $venta->save();

                    //ACTUALIZAMOS MONTOS PARA CLIENTES
                    $cliente = Cliente::findOrFail($abono->ID_cliente);
                    $cliente->montodisponible =  $cliente->montodisponible + $abono->abono;
                    $cliente->creditoutilizado =  $cliente->creditoutilizado - $abono->abono;
                    $cliente->save();
                }  
            }
            DB::commit();
        }catch (Exception $e){
            DB::rollBack();
        }
    }

    public function getCliente(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $cliente = Cliente::findOrFail($request->idCliente);
        return [
            'cliente' => $cliente
        ];
    }

    public function getCorrPaciente(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        // $cliente = Cliente::where("codCliente", 'like',$request->tipo.'%')->where('idEmpresa','=', Auth::user()->idEmpresa)
        // ->orderBy('codCliente', 'desc')
        // ->first();
        $count = Cliente::select(
        'codCliente',
        DB::raw("CAST((SUBSTRING_INDEX(codCliente,'".$request->tipo."', -1)) AS UNSIGNED ) AS corr")
        )
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('corr', 'desc')
        ->first();
    
        $lastCorr = $count->corr; 
        $newCorr = $lastCorr + 1;
        return [
            'newCorr' => $request->tipo.$newCorr
        ];
    }
}
