<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\ClienteGiro;
use Illuminate\Support\Facades\DB;

class ClienteGiroController extends Controller
{
   public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $giros = ClienteGiro::orderBy('id', 'desc')->paginate(10);
        }
        else{
            $giros = ClienteGiro::where($criterio, 'like', '%'. $buscar . '%')->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $giros->total(),
                'current_page' => $giros->currentPage(),
                'per_page'     => $giros->perPage(),
                'last_page'    => $giros->lastPage(),
                'from'         => $giros->firstItem(),
                'to'           => $giros->lastItem(),
            ],
            'giros' => $giros
        ];
    }


    public function listarPdf(){
        $giros = ClienteGiro::orderBy('codigo', 'asc')->get();        
        $cont=ClienteGiro::count();

        $pdf = \PDF::loadView('pdf.girospdf',['giros'=>$giros,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('giros.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $giro = new ClienteGiro();
        $giro->codigo = $request->codigo;
        $giro->descripcion = $request->descripcion;
        $giro->estado = "1";
        $giro->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $giro = ClienteGiro::findOrFail($request->id);
        $giro->codigo = $request->codigo;
        $giro->descripcion = $request->descripcion;
        $giro->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $giro = ClienteGiro::findOrFail($request->id);
        $giro->estado = 0;
        $giro->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $giro = ClienteGiro::findOrFail($request->id);
        $giro->estado = 1;
        $giro->save();
    }

    public function selectGiro(Request $request){
        if (!$request->ajax()) return redirect('/');
        $giros = ClienteGiro::select('id','codigo','descripcion')->where('estado','=','1')->orderBy('codigo', 'asc')->get();
        return ['giros' => $giros];
    }

    public function selectGiroSearch(Request $request){
        if (!$request->ajax()) return redirect('/');
        $giros = ClienteGiro::select('id as value',DB::raw("CONCAT(codigo,' - ',descripcion) as text"))->where('estado','=','1')->orderBy('codigo', 'asc')->get();
        return ['giros' => $giros];
    }
    public function selectActivo(Request $request){
        if (!$request->ajax()) return redirect('/');
        $activos = ActivoFijo::select('id  as value','nombre as text')
        ->with('VehiculoMotorista2')
        ->with('VehiculoMotorista2.vehiculoMotoristaEmpleado2')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('id', 'asc')->get();
        return ['activos' => $activos];
    }
}
