<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\ClienteSector;

class ClienteSectorController extends Controller
{
    public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $sectores = ClienteSector::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{
            $sectores = ClienteSector::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $sectores->total(),
                'current_page' => $sectores->currentPage(),
                'per_page'     => $sectores->perPage(),
                'last_page'    => $sectores->lastPage(),
                'from'         => $sectores->firstItem(),
                'to'           => $sectores->lastItem(),
            ],
            'sectores' => $sectores
        ];
    }


    public function listarPdf(){
        $sectores = ClienteSector::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('codigo', 'asc')->get();        
        $cont=ClienteSector::count();

        $pdf = \PDF::loadView('pdf.sectorespdf',['sectores'=>$sectores,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('sectores.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sector = new ClienteSector();
        $sector->codigo = $request->codigo;
        $sector->descripcion = $request->descripcion;
        $sector->idEmpresa = Auth::user()->idEmpresa;
        $sector->estado = "1";
        $sector->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sector = ClienteSector::findOrFail($request->id);
        $sector->codigo = $request->codigo;
        $sector->descripcion = $request->descripcion;
        $sector->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sector = ClienteSector::findOrFail($request->id);
        $sector->estado = 0;
        $sector->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sector = ClienteSector::findOrFail($request->id);
        $sector->estado = 1;
        $sector->save();
    }

     public function selectSector(Request $request){
        if (!$request->ajax()) return redirect('/');
        $sectores = ClienteSector::select('id','codigo','descripcion')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('codigo', 'asc')->get();
        return ['sectores' => $sectores];
    }
}
