<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\ClienteTipo;

class ClienteTipoController extends Controller
{
       public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $tipos = ClienteTipo::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{
            $tipos = ClienteTipo::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $tipos->total(),
                'current_page' => $tipos->currentPage(),
                'per_page'     => $tipos->perPage(),
                'last_page'    => $tipos->lastPage(),
                'from'         => $tipos->firstItem(),
                'to'           => $tipos->lastItem(),
            ],
            'tipos' => $tipos
        ];
    }


    public function listarPdf(){
        $tipos = ClienteTipo::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'asc')->get();        
        $cont=ClienteTipo::count();

        $pdf = \PDF::loadView('pdf.tipospdf',['tipos'=>$tipos,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('tipos.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $tipo = new ClienteTipo();
        $tipo->tipo = $request->tipo;
        $tipo->descripcion = $request->descripcion;
        $tipo->idEmpresa = Auth::user()->idEmpresa;
        $tipo->estado = "1";
        $tipo->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $tipo = ClienteTipo::findOrFail($request->id);
        $tipo->tipo = $request->tipo;
        $tipo->descripcion = $request->descripcion;
        $tipo->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $tipo = ClienteTipo::findOrFail($request->id);
        $tipo->estado = 0;
        $tipo->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $tipo = ClienteTipo::findOrFail($request->id);
        $tipo->estado = 1;
        $tipo->save();
    }

     public function selectTipo(Request $request){
        if (!$request->ajax()) return redirect('/');
        $tipos = ClienteTipo::select('id','tipo','descripcion')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('tipo', 'asc')->get();
        return ['tipos' => $tipos];
    }
}
