<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\ClienteVendedor;

class ClienteVendedorController extends Controller
{
        public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $vendedores = ClienteVendedor::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{
            $vendedores = ClienteVendedor::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $vendedores->total(),
                'current_page' => $vendedores->currentPage(),
                'per_page'     => $vendedores->perPage(),
                'last_page'    => $vendedores->lastPage(),
                'from'         => $vendedores->firstItem(),
                'to'           => $vendedores->lastItem(),
            ],
            'vendedores' => $vendedores
        ];
    }


    public function listarPdf(){
        $vendedores = ClienteVendedor::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('codigo', 'asc')->get();        
        $cont=ClienteVendedor::count();

        $pdf = \PDF::loadView('pdf.vendedorespdf',['vendedores'=>$vendedores,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('vendedores.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $vendedor = new ClienteVendedor();
        $vendedor->codigo = $request->codigo;
        $vendedor->nombre = $request->nombre;
        $vendedor->idEmpresa = Auth::user()->idEmpresa;
        $vendedor->estado = "1";
        $vendedor->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $vendedor = ClienteVendedor::findOrFail($request->id);
        $vendedor->codigo = $request->codigo;
        $vendedor->nombre = $request->nombre;
        $vendedor->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $vendedor = ClienteVendedor::findOrFail($request->id);
        $vendedor->estado = 0;
        $vendedor->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $vendedor = ClienteVendedor::findOrFail($request->id);
        $vendedor->estado = 1;
        $vendedor->save();
    }

     public function selectVendedor(Request $request){
        if (!$request->ajax()) return redirect('/');
        $vendedores = ClienteVendedor::select('id','codigo','nombre')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('nombre', 'asc')->get();
        return ['vendedores' => $vendedores];
    }
}
