<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\CompraChica;
use App\BodegaEmpleado;

class CompraChicaController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $bodegaEmpleado = $this->getBodegaEmpleadoSelf();
        $bodegas = array();
        $hoy = date("Y-m-d")." 00:00:00";
        if ($criterio == 'date'){
            $criterio = 'fechadoc';
        }
    
        foreach($bodegaEmpleado as $bod){
            array_push($bodegas,"".$bod->id_bodega);
        }
        
        if ($buscar==''){
            $compras = CompraChica::where('idEmpresa','=', Auth::user()->idEmpresa)
            ->where('fechadoc','=',$hoy)
            ->whereIn('idBodega', $bodegas)
            ->with('tipoCompra')
            ->orderBy('id', 'desc')->with('bodega')->paginate(10);
        }
        else{
            $compras = CompraChica::where($criterio, 'like', '%'. $buscar . '%')
            // ->where('fechadoc','=',$hoy)
            ->whereIn('idBodega', $bodegas)
            ->with('tipoCompra')
            ->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->with('bodega')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $compras->total(),
                'current_page' => $compras->currentPage(),
                'per_page'     => $compras->perPage(),
                'last_page'    => $compras->lastPage(),
                'from'         => $compras->firstItem(),
                'to'           => $compras->lastItem(),
            ],
            'compras' => $compras
        ];
    }

   
    public function listarPdf(){
        $compras = CompraChica::orderBy('id', 'asc')->where('idEmpresa','=', Auth::user()->idEmpresa)->get();        
       $cont=CompraChica::where('idEmpresa','=', Auth::user()->idEmpresa)->count(); 

        $pdf = \PDF::loadView('pdf.comprasccpdf',['compras'=>$compras,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('CompraChica.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $fecha1 = explode('/',$request->fechaDocumento);
        $fecha1 = $fecha1[2].'-'.$fecha1[1].'-'.$fecha1[0];

        $fecha2 = explode('/',$request->fechaIva);
        $fecha2 = $fecha2[2].'-'.$fecha2[1].'-'.$fecha2[0];

        $fecha3 = explode('/',$request->fechaVencimiento);
        $fecha3 = $fecha3[2].'-'.$fecha3[1].'-'.$fecha3[0];


        $compra = new CompraChica();
        $compra->idEmpresa = Auth::user()->idEmpresa;
        $compra->tipocompra = $request->idGasto;
        $compra->idBodega = $request->idBodega;
        $compra->clasedoc = $request->claseDocumento;
        $compra->tipodoc = $request->tipoDocumento;
        $compra->fechadoc = $fecha1;
        $compra->fechalibroiva = $fecha2;
        $compra->numdoc = $request->numeroDocumento;
        $compra->numreso = $request->numeroResolucion;
        $compra->numseriedoc = $request->numeroSerie;
        $compra->numfunico = $request->numeroFunico;
        $compra->numcomprobante = $request->numeroComprobante;
        $compra->nrcnit = $request->nrcNit;
        $compra->proveedor = $request->proveedor;
        $compra->tipoproveedor = $request->tipoProveedor;
        $compra->detallecompra = $request->detalleCompra;
        $compra->ver_liva = $request->verLiva;
        $compra->dias_credito = $request->diasCredito;
        $compra->fecha_vencimiento = $fecha3;
        $compra->observaciones_pago = $request->observaciones;
        $compra->numero_quedan = $request->numeroQuedan;
        $compra->subtotal = $request->subTotal;
        $compra->iva = $request->iva;
        $compra->ret = $request->ret;
        $compra->ret_renta_10 = $request->retRenta;
        $compra->fovial = $request->fovial;
        $compra->contrans = $request->contrans;
        $compra->cesc = $request->cesc;
        $compra->otros = $request->otros;
        $compra->total = $request->total;
        $compra->saldo_documento = $request->saldoDocumento;
        $compra->deducrenta = $request->deducible;
        $compra->estado = 1;
        $compra->tipo_operacion = $request->tipoOperacion;
        $compra->clasificacion = $request->clasificacion;
        $compra->sector = $request->sector;
        $compra->tipo_costo_gasto = $request->tipoCostoGasto;
        $compra->idCuenta = $request->idCuentaPadre;
        $compra->save(); 
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
         $fecha1 = explode('/',$request->fechaDocumento);
        $fecha1 = $fecha1[2].'-'.$fecha1[1].'-'.$fecha1[0];

        $fecha2 = explode('/',$request->fechaIva);
        $fecha2 = $fecha2[2].'-'.$fecha2[1].'-'.$fecha2[0];

        $fecha3 = explode('/',$request->fechaVencimiento);
        $fecha3 = $fecha3[2].'-'.$fecha3[1].'-'.$fecha3[0];
        $compra = CompraChica::findOrFail($request->id);
        $compra->tipocompra = $request->idGasto;
        $compra->clasedoc = $request->claseDocumento;
        $compra->tipodoc = $request->tipoDocumento;
        $compra->fechadoc = $fecha1;
        $compra->fechalibroiva = $fecha2;
        $compra->numdoc = $request->numeroDocumento;
        $compra->numreso = $request->numeroResolucion;
        $compra->numseriedoc = $request->numeroSerie;
        $compra->numfunico = $request->numeroFunico;
        $compra->numcomprobante = $request->numeroComprobante;
        $compra->nrcnit = $request->nrcNit;
        $compra->proveedor = $request->proveedor;
        $compra->tipoproveedor = $request->tipoProveedor;
        $compra->detallecompra = $request->detalleCompra;
        $compra->ver_liva = $request->verLiva;
        $compra->dias_credito = $request->diasCredito;
        $compra->fecha_vencimiento = $fecha3;
        $compra->observaciones_pago = $request->observaciones;
        $compra->numero_quedan = $request->numeroQuedan;
        $compra->subtotal = $request->subTotal;
        $compra->iva = $request->iva;
        $compra->ret = $request->ret;
        $compra->ret_renta_10 = $request->retRenta;
        $compra->fovial = $request->fovial;
        $compra->contrans = $request->contrans;
        $compra->cesc = $request->cesc;
        $compra->otros = $request->otros;
        $compra->total = $request->total;
        $compra->saldo_documento = $request->saldoDocumento;
        $compra->deducrenta = $request->deducible;
        $compra->tipo_operacion = $request->tipoOperacion;
        $compra->clasificacion = $request->clasificacion;
        $compra->sector = $request->sector;
        $compra->tipo_costo_gasto = $request->tipoCostoGasto;
        $compra->save();
    }

    public function selectNumResolucion(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = CompraChica::select('numreso')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numreso', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numreso', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function selectNumSerie(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = CompraChica::select('numseriedoc')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numseriedoc', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numseriedoc', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function getBodegaEmpleadoSelf()
    {
        $bodega = BodegaEmpleado::where("id_empleado", "=", Auth::user()->id)->get();
        return $bodega;
    }
   
}
