<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\CompraChica;
use App\BodegaEmpleado;
use App\Clasificacion;
use App\CompraJSON;
use App\Sector;
use App\TipoCostoGasto;
use App\TipoOperacion;

class CompraJSONController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;

        if ($buscar==''){
            $compras = CompraJSON::orderBy('id', 'desc')
            ->with('tipoOperacionCompra')
            ->with('clasificacionCompra')
            ->with('sectorCompra')
            ->with('tipoCostoGastoCompra')
            ->paginate(10);
        }
        else{
            $compras = CompraJSON::where($criterio, 'like', '%'. $buscar . '%')->orderBy('id', 'desc')
            ->with('tipoOperacionCompra')
            ->with('clasificacionCompra')
            ->with('sectorCompra')
            ->with('tipoCostoGastoCompra')
            ->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $compras->total(),
                'current_page' => $compras->currentPage(),
                'per_page'     => $compras->perPage(),
                'last_page'    => $compras->lastPage(),
                'from'         => $compras->firstItem(),
                'to'           => $compras->lastItem(),
            ],
            'compras' => $compras
        ];
    }

   
    public function listarPdf(){
        $compras = CompraChica::orderBy('id', 'asc')->where('idEmpresa','=', Auth::user()->idEmpresa)->get();        
       $cont=CompraChica::where('idEmpresa','=', Auth::user()->idEmpresa)->count(); 

        $pdf = \PDF::loadView('pdf.comprasccpdf',['compras'=>$compras,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('CompraChica.pdf');
    }

    public function store(Request $request)
    {
        $jsonData = json_decode($request->jsonData);
        $tipoDte = $jsonData->identificacion->tipoDte;
        $errorCode = "";
        $message = "";
        $compraJson = CompraJSON::where('codigoGeneracion', $jsonData->identificacion->codigoGeneracion)
        ->count();
        if ($compraJson > 0) {
            $errorCode = 2;
            $message = "El documento que desea guardar ya se encuentra registrado";
        }else{
            if ($tipoDte == "03" || $tipoDte == "05"){
                $compra = new CompraJSON();
                $compra->tipo_operacion = $request->tipoOperacion;
                $compra->clasificacion = $request->clasificacion;
                $compra->sector = $request->sector;
                $compra->tipo_costo_gasto = $request->tipoCostoGasto;
                $compra->fecha_libro_iva = $request->fechaLiva;
                $compra->idCuenta = $request->idCuentaPadre;
                $compra->tipoCompra = $request->tipoCompra;
                $compra->emi_nit = $jsonData->emisor->nit;
                $compra->emi_nrc = $jsonData->emisor->nrc;
                $compra->emi_nombre = $jsonData->emisor->nombre;
                foreach ($jsonData->resumen->tributos as $tributo) {
                    if ($tributo->codigo === 'C8') {
                        $compra->resu_tribu_c8 = $tributo->valor;
                    }
        
                    if ($tributo->codigo === 'D1') {
                        $compra->resu_tribu_d1 = $tributo->valor;
                    }
        
                    if ($tributo->codigo === '20') {
                        $compra->resu_tribu_20 = $tributo->valor;
                    }
                }
                $compra->resu_subtotal = $jsonData->resumen->subTotal;
                $compra->resu_ivaperci1 = $jsonData->resumen->ivaPerci1;
                $compra->resu_ivarete1 = $jsonData->resumen->ivaRete1;
                $compra->resu_reterenta = $jsonData->resumen->reteRenta;
                $compra->resu_totalpagar = $jsonData->resumen->totalPagar;
                $compra->codigoGeneracion = $jsonData->identificacion->codigoGeneracion;
                $compra->fechaProcesamiento = $jsonData->identificacion->fecEmi;
                $compra->idEmpresa = Auth::user()->idEmpresa;
                $compra->tipoDte = $jsonData->identificacion->tipoDte;
                $compra->save();   
                $errorCode = 0;
                $message = "Documento guardado exitósamente";    
            }else{
                //Documento inválido debe ser crédito fiscal o nota de crédito
                $errorCode = 1;
                $message = "Documento inválido, Solo se permiten documentos de crédito fiscal y notas de crédito";     
            } 
        } 

        return response()->json(
            [
                'message' => $message,
                'errorCode' => $errorCode
            ]);               
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
         $fecha1 = explode('/',$request->fechaDocumento);
        $fecha1 = $fecha1[2].'-'.$fecha1[1].'-'.$fecha1[0];

        $fecha2 = explode('/',$request->fechaIva);
        $fecha2 = $fecha2[2].'-'.$fecha2[1].'-'.$fecha2[0];

        $fecha3 = explode('/',$request->fechaVencimiento);
        $fecha3 = $fecha3[2].'-'.$fecha3[1].'-'.$fecha3[0];
        $compra = CompraChica::findOrFail($request->id);
        $compra->tipocompra = $request->idGasto;
        $compra->clasedoc = $request->claseDocumento;
        $compra->tipodoc = $request->tipoDocumento;
        $compra->fechadoc = $fecha1;
        $compra->fechalibroiva = $fecha2;
        $compra->numdoc = $request->numeroDocumento;
        $compra->numreso = $request->numeroResolucion;
        $compra->numseriedoc = $request->numeroSerie;
        $compra->numfunico = $request->numeroFunico;
        $compra->numcomprobante = $request->numeroComprobante;
        $compra->nrcnit = $request->nrcNit;
        $compra->proveedor = $request->proveedor;
        $compra->tipoproveedor = $request->tipoProveedor;
        $compra->detallecompra = $request->detalleCompra;
        $compra->ver_liva = $request->verLiva;
        $compra->dias_credito = $request->diasCredito;
        $compra->fecha_vencimiento = $fecha3;
        $compra->observaciones_pago = $request->observaciones;
        $compra->numero_quedan = $request->numeroQuedan;
        $compra->subtotal = $request->subTotal;
        $compra->iva = $request->iva;
        $compra->ret = $request->ret;
        $compra->ret_renta_10 = $request->retRenta;
        $compra->fovial = $request->fovial;
        $compra->contrans = $request->contrans;
        $compra->cesc = $request->cesc;
        $compra->otros = $request->otros;
        $compra->total = $request->total;
        $compra->saldo_documento = $request->saldoDocumento;
        $compra->deducrenta = $request->deducible;
        $compra->save();
    }

    public function delete(Request $request){
        if (!$request->ajax()) return redirect('/');
        $compra = CompraJSON::findOrFail($request->id);
        $compra->delete();
    }

    public function selectNumResolucion(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = CompraChica::select('numreso')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numreso', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numreso', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function selectNumSerie(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = CompraChica::select('numseriedoc')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numseriedoc', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numseriedoc', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function selectTipoOperacion(Request $request){
        if (!$request->ajax()) return redirect('/');
        $tipos = TipoOperacion::select('id','codigo','nombre')->orderBy('codigo', 'asc')->get();
        return ['tipos' => $tipos];
    }

    public function selectClasificacion(Request $request){
        if (!$request->ajax()) return redirect('/');
        $clasificaciones = Clasificacion::select('id','codigo','nombre')->orderBy('codigo', 'asc')->get();
        return ['clasificaciones' => $clasificaciones];
    }

    public function selectSector(Request $request){
        if (!$request->ajax()) return redirect('/');
        $sectores = Sector::select('id','codigo','nombre')->orderBy('codigo', 'asc')->get();
        return ['sectores' => $sectores];
    }

    public function selectTipoCostoGasto(Request $request){
        if (!$request->ajax()) return redirect('/');
        $tipoCostosGastos = TipoCostoGasto::select('id','codigo','nombre')->orderBy('codigo', 'asc')->get();
        return ['tipoCostosGastos' => $tipoCostosGastos];
    }

    public function getBodegaEmpleadoSelf()
    {
        $bodega = BodegaEmpleado::where("id_empleado", "=", Auth::user()->id)->get();
        return $bodega;
    }
   
}
