<?php

namespace App\Http\Controllers;
use App\Documento;
use Illuminate\Support\Facades\Storage;

use Illuminate\Http\Request;

class DocumentoController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        if ($buscar==''){
            $documentos = Documento::where('idPaciente','=', $request->idPaciente)
            ->orderBy('created_at', 'ASC')->paginate(10);
        }
        else{
            $documentos = Documento::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=', $request->idPaciente)->orderBy('created_at', 'ASC')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $documentos->total(),
                'current_page' => $documentos->currentPage(),
                'per_page'     => $documentos->perPage(),
                'last_page'    => $documentos->lastPage(),
                'from'         => $documentos->firstItem(),
                'to'           => $documentos->lastItem(),
            ],
            'documentos' => $documentos
        ];
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $documento = new Documento();
        $docBase64 = $request->docBase64;
        // $docName = "D".$request->idPaciente."_".date("YmdHis").$request->docName;
        $extension = explode('/', explode(':', substr($docBase64, 0, strpos($docBase64, ';')))[1])[1];   // .jpg .png .pdf
            if ($extension == "vnd.openxmlformats-officedocument.wordprocessingml.document"){
                $extension = "docx";
            }else if ($extension == "msword"){
                $extension = "doc";
            }
        $docName = "D".$request->idPaciente."_".date("YmdHis").".".$extension;
        $documento->idPaciente = $request->idPaciente;
        $documento->comentario = $request->comentario;
        $documento->docName =$docName;
        if ($documento->save()) {
            self::saveDoc($docBase64,$docName);
        }
    }
    public function saveDoc($docBase64,$docName){
        $replace = substr($docBase64, 0, strpos($docBase64, ',')+1); 

        // find substring fro replace here eg: data:image/png;base64,
        $documento = str_replace($replace, '', $docBase64); 
        $documento = str_replace(' ', '+', $documento); 
        Storage::disk('documentos')->put($docName, base64_decode($documento));

    }
    public function downloadDocumento(Request $request){

        if (Storage::disk('documentos')->exists($request->docName)) {
           return Storage::disk('documentos')->download($request->docName);
        }else{
            return redirect()->route('main');
        }
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $docBase64 = $request->docBase64;
        $extension = explode('/', explode(':', substr($docBase64, 0, strpos($docBase64, ';')))[1])[1];   // .jpg .png .pdf
        if ($extension == "vnd.openxmlformats-officedocument.wordprocessingml.document"){
            $extension = "docx";
        }else if ($extension == "msword"){
            $extension = "doc";
        }
       
        $documento = Documento::findOrFail($request->id);
        $nombreViejo = $documento->docName;
        $docName = "D".$documento->idPaciente."_".date("YmdHis").".".$extension;
        $documento->comentario = $request->comentario;
        $documento->docName = $docName;
        if ($documento->save()) {
            $file =  Storage::disk('documentos')->delete($nombreViejo); 
            self::saveDoc($request->docBase64,$documento->docName);
        }
    }
    public function destroy(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $documento = Documento::findOrFail($request->id);
       $nombre=  $documento->docName;
        if ($documento->delete()){
            if (Storage::disk('documentos')->exists($nombre)) {
                $file =  Storage::disk('documentos')->delete($nombre); 
             }
        }
    }
}
