<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use App\Persona;
use App\Cliente;
use App\Empleado;
use App\Departamento;
use App\Municipio;
use App\Amortizador;
use Illuminate\Support\Facades\Storage;
use App\RegistroOdontologico;
use App\CallCenterModel;
use App\SalaPaciente;
use App\Ortodoncia;
use App\OrdenTrabajo;
use App\Odontograma;
use App\DepartamentoEmpleado;
use App\PuestoEmpleado;
use App\TipoIngresoEmpleado;
use App\Deduccion;
use App\SuspensionEmpleado;
use App\SuspensionEmpleadoDet;
class EmpleadoController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $empleados = Empleado::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $empleados = Empleado::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $empleados->total(),
                'current_page' => $empleados->currentPage(),
                'per_page'     => $empleados->perPage(),
                'last_page'    => $empleados->lastPage(),
                'from'         => $empleados->firstItem(),
                'to'           => $empleados->lastItem(),
            ],
            'empleados' => $empleados
        ];
    }

    public function autoCompleteEmpleado(Request $request){
        if (!$request->ajax()) return redirect('/');

        $filtro = $request->filtro;
        $empleados = Empleado::select('id','dui','nombre','apellido')
        ->where('estado','=','1')
        ->where('nombre', 'like', '%'. $filtro . '%')
        ->orderBy('nombre', 'asc')->get();
        return ['empleados' => $empleados];
    }

    public function selectDepartamento(Request $request){
        if (!$request->ajax()) return redirect('/');
        $departamentos = DepartamentoEmpleado::select('id','codigo','nombre')->where('estado','=','1')->orderBy('codigo', 'asc')->get();
        return ['departamentos' => $departamentos];
    }

    public function selectPuesto(Request $request){
        if (!$request->ajax()) return redirect('/');
        $puestos = PuestoEmpleado::select('id','codigo','nombre')->where('estado','=','1')->orderBy('codigo', 'asc')->get();
        return ['puestos' => $puestos];
    }

    public function selectEmpleado(Request $request){ //SOLO MOTORISTAS, IDPUESTO 1000
        if (!$request->ajax()) return redirect('/');
        $empleados = Empleado::select('id as value',DB::raw("CONCAT('[',dui,'] ',nombre,' ',apellido) as text"))
        ->where('estado','=','1')
        ->where('idPuesto','=','1000')
        ->orderBy('nombre', 'asc')->get();
        return ['empleados' => $empleados];
    }

    public function listarPdf(){
        $clientes = Cliente::orderBy('nombre', 'asc')->where('idEmpresa','=', Auth::user()->idEmpresa)->get();        
        $cont=Cliente::count();

        $pdf = \PDF::loadView('pdf.clientespdf',['clientes'=>$clientes,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('clientes.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $cortarFecha = explode('/',$request->fechaNacimiento);
        $empleado = new Empleado();
        $empleado->idEmpresa = Auth::user()->idEmpresa;
        $empleado->nombre = $request->nombre;
        $empleado->apellido = $request->apellido;
        $empleado->codEmpleado = $request->codigoEmpleado;
        $empleado->fechanacimiento = $cortarFecha[2]."-".$cortarFecha[1].'-'.$cortarFecha[0];
        $empleado->direccion = $request->direccion;
        $empleado->telcasa = $request->telCasa;
        $empleado->teltrabajo = $request->telTrabajo;
        $empleado->cel = $request->celular;
        $empleado->dui = $request->dui;
        $empleado->nit = $request->nit;
        $empleado->nrc = $request->nrc;
        $empleado->tipocontribuyente = $request->contribuyente;
        $empleado->departamento = $request->idDepartamento;
        $empleado->municipio = $request->idMunicipio;
        $empleado->idClienteVendedor = 0;
        $empleado->idSector = $request->idSector;
        $empleado->idTipo = $request->idTipo;
        $empleado->idClienteGiro = $request->idGiro;
        $empleado->idTerminoCobro = $request->idTermino;
        $empleado->creditodisponible = $request->creditoDisponible;
        $empleado->limitecredito = $request->limiteCredito;
        $empleado->montodisponible = $request->limiteCredito;
        $empleado->creditoutilizado = $request->creditoUtilizado;
        $empleado->ctacobrar = $request->cuentaCobrar;
        $empleado->genero = $request->genero;
        $empleado->tipoReferencia = $request->tipoReferencia;
        $empleado->textoReferencia = $request->textoReferencia;
        $empleado->email = $request->email;
        $empleado->responsable = $request->responsable;
        $empleado->telContacto = $request->telContacto;
        $empleado->idDpto = $request->idDepartamentoEmpleado;
        $empleado->idPuesto = $request->idPuesto;
        $empleado->tipoContrato = $request->tipoContrato;
        $empleado->fechaIngresoE = $request->fechaIngreso;
        $empleado->fechaRetiro = $request->fechaRetiro;
        $empleado->estadoCivil = $request->estadoCivil;
        $empleado->banco = $request->banco;
        $empleado->estado = 1;
        $empleado->flagimage = 0;
        if ($request->imgBase64 != "empty") {
            $empleado->flagimage = 1;
        } 
        if ($empleado->save() && $request->imgBase64 != "empty") {
            self::saveFoto($request->imgBase64,$empleado->id);
        }
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $cortarFecha = explode('/',$request->fechaNacimiento);
        $empleado = Empleado::findOrFail($request->id);
        $empleado->nombre = $request->nombre;
        $empleado->apellido = $request->apellido;
        $empleado->codEmpleado = $request->codigoEmpleado;
        $empleado->fechanacimiento = $cortarFecha[2]."-".$cortarFecha[1].'-'.$cortarFecha[0];
        $empleado->direccion = $request->direccion;
        $empleado->telcasa = $request->telCasa;
        $empleado->teltrabajo = $request->telTrabajo;
        $empleado->cel = $request->celular;
        $empleado->dui = $request->dui;
        $empleado->nit = $request->nit;
        $empleado->nrc = $request->nrc;
        $empleado->tipocontribuyente = $request->contribuyente;
        $empleado->departamento = $request->idDepartamento;
        $empleado->municipio = $request->idMunicipio;
        $empleado->idClienteVendedor = 0;
        $empleado->idSector = $request->idSector;
        $empleado->idTipo = $request->idTipo;
        $empleado->idClienteGiro = $request->idGiro;
        $empleado->idTerminoCobro = $request->idTermino;
        $empleado->creditodisponible = $request->creditoDisponible;
        $empleado->limitecredito = $request->limiteCredito;
        $empleado->montodisponible = $request->limiteCredito;
        $empleado->ctacobrar = $request->cuentaCobrar;
        $empleado->creditoutilizado = $request->creditoUtilizado;
        $empleado->genero = $request->genero;
        $empleado->tipoReferencia = $request->tipoReferencia;
        $empleado->textoReferencia = $request->textoReferencia;
        $empleado->email = $request->email;
        $empleado->responsable = $request->responsable;
        $empleado->telContacto = $request->telContacto;
        $empleado->idDpto = $request->idDepartamentoEmpleado;
        $empleado->fechaIngresoE = $request->fechaIngreso;
        $empleado->fechaRetiro = $request->fechaRetiro;
        $empleado->estadoCivil = $request->estadoCivil;
        $empleado->banco = $request->banco;
        $empleado->idPuesto = $request->idPuesto;
        $empleado->tipoContrato = $request->tipoContrato;
        if ($request->imgBase64 != "empty") {
            $empleado->flagimage = 1;
        } 
        if ($empleado->save() && $request->imgBase64 != "empty") {
            self::saveFoto($request->imgBase64,$empleado->id);
        }
    }

    // public function autoCompleteEmpleado(Request $request){
    //     if (!$request->ajax()) return redirect('/');
    //     $empleados = Empleado::select('id','nombre','nrc','nit','tipocontribuyente')
    //     ->where('idEmpresa','=', Auth::user()->idEmpresa)
    //      ->where('estado','=', 1)
    //     ->where(DB::raw("CONCAT(nrc,nombre,nit)"), 'like', '%'. $request->parametro . '%')
    //     ->distinct('id')
    //     ->orderBy('nrc', 'asc')->get();
    //     return ['empleados' => $empleados];
    // }

    public function saveFoto($base64Img,$idPaciente){
        $extension = explode('/', explode(':', substr($base64Img, 0, strpos($base64Img, ';')))[1])[1];   // .jpg .png .pdf

        $replace = substr($base64Img, 0, strpos($base64Img, ',')+1); 

        // find substring fro replace here eg: data:image/png;base64,

        $image = str_replace($replace, '', $base64Img); 
        $image = str_replace(' ', '+', $image); 
        $imageName = $idPaciente.'.'."jpg";
        Storage::disk('fotosEmpleados')->put($imageName, base64_decode($image));
    }

    public function getFotoBase64(Request $request){
        $file = "" ;
        $base64img = "";
        //return $request->idPaciente;
        if (Storage::disk('fotosEmpleados')->exists($request->idPaciente.'.jpg')) {
           $file =  Storage::disk('fotosEmpleados')->get($request->idPaciente.'.jpg');
          return  "data:image/jpeg;base64,".base64_encode($file);
        }else{
            return 'empty';
        }
    }

    public function filterClientsCallCenter(Request $request){
        if (!$request->ajax()) return redirect('/');
            $letra = $request->letra;
            $diasQuery = "Max(CASE
            WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) =
            0) THEN 'sincolor'
            WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN 1 AND 30)
            THEN 'azul'
            WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
            31 AND 60) THEN 'verde'
            WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
            61 AND 90) THEN 'amarillo'
            WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
            91 AND 120) THEN 'naranja'
            WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
            121 AND 999999) THEN 'rojo' END)";

            $registros = Cliente::join('fc_factura','cliente.id','=','fc_factura.idCliente')
            ->join('in_bodega','fc_factura.idBodega','=','in_bodega.id')
           ->select(
                'cliente.id as idCliente',
                'cliente.apellido AS apellido',
                'cliente.nombre AS nombre',
                'cliente.codEmpleado AS Max_codEmpleado',
                'cliente.telcasa AS telcasa',
                'cliente.cel AS cel',
                'cliente.teltrabajo AS teltrabajo',
                'cliente.email AS email',
                'cliente.textoReferencia AS textoReferencia',
                'cliente.responsable',
                'cliente.telContacto',
                'cliente.estado',
                DB::raw("max(fc_factura.fecha_factura) AS Max_fecha_factura"),
                // DB::raw("(select count(*) from ostc_callcenter oc WHERE oc.idPaciente  = cliente.id AND  oc.no_contesto  = 1) AS countNoContesto"),
                // select oc.no_contesto  from ostc_callcenter oc  WHERE oc.idPaciente  = 17  order By created_at desc limit 1  
                DB::raw("(select oc.no_contesto  from ostc_callcenter oc  WHERE oc.idPaciente  = cliente.id AND DATEDIFF(CURDATE(),oc.created_at) < 7  order By created_at desc limit 1  ) AS countNoContesto"),
                'in_bodega.nombre AS sucursal',
                DB::raw("Max((CASE
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) =
                0) THEN 'sincolor'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN 1 AND 30)
                THEN 'azul'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                31 AND 60) THEN 'verde'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                61 AND 90) THEN 'amarillo'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                91 AND 120) THEN 'naranja'
                WHEN ((To_Days(Now()) - To_Days(fc_factura.fecha_factura)) BETWEEN
                121 AND 999999) THEN 'rojo' END)) AS utimavisita"),
           );
        if ($letra != 'all'){
            if ($request->ordenar == "apellido"){
                $registros = $registros->where('cliente.apellido', 'like', $request->letra . '%');
            }else{
                $registros = $registros->where('cliente.nombre', 'like', $request->letra . '%');
            }
            // $registros = $registros->where(function ($query) use ($request) {
            //     $query->where('cliente.nombre', 'like', $request->letra . '%')
            //         ->orWhere('cliente.apellido', 'like', $request->letra . '%');
            // });
           }
               
           $registros = $registros->where('fc_factura.anulada','=', 0)
                ->where('in_bodega.id','=', $request->bodega)
                ->where('cliente.idEmpresa','=', Auth::user()->idEmpresa)
                ->where('cliente.codEmpleado', 'like', 'OR%')
                ->where('cliente.estado', '=',1);
                
                if ($request->fecha != ''){
                    $registros = $registros->having(DB::raw("max(fc_factura.fecha_factura)"), '=', $request->fecha);
                }
                
                $registros = $registros->groupBy(['cliente.id', 'cliente.apellido','cliente.codEmpleado','cliente.telcasa','cliente.cel','cliente.teltrabajo',
                'cliente.email','cliente.textoReferencia','cliente.responsable','cliente.telContacto','in_bodega.nombre'])
                ->orderBy('cliente.'.$request->ordenar, 'asc');
                
                $color = $request->colorFilter;

                if ($color == "all"){
                    $registros = $registros->paginate(1000);
                }else{
                    $registros = $registros ->having( DB::raw($diasQuery),'=', $color)->paginate(1000);
                }
                
                // $registros = $registros ->paginate(10);
      return [
        'pagination' => [
            'total'        => $registros->total(),
            'current_page' => $registros->currentPage(),
            'per_page'     => $registros->perPage(),
            'last_page'    => $registros->lastPage(),
            'from'         => $registros->firstItem(),
            'to'           => $registros->lastItem(),
        ],
        'registros' => $registros
    ];
    }
    public function getEmpleado(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $empleado = Empleado::findOrFail($request->idCliente);
        return [
            'empleado' => $empleado
        ];
    }

    public function getCorrEmpleado(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $count = Empleado::select(
        'codEmpleado',
        DB::raw("CAST((SUBSTRING_INDEX(codEmpleado,'".$request->tipo."', -1)) AS UNSIGNED ) AS corr")
        )
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('corr', 'desc')
        ->first();
    
        $lastCorr = $count->corr; 
        $newCorr = $lastCorr + 1;
        return [
            'newCorr' => $request->tipo.$newCorr
        ];
    }

    // ===========================================FICHA DE EMPLEADO ===========================================================================================================
    public function indexCallCenter(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $calls = CallCenterModel::where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $calls = CallCenterModel::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $calls->total(),
                'current_page' => $calls->currentPage(),
                'per_page'     => $calls->perPage(),
                'last_page'    => $calls->lastPage(),
                'from'         => $calls->firstItem(),
                'to'           => $calls->lastItem(),
            ],
            'calls' => $calls
        ];
    }

    public function storeCallCenter(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = new CallCenterModel();
        $call->idPaciente = $request->idPaciente;
        $call->idUser = Auth::user()->id;
        $call->comentario = $request->comentario;
        $call->no_contesto = $request->noContesto;
        $call->save();
    }

    public function updateCallCenter(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = CallCenterModel::findOrFail($request->id);
        $call->comentario = $request->comentario;
        $call->no_contesto = $request->noContesto;
        $call->save();
    }

    public function eliminarCallCenter(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = CallCenterModel::findOrFail($request->id);
        $call->delete();
    }

    public function indexRegistroOdonto(Request $request)
{
   if (!$request->ajax()) return redirect('/');

    $buscar = $request->buscar;
    $criterio = $request->criterio;
    
    if ($buscar==''){
        $registros = RegistroOdontologico::where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
    }
    else{
        $registros = RegistroOdontologico::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
    }
    
    return [
        'pagination' => [
            'total'        => $registros->total(),
            'current_page' => $registros->currentPage(),
            'per_page'     => $registros->perPage(),
            'last_page'    => $registros->lastPage(),
            'from'         => $registros->firstItem(),
            'to'           => $registros->lastItem(),
        ],
        'registros' => $registros
    ];
}
public function storeRegistroOdonto(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $registro = new RegistroOdontologico();
    $registro->idPaciente = $request->idPaciente;
    $registro->idUser = Auth::user()->id;
    $registro->comentario = $request->comentario;
    $registro->fecha_cita = $request->fechaCita;
    $registro->save();
}

public function updateRegistroOdonto(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $registro = RegistroOdontologico::findOrFail($request->id);
    $registro->comentario = $request->comentario;
    $registro->save();
}

public function eliminarRegistroOdonto(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $registro = RegistroOdontologico::findOrFail($request->id);
    $registro->delete();
}

public function indexSalas(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $salas = SalaPaciente::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{
            $salas = SalaPaciente::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        return [
            'pagination' => [
                'total'        => $salas->total(),
                'current_page' => $salas->currentPage(),
                'per_page'     => $salas->perPage(),
                'last_page'    => $salas->lastPage(),
                'from'         => $salas->firstItem(),
                'to'           => $salas->lastItem(),
            ],
            'salas' => $salas
        ];
    }


    
    public function storeSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = new SalaPaciente();
        $sala->nombre = $request->nombre;
        $sala->descripcion = $request->descripcion;
        $sala->idEmpresa = Auth::user()->idEmpresa;
        $sala->estado = "1";
        $sala->save();
    }

    public function updateSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = SalaPaciente::findOrFail($request->id);
        $sala->nombre = $request->nombre;
        $sala->descripcion = $request->descripcion;
        $sala->save();
    }
    public function desactivarSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = SalaPaciente::findOrFail($request->id);
        $sala->estado = 0;
        $sala->save();
    }

    public function activarSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = SalaPaciente::findOrFail($request->id);
        $sala->estado = 1;
        $sala->save();
    }

     public function selectSalas(Request $request){
        if (!$request->ajax()) return redirect('/');
        $salas = SalaPaciente::select('id','nombre','descripcion')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('id', 'asc')->get();
        return ['salas' => $salas];
    }

    public function eliminarSalas(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $sala = SalaPaciente::findOrFail($request->id);
    $sala->delete();
}

//ORTODONCIA

public function indexOrtodoncia(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $paciente = $request->idPaciente;
        $fromMain = $request->fromMain;
        
        if ($buscar==''){
            if ($fromMain == 1){
                $ortodoncia = Ortodoncia::orderBy('created_at', 'ASC')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->with('ortodonciaUsuario')->paginate(10);
            }else{
                $ortodoncia = Ortodoncia::orderBy('created_at', 'ASC')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->where('anulada','=', 0)->with('ortodonciaUsuario')->paginate(10);
            }
            
        }
        else{
            if ($fromMain == 1){
                $ortodoncia = Ortodoncia::where($criterio, 'like', '%'. $buscar . '%')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->with('ortodonciaUsuario')->orderBy('id', 'desc')->paginate(10);
            }else{  
                $ortodoncia = Ortodoncia::where($criterio, 'like', '%'. $buscar . '%')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->where('anulada','=', 0)->with('ortodonciaUsuario')->orderBy('id', 'desc')->paginate(10);
            }
           
        }
        

        return [
            'pagination' => [
                'total'        => $ortodoncia->total(),
                'current_page' => $ortodoncia->currentPage(),
                'per_page'     => $ortodoncia->perPage(),
                'last_page'    => $ortodoncia->lastPage(),
                'from'         => $ortodoncia->firstItem(),
                'to'           => $ortodoncia->lastItem(),
            ],
            'ortodoncia' => $ortodoncia
        ];
    }
    public function storeOrtodoncia(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $orto = new Ortodoncia();

        $orto->id_empresa = Auth::user()->idEmpresa;
        $orto->id_paciente = $request->id_paciente;
        $orto->arniti_12_sup = $request->arniti_12_sup;
        $orto->arniti_12_inf = $request->arniti_12_inf;

        $orto->arniti_14_sup = $request->arniti_14_sup;
        $orto->arniti_14_inf = $request->arniti_14_inf;
        $orto->arniti_16_sup = $request->arniti_16_sup;
        $orto->arniti_16_inf = $request->arniti_16_inf;

        $orto->arniti_18_sup = $request->arniti_18_sup;
        $orto->arniti_18_inf = $request->arniti_18_inf;
        $orto->arace_18_sup = $request->arace_18_sup;
        $orto->arace_18_inf = $request->arace_18_inf;

        $orto->arace_18_ctsup = $request->arace_18_ctsup;
        $orto->arace_18_ctinf = $request->arace_18_ctinf;
        $orto->ar_16x_niti = $request->ar_16x_niti;
        $orto->ar_16x_acero = $request->ar_16x_acero;

        $orto->ar_16x_sup = $request->ar_16x_sup;
        $orto->ar_16x_inf = $request->ar_16x_inf;
        $orto->ar_18x_niti = $request->ar_18x_niti;

        $orto->ar_18x_acero = $request->ar_18x_acero;
        $orto->ar_18x_sup = $request->ar_18x_sup;
        $orto->ar_18x_inf = $request->ar_18x_inf;
        $orto->dista_cuadra1 = $request->dista_cuadra1;

        $orto->dista_cuadra2 = $request->dista_cuadra2;
        $orto->dista_cuadra3 = $request->dista_cuadra3;
        $orto->dista_cuadra4 = $request->dista_cuadra4;
        $orto->cie_ccadena_sup = $request->cie_ccadena_sup;

        $orto->cie_ccadena_inf = $request->cie_ccadena_inf;
        $orto->cadena_de = $request->cadena_de;
        $orto->resorte_entre = $request->resorte_entre;
        $orto->cad_trac_pie_ret = $request->cad_trac_pie_ret;

        $orto->tec_arcsobr_sup = $request->tec_arcsobr_sup;
        $orto->tec_arcsobr_inf = $request->tec_arcsobr_inf;
        $orto->ela_intermax_cla2 = $request->ela_intermax_cla2;
        $orto->ela_intermax_cla3 = $request->ela_intermax_cla3;

        $orto->ela_intermax_caja = $request->ela_intermax_caja;
        $orto->ela_intermax_trian = $request->ela_intermax_trian;
        $orto->ela_intermax_dboton = $request->ela_intermax_dboton;
        $orto->feru_comp_sup = $request->feru_comp_sup;

        $orto->feru_comp_inf = $request->feru_comp_inf;
        $orto->ferubloq_antsup = $request->ferubloq_antsup;
        $orto->ferubloq_antinf = $request->ferubloq_antinf;
        $orto->ferubloq_cuadr1 = $request->ferubloq_cuadr1;
    
        $orto->ferubloq_cuadr2 = $request->ferubloq_cuadr2;
        $orto->ferubloq_cuadr3 = $request->ferubloq_cuadr3;
        $orto->ferubloq_cuadr4 = $request->ferubloq_cuadr4;
        
        $orto->incobrak_antsup = $request->incobrak_antsup;
        $orto->incobrak_antinf = $request->incobrak_antinf;
        $orto->incobrak_postsup = $request->incobrak_postsup;
        $orto->incobrak_postinf = $request->incobrak_postinf;

        $orto->incobrak_bracket = $request->incobrak_bracket;
        $orto->bracket_despega = $request->bracket_despega;
        $orto->botones_en = $request->botones_en;
        $orto->nota_cita = $request->nota_cita;
        $orto->userId = Auth::user()->id;
        $orto->save();
    }

    public function anularOrtodoncia(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $ortodoncia = Ortodoncia::findOrFail($request->idOrtodoncia);
        $ortodoncia->anulada = 1;
        $ortodoncia->save();
    }

    //===============================================ORDEN DE TRABAJO =======================================================
    public function indexOrdenTrabajo(Request $request)
    {
       if (!$request->ajax()) return redirect('/');
    
        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $registros = RegistroOdontologico::where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $registros = RegistroOdontologico::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $registros->total(),
                'current_page' => $registros->currentPage(),
                'per_page'     => $registros->perPage(),
                'last_page'    => $registros->lastPage(),
                'from'         => $registros->firstItem(),
                'to'           => $registros->lastItem(),
            ],
            'registros' => $registros
        ];
    }
    public function storeOrdenTrabajo(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $orden = new OrdenTrabajo();
        $orden->idPaciente = $request->idPaciente;
        $orden->idUser = Auth::user()->id;
        // $orden->num_otrabajo = $request->num_otrabajo;
        $orden->num_otrabajo = "OT01";
        $orden->fechaot = $request->fechaot;
        $orden->numero_contacto = $request->numero_contacto;
        $orden->descripcion = $request->descripcion;
        $orden->fechaentrega = $request->fechaentrega;
        $orden->color = $request->color;
        $orden->guia = $request->guia;
        $orden->observaciones = $request->observaciones;
        $orden->rodete = $request->rodete;
        $orden->metal = $request->metal;
        $orden->ceromero = $request->ceromero;
        $orden->prueba_dien = $request->prueba_dien;
        $orden->porcelana = $request->porcelana;
        $orden->ferula = $request->ferula;
        $orden->ppr_metal = $request->ppr_metal;
        $orden->terminado = $request->terminado;
        $orden->save();
    }
    
    public function updateOrdenTrabajo(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $registro = RegistroOdontologico::findOrFail($request->id);
        $registro->comentario = $request->comentario;
        $registro->save();
    }
    
    public function eliminarOrdenTrabajo(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $registro = RegistroOdontologico::findOrFail($request->id);
        $registro->delete();
    }


    //ODONTOGRAMA
    public function storeOdontograma(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $odonto = new Odontograma();
        $odonto->pac_id = $request->idPaciente;
        $odonto->otros = $request->idPaciente;
      
        $odonto->disponibleMX = 0;
        $odonto->necesarioMX = 0;
        $odonto->disponibleMD = 0;
        $odonto->necesarioMD = 0;
        $odonto->boltonMm = 0;
        $odonto->boltonMX = 0;
        $odonto->boltonMD = 0;
        $odonto->boltonnteriorMm = 0;
        $odonto->boltonnteriorMX = 0;
        $odonto->boltonnteriorMD = 0;
        
        $odonto->es_hta = $request->hta;
        $odonto->es_hepatitis = $request->hepa;
        $odonto->es_anemia = $request->anemia;
        $odonto->es_fumador = $request->fumador;
        $odonto->es_diabetes = $request->diabetes;
        $odonto->es_prenales = $request->renales;
        $odonto->es_ugastricas = $request->gastricas;
        $odonto->es_osteopo = $request->osteoporosis;
        $odonto->es_cancer = $request->cancer;
        $odonto->es_rquimote = $request->radiacion;
        $odonto->es_pcardia = $request->gastrcardiacosicas;
        $odonto->es_otros = $request->otros;
        $odonto->od_rellenos = $request->rellenos;
        $odonto->od_sellantes = $request->sellantes;
        $odonto->od_extracc = $request->extracciones;
        $odonto->od_limpiezas = $request->limpiezas;
        $odonto->od_endodon = $request->endodoncias;
        $odonto->od_implantes = $request->implantes;
        $odonto->od_prostodoncia = $request->prostodoncias;
        $odonto->ort_mdesvia = $request->mordidaDesviacion;
        $odonto->ort_mabierta = $request->mordidaAbierta;
        $odonto->ort_mprofu = $request->mordidaProfunda;
        $odonto->ort_mcruant = $request->morCruzadaAnterior;
        $odonto->ort_mcrupost = $request->morCruzadaPosterior;
        $odonto->ort_mtije = $request->mordidaTijera;
        $odonto->ort_apileve = $request->apiLeve;
        $odonto->ort_apimode = $request->apiModerado;

        $odonto->ort_apisevero = $request->apiSevero;
        $odonto->ort_perfil = $request->perfil;
        $odonto->ort_lmedias = $request->liMedia;
        $odonto->ort_overyet = $request->overyet;
        $odonto->ort_diastemas = $request->diastemas;
        $odonto->ort_ttrata = $request->tipoTratamiendo;
        $odonto->ort_duracion = $request->duracion;
        $odonto->ort_notas = $request->notas;
        $odonto->imgName = $request->idPaciente."_".date("mdyHis");
        $odonto->save();

        if ($request->imgBase64 != "empty") {
            self::saveFoto($request->imgBase64, $request->idPaciente."_".date("mdyHis"));
        }        
    }
    public function saveFotoOdonto($base64Img,$name){
        $extension = explode('/', explode(':', substr($base64Img, 0, strpos($base64Img, ';')))[1])[1];   // .jpg .png .pdf

        $replace = substr($base64Img, 0, strpos($base64Img, ',')+1); 

        // find substring fro replace here eg: data:image/png;base64,

        $image = str_replace($replace, '', $base64Img); 
        $image = str_replace(' ', '+', $image); 
        $imageName = $name.'.'."jpg";
        Storage::disk('odontogramas')->put($imageName, base64_decode($image));
    }

    public function indexOdontograma(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        if ($buscar==''){
            $odontogramas = Odontograma::where('pac_id','=', $request->idPaciente)
            ->orderBy('created_at', 'ASC')->paginate(10);
        }
        else{
            $odontogramas = Odontograma::where($criterio, 'like', '%'. $buscar . '%')->where('pac_id','=', $request->idPaciente)->orderBy('created_at', 'ASC')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $odontogramas->total(),
                'current_page' => $odontogramas->currentPage(),
                'per_page'     => $odontogramas->perPage(),
                'last_page'    => $odontogramas->lastPage(),
                'from'         => $odontogramas->firstItem(),
                'to'           => $odontogramas->lastItem(),
            ],
            'odontogramas' => $odontogramas
        ];
    }

    public function getFotoOdontoBase64(Request $request){
        $file = "" ;
        $base64img = "";
        //return $request->idPaciente;
        if (Storage::disk('odontogramas')->exists($request->imgName)) {
           $file =  Storage::disk('odontogramas')->get($request->imgName);
          return  "data:image/jpeg;base64,".base64_encode($file);
        }else{
            return 'empty';
        }
    }

    // ===========================================END FICHA EMPLEADO===========================================================================================================


    //TIPO DE INGRESO EMPLEADO

    public function indexTipoIngreso(Request $request)
    {
       if (!$request->ajax()) return redirect('/');
    
        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $tiposIngresos = TipoIngresoEmpleado::where('idempresa','=',Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $tiposIngresos = TipoIngresoEmpleado::where($criterio, 'like', '%'. $buscar . '%')->where('idempresa','=',Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $tiposIngresos->total(),
                'current_page' => $tiposIngresos->currentPage(),
                'per_page'     => $tiposIngresos->perPage(),
                'last_page'    => $tiposIngresos->lastPage(),
                'from'         => $tiposIngresos->firstItem(),
                'to'           => $tiposIngresos->lastItem(),
            ],
            'tiposIngresos' => $tiposIngresos
        ];
    }
    public function storeTipoIngreso(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $tipoIngreso = new TipoIngresoEmpleado();
        $tipoIngreso->idempresa = Auth::user()->idEmpresa;
        $tipoIngreso->cod_ingreso = $request->codigo;
        $tipoIngreso->nombre = $request->nombre;
        $tipoIngreso->aplica_ss = $request->aplicaSS;
        $tipoIngreso->aplica_afp = $request->aplicaAfp;
        $tipoIngreso->aplica_isr = $request->aplicaISR;
        $tipoIngreso->es_bono = $request->esBono;
        $tipoIngreso->es_hora_extra = $request->esHoraExtra;
        $tipoIngreso->ingreso_gravado = $request->ingresoGravado;
        $tipoIngreso->es_vacacion = $request->esVacacion;
        $tipoIngreso->porcentaje_hxtra = $request->porcentaje;
        $tipoIngreso->save();
    }

    public function updateTipoIngreso(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $tipoIngreso = TipoIngresoEmpleado::findOrFail($request->id);
        $tipoIngreso->cod_ingreso = $request->codigo;
        $tipoIngreso->nombre = $request->nombre;
        $tipoIngreso->aplica_ss = $request->aplicaSS;
        $tipoIngreso->aplica_afp = $request->aplicaAfp;
        $tipoIngreso->aplica_isr = $request->aplicaISR;
        $tipoIngreso->es_bono = $request->esBono;
        $tipoIngreso->es_hora_extra = $request->esHoraExtra;
        $tipoIngreso->ingreso_gravado = $request->ingresoGravado;
        $tipoIngreso->es_vacacion = $request->esVacacion;
        $tipoIngreso->porcentaje_hxtra = $request->porcentaje;
        $tipoIngreso->save();
    }

    public function eliminarTipoIngreso(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $tipoIngreso = TipoIngresoEmpleado::findOrFail($request->id);
        $tipoIngreso->delete();
    }

    //DEDUCCIONES
    public function indexDeduccion(Request $request)
    {
       if (!$request->ajax()) return redirect('/');
    
        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $deducciones = Deduccion::where('idempresa','=',Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $tiposIngresos = Deduccion::where($criterio, 'like', '%'. $buscar . '%')->where('idempresa','=',Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $deducciones->total(),
                'current_page' => $deducciones->currentPage(),
                'per_page'     => $deducciones->perPage(),
                'last_page'    => $deducciones->lastPage(),
                'from'         => $deducciones->firstItem(),
                'to'           => $deducciones->lastItem(),
            ],
            'deducciones' => $deducciones
        ];
    }
    public function storeDeduccion(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $deduccion = new Deduccion();
        $deduccion->idempresa = Auth::user()->idEmpresa;
        $deduccion->codigo = $request->codigo;
        $deduccion->nombre = $request->nombre;
        $deduccion->valor = $request->valor;
        $deduccion->valor_maximo = $request->valorMaximo;
        $deduccion->idcuentaconta = 0;
        $deduccion->save();
    }

    public function updateDeduccion(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $deduccion = Deduccion::findOrFail($request->id);
        $deduccion->codigo = $request->codigo;
        $deduccion->nombre = $request->nombre;
        $deduccion->valor = $request->valor;
        $deduccion->valor_maximo = $request->valorMaximo;
        $deduccion->save();
    }

    public function eliminarDeduccion(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $deduccion = Deduccion::findOrFail($request->id);
        $deduccion->delete();
    }


     //SUSPENSIONES
     public function indexSuspension(Request $request)
     {
        if (!$request->ajax()) return redirect('/');
     
         $buscar = $request->buscar;
         $criterio = $request->criterio;
         
         if ($buscar==''){
             $suspensiones = SuspensionEmpleado::orderBy('id', 'desc')->paginate(10);
         }
         else{
             $suspensiones = SuspensionEmpleado::where($criterio, 'like', '%'. $buscar . '%')->orderBy('id', 'desc')->paginate(10);
         }
         
         return [
             'pagination' => [
                 'total'        => $suspensiones->total(),
                 'current_page' => $suspensiones->currentPage(),
                 'per_page'     => $suspensiones->perPage(),
                 'last_page'    => $suspensiones->lastPage(),
                 'from'         => $suspensiones->firstItem(),
                 'to'           => $suspensiones->lastItem(),
             ],
             'suspensiones' => $suspensiones
         ];
     }
     public function storeSuspension(Request $request)
     {
         if (!$request->ajax()) return redirect('/');
         $suspension = new SuspensionEmpleado();
         $suspension->tipo = $request->tipo;
         $suspension->nombre = $request->nombre;
         $suspension->porcentaje_ordinar = $request->porcentajeOrdinar;
         $suspension->porcentaje_bono = $request->porcentajeBono;
         $suspension->porcentaje_ss = $request->porcentajeSS;
         $suspension->save();
     }
 
     public function updateSuspension(Request $request)
     {
         if (!$request->ajax()) return redirect('/');
         $suspension = SuspensionEmpleado::findOrFail($request->id);
         $suspension->tipo = $request->tipo;
         $suspension->nombre = $request->nombre;
         $suspension->porcentaje_ordinar = $request->porcentajeOrdinar;
         $suspension->porcentaje_bono = $request->porcentajeBono;
         $suspension->porcentaje_ss = $request->porcentajeSS;
         $suspension->save();
     }
 
     public function eliminarSuspension(Request $request)
     {
         if (!$request->ajax()) return redirect('/');
         $suspension = SuspensionEmpleado::findOrFail($request->id);
         $suspension->delete();
     }


     public function selectSupension(Request $request){
        if (!$request->ajax()) return redirect('/');
        $suspensiones = SuspensionEmpleado::select('id','tipo','nombre')
        ->orderBy('id', 'asc')->get();
        return ['suspensiones' => $suspensiones];
    }

     public function getListadoEmpleados(Request $request){
        if (!$request->ajax()) return redirect('/');
        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $departamento = $request->departamento;
        if ($buscar==''){
            $empleados = Empleado::select(
                'id',
                'nombre',
                'apellido',
                'idDpto',
                'codEmpleado',
                DB::raw("0 AS cantDias"),
                DB::raw("'' AS comentarios"),
                DB::raw("'' AS fecha_desde"),
                DB::raw("'' AS fecha_hasta")
                )
            ->with("departamentoEmpleado")
            ->where('idEmpresa','=', Auth::user()->idEmpresa);
            if ($departamento > 0 ){
                $empleados=$empleados->where('idDpto','=',$departamento);
            }
            $empleados=$empleados->orderBy('id', 'desc')->paginate(200);
        }
        else{
            $empleados = Empleado::select(
                'id',
                'nombre',
                'apellido',
                'idDpto',
                'codEmpleado',
                DB::raw("0 AS cantDias"),
                DB::raw("'' AS comentarios"),
                DB::raw("'' AS fecha_desde"),
                DB::raw("'' AS fecha_hasta")
                )
            ->with("departamentoEmpleado")
            ->where('idEmpresa','=', Auth::user()->idEmpresa)
            ->where($criterio, 'like', '%'. $buscar . '%')
            ->orderBy('id', 'desc')->paginate(200);
            // $empleados = Empleado::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(200);
        }
        

        return [
            'pagination' => [
                'total'        => $empleados->total(),
                'current_page' => $empleados->currentPage(),
                'per_page'     => $empleados->perPage(),
                'last_page'    => $empleados->lastPage(),
                'from'         => $empleados->firstItem(),
                'to'           => $empleados->lastItem(),
            ],
            'empleados' => $empleados
        ];
     }

     public function storeSuspensionesEmpleados(Request $request)
     {
         if (!$request->ajax()) return redirect('/');
         try{
             $suspensiones =  $request->suspensiones;
             DB::beginTransaction();
             //GUARDAMOS ENCABEZADO STATUS
            //  $historico = new HistoricoStatusAbono();
            //  $historico->ID_numreporte = $request->numeroAbono;
            //  $historico->status = 1;
            //  $historico->usuario_genera = Auth::user()->id;
            //  $historico->usuario_revisa = 0;
            //  $historico->fecha_aprueba = $request->fechaAbono;
            //  $historico->save();
             //GUARDAMOS SUSPENSIONES
             foreach($suspensiones as $ep=>$sus){ 
                 if ($sus["cantDias"] > 0){
                     $suspension = new SuspensionEmpleadoDet();
                     $suspension->idempleado = $sus["id"];
                     $suspension->id_tipo_suspension = $request->idSuspension;
                     $suspension->idplanilla = 0;
                     $suspension->comentarios = $sus["comentarios"];
                     $suspension->numero_dias = $sus["cantDias"];
                     $suspension->fecha_inicial = $sus["fecha_desde"];
                     $suspension->fecha_final = $sus["fecha_hasta"];
                     $suspension->save();
 
                     //ACTUALIZAMOS SALDO DE DOCUMENTO EN FACTURA
                    //  $venta = Venta::findOrFail($abono->ID_factura);
                    //  $saldoDocumento =  $venta->saldo_documento;
                    //  $venta->saldo_documento = $saldoDocumento - $abono->abono;
                    //  $venta->save();
 
                     //ACTUALIZAMOS MONTOS PARA CLIENTES
                    //  $cliente = Cliente::findOrFail($abono->ID_cliente);
                    //  $cliente->montodisponible =  $cliente->montodisponible + $abono->abono;
                    //  $cliente->creditoutilizado =  $cliente->creditoutilizado - $abono->abono;
                    //  $cliente->save();
                 }  
                 
             }
             DB::commit();
         }catch (Exception $e){
             DB::rollBack();
         }
     }

}
