<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Empresa;

class EmpresaController extends Controller
{
    public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $empresas = Empresa::orderBy('id', 'desc')->paginate(10);
        }
        else{
            $empresas = Empresa::where($criterio, 'like', '%'. $buscar . '%')->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $empresas->total(),
                'current_page' => $empresas->currentPage(),
                'per_page'     => $empresas->perPage(),
                'last_page'    => $empresas->lastPage(),
                'from'         => $empresas->firstItem(),
                'to'           => $empresas->lastItem(),
            ],
            'empresas' => $empresas
        ];
    }


    public function listarPdf(){
        $empresas = Empresa::orderBy('empresa.nombre', 'asc')->get();        
        $cont=Empresa::count();

        $pdf = \PDF::loadView('pdf.empresaspdf',['empresas'=>$empresas,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('empresas.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $empresa = new Empresa();
        $empresa->nombre = $request->nombre;
        $empresa->nit = $request->nit;
        $empresa->nrc = $request->nrc;
        $empresa->direccion = $request->direccion;
        $empresa->telefono = $request->telefono;
        $empresa->estado = "1";
        
        $empresa->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $empresa = Empresa::findOrFail($request->id);
        $empresa->nombre = $request->nombre;
        $empresa->nit = $request->nit;
        $empresa->nrc = $request->nrc;
        $empresa->direccion = $request->direccion;
        $empresa->telefono = $request->telefono;
        $empresa->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $empresa = Empresa::findOrFail($request->id);
        $empresa->estado = 0;
        $empresa->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $empresa = Empresa::findOrFail($request->id);
        $empresa->estado = 1;
        $empresa->save();
    }
}
