<?php

namespace App\Http\Controllers;

use App\Bodega;
use Illuminate\Http\Request;
use App\Venta;
use App\DetalleVenta;
use Illuminate\Support\Facades\Storage;
use App\Cliente;
use Illuminate\Support\Facades\DB;
use App\Services\FacturaElectronicaServiceService;
use App\Parameter;
use Carbon\Carbon;
use App\FacturaElectronicaLog;
use Ramsey\Uuid\Uuid;
use Mpdf\Mpdf;
use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Illuminate\Support\Facades\Response;
use App\Incoterm;
use App\Pais;
use App\RegimenExportacion;
use App\RecintoFiscal;
use App\TipoDocumentoDTE;
class FacturaElectronicaController extends Controller
{
    public function buildFacturaJSON($facturaId,$enContingencia)
    {
        $factura = Venta::with('paisRelacion')->with('incotermRelacion')->findOrFail($facturaId);
        $tipoDte = "01";
        if ($factura->tipo_documento == "03" || $factura->tipo_documento == "05" ) {
            $tipoDte = "03";
        }
        $ambiente =  Parameter::where('name', 'ambiente')->first();
        $user =  Parameter::where('name', 'user')->first();
        $nrc =  Parameter::where('name', 'nrc')->first();
        $nombreEmisor =  Parameter::where('name', 'nombreEmisor')->first();
        $esConsumidorFinal = $factura->tipo_documento == "01" ? true : false;
        $esCreditoFiscal = $factura->tipo_documento == "03" ? true : false;
        $esSujetoExcluido = $factura->tipo_documento == "14" ? true : false;
        $esNotaCredito = $factura->tipo_documento == "05" ? true : false;
        $esExportacion = $factura->tipo_documento == "11" ? true : false;
        $esRetencion = $factura->tipo_documento == "07" ? true : false;
        $esClientesVarios = $factura->idCliente == 1611 ? true : false;
        $numeroControl =  $factura->numeroControl;
        $decimalPlaces = 4;
        $horaEmision = Carbon::now('America/El_Salvador')->format('H:i:s');
        $identificacion = [
            'version' => intval($tipoDte),
            'ambiente' => $ambiente->value,
            'tipoDte' =>  $factura->tipo_documento,
            'numeroControl' => $numeroControl,
            'codigoGeneracion' => $factura->codigoGeneracion,
            'tipoModelo' => $enContingencia == 1 ? 2 : 1,
            'tipoOperacion' => $enContingencia == 1 ? 2 : 1,
            'tipoContingencia' => $enContingencia == 1 ? 1 : null ,
            'fecEmi' => $factura->fecha_factura,
            'horEmi' => $horaEmision,
            'tipoMoneda' => "USD"
        ];
        if ($esExportacion) {
            $identificacion['motivoContigencia'] = null;
        }else{
            $identificacion['motivoContin'] = null;
        }

        if ($esNotaCredito || $esRetencion) {
        //======================INICIO JSON DOCUMENTO RELACIONADO============================================================================//
            $documentos = DB::select("SELECT * from fc_factura_dte_docrela WHERE idfactura = " . $facturaId);
            $documentosJSON = [];
            foreach ($documentos as $row) {
                $documento = [
                    'tipoDocumento' => $row->tipo_documento,
                    'tipoGeneracion' => $row->tipo_generacion,
                    'numeroDocumento' => $row->numero_documento,
                    'fechaEmision' => $row->fecha_emision,
                ];
                if ($esRetencion){
                    $documento['valordocumento'] = $row->valordocumento;
                }
                $documentosJSON[] = $documento;
            }
        }
       
        //======================INICIO JSON EMISOR============================================================================//
        $direccion = [
            'departamento' => "05",
            'municipio' => "11",
            'complemento' => "C. Daniel Hernández 8-10",
        ];
        $emisor = [
            'nit' => $user->value,
            'nrc' => $nrc->value,
            'nombre' => $nombreEmisor->value,
            'codActividad' => "47739",
            'descActividad' => "Venta al por menor de otros productos n.c.p.",
            'direccion' => $direccion,
            'telefono' => "22287661",
            'correo' => "alupita@alimentoslupita.com"
        ];
        if (!$esNotaCredito && !$esRetencion){
            $emisor['codEstableMH'] = "0000";
            $emisor['codEstable'] = "M001";
            $emisor['codPuntoVentaMH'] = "0000";
            $emisor['codPuntoVenta'] = "0000";
        }

        if ($esRetencion){
            $emisor['puntoVentaMH'] = null;
            $emisor['codigoMH'] = null;
            $emisor['puntoVenta'] = null;
            $emisor['codigo'] = null;
        }
        if (!$esSujetoExcluido){
            $emisor['tipoEstablecimiento'] = "01";
            $emisor['nombreComercial'] = "ENCURTIDOS lUPITA";
            // $emisor['tipoEstablecimiento'] = "GRUPO DIT";
        }

        if ($esExportacion){
            $emisor['regimen'] = $factura->regimenExportacion;
            $emisor['recintoFiscal'] = $factura->recintoFiscal;
            $emisor['tipoItemExpor'] = 1;
        }

        //======================FIN JSON EMISOR============================================================================//

        //======================INICIO JSON RECEPTOR============================================================================//

        $receptor = Cliente::join('fc_factura', 'cliente.id', '=', 'fc_factura.idCliente')
            ->join('cliente_gironegocio', 'cliente.idClienteGiro', '=', 'cliente_gironegocio.id')
            ->leftJoin('fc_sucxcliente', 'cliente.id', '=', 'fc_sucxcliente.idCliente')
            ->select(
                'cliente.nit AS nit',
                'cliente.nrc AS nrc',
                'cliente.dui AS dui',
                'cliente.nombre AS nombre',
                'cliente_gironegocio.codigo AS codActividad',
                'cliente_gironegocio.descripcion AS descActividad',
                DB::raw("'null' as nombreComercial"),
                'cliente.departamento AS departamento',
                'cliente.municipio AS municipio',
                'cliente.direccion AS complemento',
                'cliente.teltrabajo AS telefono',
                'cliente.email AS correo',
                DB::raw("'null' as otrosDocumento"),
                DB::raw("'null' as ventaTercero"),
                'fc_factura.id',
                'fc_sucxcliente.codigo as numeroTienda'
            )
            ->where('fc_factura.id', '=', $facturaId)
            ->first();

            if ($esClientesVarios) {
                $direccion = null;
            }else{
                $direccion = [
                    'departamento' => $receptor->departamento,
                    'municipio' => $receptor->municipio,
                    'complemento' => $receptor->complemento,
                ];
            }
        $nombreReceptor = $esSujetoExcluido ? "sujetoExcluido" : "receptor";
        $receptorJSON = [
            'descActividad' => $receptor->descActividad,
            'telefono' => ($receptor->telefono == "0" || $receptor->telefono == 0) ? null :  $receptor->telefono,
            'correo' => $receptor->correo,
        ];

        if ($esExportacion) {
            $receptorJSON['complemento'] = $receptor->complemento;
            $receptorJSON['tipoDocumento'] = "37";
            $receptorJSON['numDocumento'] = ($receptor->nrc == "0" || $receptor->nrc == 0) ? null: self::removeCharacterFromString("-",$receptor->nrc);
        }else{
            $receptorJSON['direccion'] = $direccion;
            $receptorJSON['codActividad'] =  $receptor->codActividad;
            $receptorJSON['nrc'] =  ($receptor->nrc == "0" || $receptor->nrc == 0) ? null: self::removeCharacterFromString("-",$receptor->nrc);
        }
        $sujetoExcluidoJSON = [
            'tipoDocumento' =>  "36",
            'numDocumento' => ($receptor->dui == "0" || $receptor->dui == 0) ? "null" : self::removeCharacterFromString("-",$receptor->dui),
            'nombre' => $receptor->nombre,
            'codActividad' => null,
            'descActividad' => $receptor->descActividad,
            'direccion' => $direccion,
            'telefono' => $receptor->telefono,
            'correo' => $receptor->correo,
        ];
        if ($esConsumidorFinal || $esClientesVarios) {
            $receptorJSON['tipoDocumento'] = "36";
            $receptorJSON['numDocumento'] = ($receptor->dui == "0" || $receptor->dui == 0) ? "null" : self::removeCharacterFromString("-",$receptor->dui);
        }else{
            if (!$esExportacion && !$esRetencion){
                $receptorJSON['nit'] = self::removeCharacterFromString("-",$receptor->nit);
            }
            $receptorJSON['nombreComercial'] = null;
            
        }
        if ($esClientesVarios) {
            $receptorJSON['nombre'] = $factura->facturaNombre;
            $receptorJSON['codActividad'] = null;
            $receptorJSON['descActividad'] = null;
            $receptorJSON['correo'] = null;
            $receptorJSON['nrc'] = null;
        }else{
            $receptorJSON['nombre'] = $receptor->nombre;
            if(!$esCreditoFiscal) {
                // $receptorJSON['id'] = $receptor->id;
                // $receptorJSON['otrosDocumento'] = null;
                // $receptorJSON['ventaTercero'] = null;
            }
        }
        if ($esExportacion) {
            $receptorJSON['codPais'] = $factura->pais;
            $receptorJSON['nombrePais'] = $factura->paisRelacion->pais;
            $receptorJSON['tipoPersona'] = 2;
        }
        if ($esRetencion){
            $receptorJSON['tipoDocumento'] = "36";
            $receptorJSON['numDocumento'] = self::removeCharacterFromString("-",$receptor->nit);
        }
        //======================INICIO JSON DETALLE DE VENTA============================================================================//
        DB::statement('SET @row_number := 0');
        $detalleVenta = DB::select(
            "
        SELECT
            (@row_number:=@row_number + 1) AS numItem,
            articulos.tipo_producto AS tipoItem,
            articulos.codigo AS codigo,
            articulos.nombre AS descripcion,
            fc_factura_detalle.cantidad AS cantidad,
            articulos.umedida AS uniMedida,
            articulos.cuenta_contable1 AS cotransFovial,
            fc_factura_detalle.precio_venta AS precioUni,
            fc_factura_detalle.sub_total_producto AS ventaGravada,
            fc_factura_detalle.ivaLinea AS ivaItem,
            fc_factura_detalle.servicio_conta AS servicio_conta,
            fc_factura_detalle.observaciones AS observaciones,
            fc_factura_detalle.numeroDocumento AS numeroDocumento,
            conta_cuentas_contables.codigo as codigoCuenta
            FROM (fc_factura
            JOIN fc_factura_detalle ON fc_factura.id = fc_factura_detalle.idFactura)
            LEFT JOIN articulos ON fc_factura_detalle.idProducto = articulos.id
            LEFT JOIN conta_cuentas_contables ON fc_factura_detalle.idProducto = conta_cuentas_contables.id
            WHERE fc_factura.id = " . $facturaId
        );
        $detalleVentaJSON = [];
        $llevaFovialCotrans = false;
        $totalIva = 0.0;
        foreach ($detalleVenta as $row) {
            $tributos = [];
            // if ($row -> cotransFovial == 1) {
            //     $tributos[] = "D1";
            //     $tributos[] = "C8";
            //     $llevaFovialCotrans = true;
            // }
            if ($esCreditoFiscal || $esNotaCredito){
                $tributos[] = "20";
            }else{
                $tributos = null;
            }
            // if ($esConsumidorFinal && $row -> cotransFovial == 2) { //NO ES AFIN AL CONTRANS
                // $tributos = null;
            // }

            if ($row->servicio_conta == 1) { //SERVICIO
                $row->tipoItem = 2;
                $row->descripcion = $row->observaciones;
                $row->uniMedida = "99";
                $row->codigo = $row->codigoCuenta;
            }
            $lineaProducto = [
                'numItem' => $row->numItem,
                'codigo' => $row->codigo,
                'descripcion' => $row->descripcion,
                'cantidad' => $row->cantidad,
                'uniMedida' => intval($row->uniMedida),
                'precioUni' => $row->precioUni,
                'montoDescu' => 0.0
            ];
            if (!$esExportacion) {
                $lineaProducto['tipoItem'] = $row->tipoItem;
            }
            if (!$esSujetoExcluido){
                if ($esNotaCredito) {
                    $lineaProducto['numeroDocumento'] =  $row->numeroDocumento;
                }else if (!$esExportacion){
                    $lineaProducto['numeroDocumento'] = null;
                }
                
                if (!$esExportacion) {
                    $lineaProducto['ventaNoSuj'] = 0.0;
                    $lineaProducto['ventaExenta'] = 0.0;
                    $lineaProducto['codTributo'] = null;
                }
                
                $lineaProducto['ventaGravada'] = round($row->ventaGravada, $decimalPlaces);
                $lineaProducto['tributos'] = $tributos;
                if (!$esNotaCredito){
                    if (!$esExportacion) {
                        $lineaProducto['psv'] = 0.0;
                    }
                    $lineaProducto['noGravado'] = 0.0;
                }
            }else{
                $lineaProducto['compra'] = round($row->ventaGravada, $decimalPlaces);
            }
            if (!$esSujetoExcluido) {
                if ($esConsumidorFinal){
                    $ventaGravada =  $row->ventaGravada;
                    $precioSinIva =  $ventaGravada/1.13;
                    $ivaCalculado = $ventaGravada - $precioSinIva;
                    $lineaProducto['ivaItem'] = round($ivaCalculado, $decimalPlaces);
                    $totalIva = $totalIva + $ivaCalculado;
                }
                // else{
                //     $lineaProducto['ivaItem'] = $row->ivaItem;
                // }
            }
            if ($esRetencion){
                $facturaSujExcludio = $documentosJSON[0];
                $lineaProducto = [
                    'descripcion' => $facturaSujExcludio["numeroDocumento"],
                    'numDocumento' => $facturaSujExcludio["numeroDocumento"],
                    'tipoDoc' => 1,
                    'codigoRetencionMH' => "C4",
                    'fechaEmision' => $facturaSujExcludio["fechaEmision"],
                    'montoSujetoGrav' => $facturaSujExcludio["valordocumento"],
                    'tipoDte' => "03",
                    "numItem" => $row->numItem,
                    "ivaRetenido" => $factura->ret13Iva
                ];
            }
            $detalleVentaJSON[] = $lineaProducto;
        }

        //======================FIN JSON DETALLE DE VENTA============================================================================//
        $tipoDocumento = $factura->tipo_documento;
        $iva = [
            'codigo' => '20',
            'descripcion' => 'Impuesto al Valor Agregado 13%',
            'valor' => $factura->iva_fc,
        ];

        $fovial = [
            'codigo' => 'D1',
            'descripcion' => 'FOVIAL ($0.20 Ctvs. por galón)',
            'valor' => $factura->fovial,
        ];

        $cotrans = [
            'codigo' => 'C8',
            'descripcion' => 'COTRANS ($0.10 Ctvs. por galón)',
            'valor' => $factura->cotrans,
        ];
        $tributosResumen = [];
        if ($llevaFovialCotrans && !$esSujetoExcluido) {
            $tributosResumen = [
                $fovial,
                $cotrans
            ];
        }
        if($esCreditoFiscal || $esNotaCredito) {
            $tributosResumen[] = $iva;
        }

         //=================PAGOS FACTURA ===================================================================//
         $pagosArray = [];
        //  $countPagos = 1;
        //  if ($factura->tipoPago > 0) { //EFECTIVO
        //      $pagosArray [] = [
        //          'codigo' => $countPagos > 10 ? ''.$countPagos : '0'.$countPagos ,
        //          'montoPago' => floatval($factura->tipoPago),
        //          'referencia' => null,
        //          'plazo' => null,
        //          'periodo' => null
        //      ];
        //      $countPagos  = $countPagos + 1;
        //  }
        //  if ($factura->tipoPago2 > 0) { //TARJETA
        //      $pagosArray [] = [
        //          'codigo' => $countPagos > 10 ? ''.$countPagos : '0'.$countPagos ,
        //          'montoPago' => floatval($factura->tipoPago2),
        //          'referencia' => null,
        //          'plazo' => null,
        //          'periodo' => null
        //      ];
        //      $countPagos  = $countPagos + 1;
        //  }
 
        //  if ($factura->tipoPago4 > 0) { //TRANSFERENCIA
        //      $pagosArray [] = [
        //          'codigo' => $countPagos > 10 ? ''.$countPagos : '0'.$countPagos ,
        //          'montoPago' => floatval($factura->tipoPago4),
        //          'referencia' => null,
        //          'plazo' => null,
        //          'periodo' => null
        //      ];
        //      $countPagos  = $countPagos + 1;
        //  }
        $pagosArray [] = [
            'codigo' => "03",
            'montoPago' => floatval($factura->total_fc),
            'referencia' => null,
            'plazo' => $factura->plazo_dte,
            'periodo' => $factura->dias_credito > 0 ? $factura->dias_credito : 1
        ];
        $resumenFactura = [
            'totalDescu' => 0.0,
            'totalLetras' => self::numberToText($factura->total_fc),
            'condicionOperacion' => $factura->condicion_venta
        ];
        if (!$esExportacion) {
            $resumenFactura['ivaRete1'] =  $factura->retencion_1;
            $resumenFactura['subTotal'] =   round($factura->sub_total_fc,2);
            $resumenFactura['reteRenta'] =   $factura->retencion_10;
        }
        if (!$esNotaCredito){
            $resumenFactura['totalPagar'] =$factura->total_fc;
            $resumenFactura['pagos'] = $pagosArray;
        }
        if ($esSujetoExcluido) {
            $resumenFactura['totalCompra'] =$factura->sub_total_fc;
            $resumenFactura['subTotal'] =round($factura->sub_total_fc,2);
            $resumenFactura['descu'] = 0.0;
            $resumenFactura['observaciones'] = null;
        }else{
            if (!$esExportacion) {
                $resumenFactura['totalNoSuj'] = 0.0;
                $resumenFactura['descuNoSuj'] = 0.0;
                $resumenFactura['subTotalVentas'] =round($factura->sub_total_fc,2);
                $resumenFactura['tributos'] = $tributosResumen;
                $resumenFactura['descuExenta'] = 0.0;
                $resumenFactura['descuGravada'] = 0.0;
                $resumenFactura['totalExenta'] = 0.0;
            }
            $resumenFactura['totalGravada'] =round($factura->sub_total_fc,2);
            if ($esNotaCredito){
                $resumenFactura['montoTotalOperacion'] = round($factura->total_fc, 2);
            }else{
                $resumenFactura['montoTotalOperacion'] = round($factura->total_fc + $factura->retencion_1 - $factura->percepcion_1, 2); 
            }

            if (!$esNotaCredito){
                $resumenFactura['porcentajeDescuento'] = 0.0;
                $resumenFactura['totalNoGravado'] = 0.0;
                if (!$esExportacion) {
                    $resumenFactura['saldoFavor'] = 0.0;
                }
                $resumenFactura['numPagoElectronico'] = null;
            }
        }
        if ($esExportacion) {
            $resumenFactura['descuento'] = 0.0;
            $resumenFactura['flete'] = round($factura->flete, 2);
            $resumenFactura['seguro'] =round($factura->seguro, 2);
            $resumenFactura['codIncoterms'] = $factura->incoterm;
            $resumenFactura['descIncoterms'] = $factura->incotermRelacion->incoterms;
            $resumenFactura['observaciones'] = null;
            // incotermRelacion
        }

        if (!$esSujetoExcluido){
            if ($esConsumidorFinal) {
                $resumenFactura['totalIva'] = round($totalIva, 2);
            }
            // else{
            //     $resumenFactura['totalIva'] = $factura->iva_fc;
            // }
        }
        
        $resumenFactura['condicionOperacion'] = 1;

        // if (!$esCreditoFiscal && !$esSujetoExcluido) {
        //     $resumenFactura['extension'] = null;
        // }
        if ($esCreditoFiscal || $esNotaCredito){
            $resumenFactura['ivaPerci1'] = $factura->percepcion_1;
        }

        if ($esRetencion){
            $resumenFactura= [
                'totalSujetoRetencion' => round($factura->sub_total_fc,2),
                'totalIVAretenidoLetras' => self::numberToText($factura->ret13Iva),
                'totalIVAretenido' => round($factura->ret13Iva,2)
            ];
        }
        $unifiedData = [
            'identificacion' => $identificacion,
            'emisor' => $emisor,
            $nombreReceptor => $esSujetoExcluido ? $sujetoExcluidoJSON : $receptorJSON,
            // "otrosDocumentos" => null,
            // "ventaTercero" => null,
            "cuerpoDocumento" => $detalleVentaJSON,
            "resumen" => $resumenFactura
        ];
        if ($esNotaCredito) {
            $unifiedData['documentoRelacionado'] = $documentosJSON;
        }
        if (!$esSujetoExcluido){
            if (!$esNotaCredito && !$esRetencion){
                $unifiedData['otrosDocumentos'] = null;
            }
            if (!$esNotaCredito && !$esExportacion && !$esRetencion) {
                $unifiedData['documentoRelacionado'] = null;
            }
            if  (!$esRetencion){
                $unifiedData['ventaTercero'] = null;
            }
            if (!$esExportacion) {
                $unifiedData['extension'] = null;
            }
        }
        // if ($esClientesVarios) {
            // if ($esCreditoFiscal || $esClientesVarios){
            //     $unifiedData['ventaTercero'] = null;
            //     $unifiedData['extension'] = null;
            // }
            
            $apendiceUsuario = [
                "campo" => "EMP",
                "etiqueta" => "USUARIO GENERA",
                "valor" => "".$factura->idVendedor
            ];

            $bodega = Bodega::findOrFail($factura->idBodega);
           // $apendiceSucursal = [
             //   "campo" => "SUCURSAL",
             //   "etiqueta" => "SUCURSAL",
             //   "valor" => $bodega->nombre
           // ];

            $apendiceWmoc = [
                "campo" => "WMOC",
                "etiqueta" => "Numero de Orden de compra(10 dígitos)",
                "valor" => "".$factura->ruta_cobro
            ];

            $apendiceWmven = [
                "campo" => "WMVEN",
                "etiqueta" => "Numero proveedor Walmart(9digitos)",
                "valor" => "035931130"
            ];
            

       //     $apendiceWmgln = [
         //       "campo" => "WMGLN",
           //     "etiqueta" => "GLN equivalente al numero de Tienda(13 digitos)",
            //    "valor" => $receptor -> numeroTienda
           // ];

            $apendiceWmtipo = [
                "campo" => "WMTIPO",
                "etiqueta" => "Bienes para venta(MERC),no para reventa(GASTO)",
                "valor" => "MERC"
            ];

            $apendice = [
                $apendiceUsuario,
           //     $apendiceSucursal
            ];
            if ($esCreditoFiscal){
                $apendice[] =  $apendiceWmoc;
                $apendice[] = $apendiceWmven;
             //   $apendice[] = $apendiceWmgln;
                $apendice[] = $apendiceWmtipo;
            }
            $unifiedData['apendice'] =  $apendice;
        // }
       
        $service = new  FacturaElectronicaServiceService();
        $response = $service->signDocument($unifiedData,$facturaId,$tipoDte,$factura->tipo_documento,$enContingencia,1);
        return response()->json($response);
    }

    private function numberToText($monto)
    {
        $xarray = array(
            0 => "Cero",
            1 => "UN", "DOS", "TRES", "CUATRO", "CINCO", "SEIS", "SIETE", "OCHO", "NUEVE",
            "DIEZ", "ONCE", "DOCE", "TRECE", "CATORCE", "QUINCE", "DIECISEIS", "DIECISIETE", "DIECIOCHO", "DIECINUEVE",
            "VEINTI", 30 => "TREINTA", 40 => "CUARENTA", 50 => "CINCUENTA", 60 => "SESENTA", 70 => "SETENTA", 80 => "OCHENTA", 90 => "NOVENTA",
            100 => "CIENTO", 200 => "DOSCIENTOS", 300 => "TRESCIENTOS", 400 => "CUATROCIENTOS", 500 => "QUINIENTOS", 600 => "SEISCIENTOS", 700 => "SETECIENTOS", 800 => "OCHOCIENTOS", 900 => "NOVECIENTOS"
        );
        //
        $xcifra = trim($monto);
        $xlength = strlen($xcifra);
        $xpos_punto = strpos($xcifra, ".");
        $xaux_int = $xcifra;
        $xdecimales = "00";
        if (!($xpos_punto === false)) {
            if ($xpos_punto == 0) {
                $xcifra = "0" . $xcifra;
                $xpos_punto = strpos($xcifra, ".");
            }
            $xaux_int = substr($xcifra, 0, $xpos_punto); // obtengo el entero de la cifra a covertir
            $xdecimales = substr($xcifra . "00", $xpos_punto + 1, 2); // obtengo los valores decimales
        }

        $XAUX = str_pad($xaux_int, 18, " ", STR_PAD_LEFT); // ajusto la longitud de la cifra, para que sea divisible por centenas de miles (grupos de 6)
        $xcadena = "";
        for ($xz = 0; $xz < 3; $xz++) {
            $xaux = substr($XAUX, $xz * 6, 6);
            $xi = 0;
            $xlimite = 6; // inicializo el contador de centenas xi y establezco el l�mite a 6 d�gitos en la parte entera
            $xexit = true; // bandera para controlar el ciclo del While
            while ($xexit) {
                if ($xi == $xlimite) { // si ya lleg� al l�mite m�ximo de enteros
                    break; // termina el ciclo
                }

                $x3digitos = ($xlimite - $xi) * -1; // comienzo con los tres primeros digitos de la cifra, comenzando por la izquierda
                $xaux = substr($xaux, $x3digitos, abs($x3digitos)); // obtengo la centena (los tres d�gitos)
                for ($xy = 1; $xy < 4; $xy++) { // ciclo para revisar centenas, decenas y unidades, en ese orden
                    switch ($xy) {
                        case 1: // checa las centenas
                            if (substr($xaux, 0, 3) < 100) { // si el grupo de tres d�gitos es menor a una centena ( < 99) no hace nada y pasa a revisar las decenas

                            } else {
                                $key = (int) substr($xaux, 0, 3);
                                if (TRUE === array_key_exists($key, $xarray)) {  // busco si la centena es n�mero redondo (100, 200, 300, 400, etc..)
                                    $xseek = $xarray[$key];
                                    $xsub = self::subfijo($xaux); // devuelve el self::subfijo correspondiente (Mill�n, Millones, Mil o nada)
                                    if (substr($xaux, 0, 3) == 100)
                                        $xcadena = " " . $xcadena . " CIEN " . $xsub;
                                    else
                                        $xcadena = " " . $xcadena . " " . $xseek . " " . $xsub;
                                    $xy = 3; // la centena fue redonda, entonces termino el ciclo del for y ya no reviso decenas ni unidades
                                } else { // entra aqu� si la centena no fue numero redondo (101, 253, 120, 980, etc.)
                                    $key = (int) substr($xaux, 0, 1) * 100;
                                    $xseek = $xarray[$key]; // toma el primer caracter de la centena y lo multiplica por cien y lo busca en el arreglo (para que busque 100,200,300, etc)
                                    $xcadena = " " . $xcadena . " " . $xseek;
                                } // ENDIF ($xseek)
                            } // ENDIF (substr($xaux, 0, 3) < 100)
                            break;
                        case 2: // checa las decenas (con la misma l�gica que las centenas)
                            if (substr($xaux, 1, 2) < 10) {
                            } else {
                                $key = (int) substr($xaux, 1, 2);
                                if (TRUE === array_key_exists($key, $xarray)) {
                                    $xseek = $xarray[$key];
                                    $xsub = self::subfijo($xaux);
                                    if (substr($xaux, 1, 2) == 20)
                                        $xcadena = " " . $xcadena . " VEINTE " . $xsub;
                                    else
                                        $xcadena = " " . $xcadena . " " . $xseek . " " . $xsub;
                                    $xy = 3;
                                } else {
                                    $key = (int) substr($xaux, 1, 1) * 10;
                                    $xseek = $xarray[$key];
                                    if (20 == substr($xaux, 1, 1) * 10)
                                        $xcadena = " " . $xcadena . " " . $xseek;
                                    else
                                        $xcadena = " " . $xcadena . " " . $xseek . " Y ";
                                } // ENDIF ($xseek)
                            } // ENDIF (substr($xaux, 1, 2) < 10)
                            break;
                        case 3: // checa las unidades
                            if (substr($xaux, 2, 1) < 1) { // si la unidad es cero, ya no hace nada

                            } else {
                                $key = (int) substr($xaux, 2, 1);
                                $xseek = $xarray[$key]; // obtengo directamente el valor de la unidad (del uno al nueve)
                                $xsub = self::subfijo($xaux);
                                $xcadena = " " . $xcadena . " " . $xseek . " " . $xsub;
                            } // ENDIF (substr($xaux, 2, 1) < 1)
                            break;
                    } // END SWITCH
                } // END FOR
                $xi = $xi + 3;
            } // ENDDO

            if (substr(trim($xcadena), -5, 5) == "ILLON") // si la cadena obtenida termina en MILLON o BILLON, entonces le agrega al final la conjuncion DE
                $xcadena .= " DE";

            if (substr(trim($xcadena), -7, 7) == "ILLONES") // si la cadena obtenida en MILLONES o BILLONES, entoncea le agrega al final la conjuncion DE
                $xcadena .= " DE";

            // ----------- esta l�nea la puedes cambiar de acuerdo a tus necesidades o a tu pa�s -------
            if (trim($xaux) != "") {
                switch ($xz) {
                    case 0:
                        if (trim(substr($XAUX, $xz * 6, 6)) == "1")
                            $xcadena .= "UN BILLON ";
                        else
                            $xcadena .= " BILLONES ";
                        break;
                    case 1:
                        if (trim(substr($XAUX, $xz * 6, 6)) == "1")
                            $xcadena .= "UN MILLON ";
                        else
                            $xcadena .= " MILLONES ";
                        break;
                    case 2:
                        if ($xcifra < 1) {
                            $xcadena = "CERO $xdecimales/100 USD ";
                        }
                        if ($xcifra >= 1 && $xcifra < 2) {
                            $xcadena = "UNO $xdecimales/100 USD ";
                        }
                        if ($xcifra >= 2) {
                            $xcadena .= " $xdecimales/100 USD"; //
                        }
                        break;
                } // endswitch ($xz)
            } // ENDIF (trim($xaux) != "")
            // ------------------      en este caso, para M�xico se usa esta leyenda     ----------------
            $xcadena = str_replace("VEINTI ", "VEINTI", $xcadena); // quito el espacio para el VEINTI, para que quede: VEINTICUATRO, VEINTIUN, VEINTIDOS, etc
            $xcadena = str_replace("  ", " ", $xcadena); // quito espacios dobles
            $xcadena = str_replace("UN UN", "UN", $xcadena); // quito la duplicidad
            $xcadena = str_replace("  ", " ", $xcadena); // quito espacios dobles
            $xcadena = str_replace("BILLON DE MILLONES", "BILLON DE", $xcadena); // corrigo la leyenda
            $xcadena = str_replace("BILLONES DE MILLONES", "BILLONES DE", $xcadena); // corrigo la leyenda
            $xcadena = str_replace("DE UN", "UN", $xcadena); // corrigo la leyenda
        } // ENDFOR ($xz)
        return trim($xcadena);
    }

    private function subfijo($xx)
    { // esta funci�n regresa un self::subfijo para la cifra
        $xx = trim($xx);
        $xstrlen = strlen($xx);
        if ($xstrlen == 1 || $xstrlen == 2 || $xstrlen == 3)
            $xsub = "";
        //
        if ($xstrlen == 4 || $xstrlen == 5 || $xstrlen == 6)
            $xsub = "MIL";
        //
        return $xsub;
    }

    public function downloadJSONFile(Request $request)
    {
        $idFactura = $request->idFactura;
        $factura = Venta::findOrFail($idFactura);
       
        // $tipoFactura = $request->tipoFactura;
        
        $fileName = $factura->codigoGeneracion.".json";
        // if ($tipoFactura == "03") { //CREDITO FISCAL
        //     $fileName = 'ccf' . $idFactura . '.json';
        // }else{
        //     $fileName = 'cf' . $idFactura . '.json';
        // }
        $storageName = 'jsonfacturas';
        $filePath = storage_path("app/public/uploads/{$storageName}/{$fileName}");
        // Check if the file exists
        if (Storage::disk($storageName)->exists($fileName)) {
            // Return a response with the file for download
            return response()->download($filePath, $fileName);
        } else {
            return response()->json(['error' => 'File not found.'], 404);
        }
    }

    public function createNumeroControl($idFactura,$tipoDte,$bodega){
        // Get the current date using Carbon
        $currentDate = Carbon::now();
        // Format the date as "dmy" (day-month-year) without separators
        $formattedDate = $currentDate->format('dmy');
        $puntoVenta =  Parameter::where('name', 'puntoVenta')->first();
        $idBodega = "";
        if (intval($bodega) > 9) {
            $idBodega = "".$bodega;
        }else{
            $idBodega = "0".$bodega;
        }
        $numeroControl = "DTE-".$tipoDte."-".$idBodega.$puntoVenta->value."-".str_pad($idFactura, 15, '0', STR_PAD_LEFT);
        return $numeroControl;
    }

    public function removeCharacterFromString($char,$value){
        if (empty(trim($value))) {
            return $value;
        } else {
            return str_replace($char, '', $value);
        }
    }

    public function vistaDescargaDTE($codigoGeneracion)
    {
        // $venta = Venta::where('codigoGeneracion', $codigoGeneracion)
        // ->with('ventaCliente.clienteGiro')
        // ->with('ventaUsuario')
        // ->firstOrFail();
        $log = FacturaElectronicaLog::where('codigoGeneracion', $codigoGeneracion)->where("serviceType","=","ENVIO")->first();
        // self::generatePDF($venta);
        // $service = new  FacturaElectronicaServiceService();
        // $service->sendEmailWithAttachments($codigoGeneracion);

        if (empty($log)){
            //NO HAY UN LOG DE ENVIO DE DOCUMENTO
            return view('layouts.descargaDTE', ['errorCode' =>"-1",'id' => $codigoGeneracion]);
        }else{
            //VERIFICAMOS SI EL ENVIO FUE EXITOSO
            $dataArray = json_decode($log->jsonResponse, true);
            $selloRecibido = $dataArray["selloRecibido"];
            if ($selloRecibido === null) {
                return view('layouts.descargaDTE',  ['errorCode' => "-2",'id' => $codigoGeneracion]);
            }else{
                $urlBase =  Parameter::where('name', 'urlBase')->first();
                return view('layouts.descargaDTE',  ['errorCode' => "0",'id' => $codigoGeneracion,'ruta' => $urlBase->value]);
            }
        } 
    }

    public function generatePDF($factura)
    {
        $nit =  Parameter::where('name', 'user')->first();
        $nombre =  Parameter::where('name', 'nombreEmisor')->first();
        $nrc =  Parameter::where('name', 'nrc')->first();
        $direccion =  Parameter::where('name', 'direccionEmisor')->first();
        $correo =  Parameter::where('name', 'correoEmisor')->first();
        $telefono =  Parameter::where('name', 'telefonoEmisor')->first();
        $urlBase =  Parameter::where('name', 'urlBase')->first();
        $ambiente =  Parameter::where('name', 'ambiente')->first();
        $log = FacturaElectronicaLog::where('codigoGeneracion', $factura->codigoGeneracion)->where("serviceType","=","ENVIO")->first();
        $fechaProcesamiento = "-";
        if (!empty($log)){
            $dataArray = json_decode($log->jsonResponse, true);
            $fechaProcesamiento = $dataArray["fhProcesamiento"];
        }
        $emisor = [
            'nit' => $nit->value,
            'nombre' => $nombre->value,
            'direccion' => $direccion->value,
            'correo' =>  $correo->value,
            'telefono' =>  $telefono->value,
            'nrc' =>  $nrc->value
        ];
        $detalleVenta = DB::select(
            "
        SELECT
            (@row_number:=@row_number + 1) AS numItem,
            articulos.tipo_producto AS tipoItem,
            articulos.codigo AS codigo,
            articulos.nombre AS descripcion,
            fc_factura_detalle.cantidad AS cantidad,
            unidadmedida.nombre AS uniMedida,
            articulos.cuenta_contable1 AS cotransFovial,
            fc_factura_detalle.precio_venta AS precioUni,
            fc_factura_detalle.sub_total_producto AS ventaGravada,
            fc_factura_detalle.ivaLinea AS ivaItem,
            fc_factura_detalle.servicio_conta AS servicio_conta,
            fc_factura_detalle.observaciones AS observaciones,
                fc_factura_detalle.codigoPlu  AS cplu,
            conta_cuentas_contables.codigo as codigoCuenta
            FROM (fc_factura
            JOIN fc_factura_detalle ON fc_factura.id = fc_factura_detalle.idFactura)
            LEFT JOIN articulos ON fc_factura_detalle.idProducto = articulos.id
            LEFT JOIN conta_cuentas_contables ON fc_factura_detalle.idProducto = conta_cuentas_contables.id
            LEFT JOIN unidadmedida ON articulos.umedida = unidadmedida.codigo
            WHERE fc_factura.id = " . $factura->id
        );
        $dataQrPropio = $urlBase->value.'/sercomfe/public/descargaDTE/'.$factura->codigoGeneracion;
        $carbonDate = Carbon::parse($factura->updated_at);
        $formattedDate = $carbonDate->format('Y-m-d');
        $data = "https://admin.factura.gob.sv/consultaPublica?ambiente=".$ambiente->value."&codGen=".$factura->codigoGeneracion."&fechaEmi=".$formattedDate;
        // Generar el código QR con Simple QrCode
        $qrCode = QrCode::size(100)->generate($data);
        $qrCodePropio = QrCode::size(70)->generate($dataQrPropio);
        $svgWithoutXMLDeclaration = preg_replace('/<\?xml.*\?>/', '', $qrCode);
        $svgWithoutXMLDeclarationPropio = preg_replace('/<\?xml.*\?>/', '', $qrCodePropio);
        // Crear una instancia de mPDF
        $mpdf = new Mpdf();
        $valores = [
            'svgWithoutXMLDeclaration' => $svgWithoutXMLDeclaration,
            'svgWithoutXMLDeclarationPropio' => $svgWithoutXMLDeclarationPropio,
            'factura' => $factura,
            'detalleVenta' => $detalleVenta,
            'totalLetras' =>  self::numberToText($factura->total_fc),
            'emisor' => $emisor,
            'fechaProcesamiento' => $fechaProcesamiento,
            'ambiente' => $ambiente->value
        ];
        // Crear el contenido HTML para el PDF
        $html = view('pdf.facturapdf',$valores)->render(); // Reemplaza 'pdf.example' con tu vista o contenido HTML

        // Configuración adicional de mPDF (si es necesario)
        // $mpdf->SetHeader('Header'); // Establecer encabezado
        // $mpdf->SetFooter('Footer'); // Establecer pie de página

        // Escribir el contenido HTML en mPDF
        $mpdf->WriteHTML($html);

        // Generar el PDF
        // $mpdf->Output(storage_path('app/public/mi_pdf_generado.pdf'), 'F');
        $pdfContent = $mpdf->Output('', \Mpdf\Output\Destination::STRING_RETURN);
        Storage::disk('pdfFacturas')->put($factura->codigoGeneracion.'.pdf', $pdfContent);

        // Devolver la ruta del archivo PDF guardado o realizar otras acciones necesarias
        return response()->json(['pdf_path' => storage_path('app/public/mi_pdf_generado.pdf')]);
    }

    public function downloadPublicJsonFile(Request $request){
        $codigoGeneracion = $request->codigoGeneracion;
        $venta = Venta::where('codigoGeneracion', $codigoGeneracion)->firstOrFail();
        $tipoFactura = $venta->tipo_documento;
        $fileName = $codigoGeneracion.".json";
        // if ($tipoFactura == "03") { //CREDITO FISCAL
        //     $fileName = 'ccf' . $venta->id . '.json';
        // }else{
        //     $fileName = 'cf' . $venta->id . '.json';
        // }
        $storageName = 'jsonfacturas';
        $filePath = storage_path("app/public/uploads/{$storageName}/{$fileName}");
        // Check if the file exists
        if (Storage::disk($storageName)->exists($fileName)) {
            // Return a response with the file for download
            return response()->download($filePath, $fileName);
        } else {
            return response()->json(['error' => 'File not found.'], 404);
        }
    }

    public function downloadPdfDocument(Request $request){
        $onlyView = $request->view;
        $isTicket = $request->ticket;
        if (!empty($isTicket) && !is_null($isTicket)){
            return self::generateTicket($request->codigoGeneracion);
        }else if (!empty($onlyView) && !is_null($onlyView)){
            // // Verifica si el archivo existe
            // if (Storage::disk('pdfFacturas')->exists($request->codigoGeneracion.'.pdf')) {
            //     // Obtén el contenido del archivo desde el almacenamiento
            //     $contenidoPDF =Storage::disk('pdfFacturas')->get($request->codigoGeneracion.'.pdf');

            //     // Retorna el contenido del PDF como respuesta HTTP con los encabezados adecuados
            //     return response()->make($contenidoPDF, 200, [
            //         'Content-Type' => 'application/pdf',
            //         'Content-Disposition' => 'inline; filename=' . $request->codigoGeneracion . '.pdf',
            //     ]);
            // } else {
            //     return response()->json(['error' => 'Archivo no encontrado'], 404);
            // }
            return self::generatePDFDocumentView($request->codigoGeneracion);
        }else{
            if (Storage::disk('pdfFacturas')->exists($request->codigoGeneracion.'.pdf')) {
                return Storage::disk('pdfFacturas')->download($request->codigoGeneracion.'.pdf');
             }else{
                 return redirect()->route('main');
             }
        }
    }

    public function generatePDFDocumentView($codigoGeneracion)
    {
        $factura = Venta::where('codigoGeneracion', $codigoGeneracion)
        ->with('ventaCliente.clienteGiro')
        ->with('ventaUsuario')
        ->with('sucursalCliente')
        ->firstOrFail();
        $nit =  Parameter::where('name', 'user')->first();
        $nombre =  Parameter::where('name', 'nombreEmisor')->first();
        $nrc =  Parameter::where('name', 'nrc')->first();
        $direccion =  Parameter::where('name', 'direccionEmisor')->first();
        $correo =  Parameter::where('name', 'correoEmisor')->first();
        $telefono =  Parameter::where('name', 'telefonoEmisor')->first();
        $urlBase =  Parameter::where('name', 'urlBase')->first();
        $ambiente =  Parameter::where('name', 'ambiente')->first();
        $log = FacturaElectronicaLog::where('codigoGeneracion', $factura->codigoGeneracion)->where("serviceType","=","ENVIO")->first();
        $fechaProcesamiento = "-";
        if (!empty($log)){
            $dataArray = json_decode($log->jsonResponse, true);
            $fechaProcesamiento = $dataArray["fhProcesamiento"];
        }
        $emisor = [
            'nit' => $nit->value,
            'nombre' => $nombre->value,
            'direccion' => $direccion->value,
            'correo' =>  $correo->value,
            'telefono' =>  $telefono->value,
            'nrc' =>  $nrc->value
        ];
        $detalleVenta = DB::select(
            "
        SELECT
            (@row_number:=@row_number + 1) AS numItem,
            articulos.tipo_producto AS tipoItem,
            articulos.codigo AS codigo,
            articulos.nombre AS descripcion,
            fc_factura_detalle.cantidad AS cantidad,
            unidadmedida.nombre AS uniMedida,
            articulos.cuenta_contable1 AS cotransFovial,
            fc_factura_detalle.precio_venta AS precioUni,
            fc_factura_detalle.sub_total_producto AS ventaGravada,
            fc_factura_detalle.ivaLinea AS ivaItem,
            fc_factura_detalle.servicio_conta AS servicio_conta,
            fc_factura_detalle.observaciones AS observaciones,
            fc_factura_detalle.codigoPlu  AS cplu,
            conta_cuentas_contables.codigo as codigoCuenta
            FROM (fc_factura
            JOIN fc_factura_detalle ON fc_factura.id = fc_factura_detalle.idFactura)
            LEFT JOIN articulos ON fc_factura_detalle.idProducto = articulos.id
            LEFT JOIN conta_cuentas_contables ON fc_factura_detalle.idProducto = conta_cuentas_contables.id
            LEFT JOIN unidadmedida ON articulos.umedida = unidadmedida.codigo
            WHERE fc_factura.id = " . $factura->id
        );
        
        $carbonDate = Carbon::parse($factura->updated_at);
        $formattedDate = $carbonDate->format('Y-m-d');
        $data = "https://admin.factura.gob.sv/consultaPublica?ambiente=".$ambiente->value."&codGen=".$factura->codigoGeneracion."&fechaEmi=".$formattedDate;
        $dataQrPropio = $urlBase->value.'/sercomfe/public/descargaDTE/'.$factura->codigoGeneracion;

        // Generar el código QR con Simple QrCode
        $qrCode = QrCode::size(100)->generate($data);
        $qrCodePropio = QrCode::size(70)->generate($dataQrPropio);
        $svgWithoutXMLDeclaration = preg_replace('/<\?xml.*\?>/', '', $qrCode);
        $svgWithoutXMLDeclarationPropio = preg_replace('/<\?xml.*\?>/', '', $qrCodePropio);
        // Crear una instancia de mPDF
        $mpdf = new Mpdf();
        $valores = [
            'svgWithoutXMLDeclaration' => $svgWithoutXMLDeclaration,
            'svgWithoutXMLDeclarationPropio' => $svgWithoutXMLDeclarationPropio,
            'factura' => $factura,
            'detalleVenta' => $detalleVenta,
            'totalLetras' =>  self::numberToText($factura->total_fc),
            'emisor' => $emisor,
            'fechaProcesamiento' => $fechaProcesamiento,
            'ambiente' => $ambiente->value
        ];
        // Crear el contenido HTML para el PDF
        $html = view('pdf.facturapdf',$valores)->render(); // Reemplaza 'pdf.example' con tu vista o contenido HTML

        // Configuración adicional de mPDF (si es necesario)
        // $mpdf->SetHeader('Header'); // Establecer encabezado
        // $mpdf->SetFooter('Footer'); // Establecer pie de página

        // Escribir el contenido HTML en mPDF
        $mpdf->WriteHTML($html);

         // Obtener el contenido del PDF
        $contenidoPDF = $mpdf->output();

        // Devolver el contenido del PDF como respuesta HTTP
        return response()->make($contenidoPDF, 200, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename='.$codigoGeneracion.'.pdf', // Puedes cambiar el nombre del archivo si lo deseas
        ]);
    }

    public function generateTicket($codigoGeneracion){
        $factura = Venta::where('codigoGeneracion', $codigoGeneracion)
        ->with('ventaCliente.clienteGiro')
        ->with('ventaUsuario')
        ->with('sucursalCliente')
        ->firstOrFail();

        if (is_null($factura->selloRecibido) || empty($factura->selloRecibido) || $factura->selloRecibido == "") {
            return response()->json(['error' => 'Ticket no disponible'], 404);
        }
        $nit =  Parameter::where('name', 'user')->first();
        $nombre =  Parameter::where('name', 'nombreEmisor')->first();
        $nrc =  Parameter::where('name', 'nrc')->first();
        $direccion =  Parameter::where('name', 'direccionEmisor')->first();
        $correo =  Parameter::where('name', 'correoEmisor')->first();
        $telefono =  Parameter::where('name', 'telefonoEmisor')->first();
        $urlBase =  Parameter::where('name', 'urlBase')->first();
        $ambiente =  Parameter::where('name', 'ambiente')->first();
        $log = FacturaElectronicaLog::where('codigoGeneracion', $factura->codigoGeneracion)->where("serviceType","=","ENVIO")->first();
        $fechaProcesamiento = "-";
        if (!empty($log)){
            $dataArray = json_decode($log->jsonResponse, true);
            $fechaProcesamiento = $dataArray["fhProcesamiento"];
        }
        $emisor = [
            'nit' => $nit->value,
            'nombre' => $nombre->value,
            'direccion' => $direccion->value,
            'correo' =>  $correo->value,
            'telefono' =>  $telefono->value,
            'nrc' =>  $nrc->value
        ];

        // $data = $urlBase->value.'/elupita/public/descargaDTE/'.$factura->codigoGeneracion;
        $carbonDate = Carbon::parse($factura->updated_at);
        $formattedDate = $carbonDate->format('Y-m-d');
        $data = "https://admin.factura.gob.sv/consultaPublica?ambiente=".$ambiente->value."&codGen=".$factura->codigoGeneracion."&fechaEmi=".$formattedDate;
        // Generar el código QR con Simple QrCode
        $qrCode = QrCode::size(100)->generate($data);
        $svgWithoutXMLDeclaration = preg_replace('/<\?xml.*\?>/', '', $qrCode);
        // Crear una instancia de mPDF
        $mpdf = new Mpdf();
        $valores = [
            'svgWithoutXMLDeclaration' => $svgWithoutXMLDeclaration,
            'factura' => $factura,
            'totalLetras' =>  self::numberToText($factura->total_fc),
            'emisor' => $emisor,
            'fechaProcesamiento' => $fechaProcesamiento,
            'ambiente' => $ambiente->value
        ];
        $mpdf = new Mpdf([
            'format' => [76.2, 279.4], // Tamaño de página personalizado de 3x11 pulgadas en milímetros
            'margin_left' => 5, // Margen izquierdo
            'margin_right' => 5, // Margen derecho
            'margin_top' => 5, // Margen superior
            'margin_bottom' => 5, // Margen inferior
        ]);
        $html = view('pdf.ticketpdf',$valores)->render();
         // Establecer las opciones del mPDF para el tamaño de página automático y margen cero
        // Establecer las opciones del mPDF para el tamaño de página automático y margen cero
        // $mpdf->SetAutoPageBreak(true, 0);
        // $mpdf->SetMargins(0, 0, 0); // Establecer el margen superior automático en cero
        $mpdf->WriteHTML($html);
         // Obtener el contenido del PDF
        $contenidoPDF = $mpdf->output();

        // Devolver el contenido del PDF como respuesta HTTP
        return response()->make($contenidoPDF, 200, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename=ticket.pdf', // Puedes cambiar el nombre del archivo si lo deseas
        ]);
    }

    public function anularFactura($factura, $cliente){
        $horaAnulacion = Carbon::now('America/El_Salvador')->format('H:i:s');
        $fechaAnulacion = Carbon::now('America/El_Salvador')->format('Y-m-d');
        $ambiente =  Parameter::where('name', 'ambiente')->first();
        $user =  Parameter::where('name', 'user')->first();
        $nombreEmisor =  Parameter::where('name', 'nombreEmisor')->first();
        // Parse the datetime string using Carbon
        $carbonDate = Carbon::parse($factura->updated_at);
        $codigoGeneracionAnulacion = strtoupper(Uuid::uuid4()->toString());
        $tipoDte = "01";
        if ($factura->tipo_documento == "03" || $factura->tipo_documento == "05" ) {
            $tipoDte = "03";
        }
        // Format the Carbon instance to get only the date
        $formattedDate = $carbonDate->format('Y-m-d');
        $identificacion = [
            'version' => 2,
            'ambiente' => $ambiente->value,
            'codigoGeneracion' => $codigoGeneracionAnulacion,
            'fecAnula' => $fechaAnulacion,
            'horAnula' => $horaAnulacion
        ];

        $emisor = [
            'nit' => $user->value,
            'nombre' => $nombreEmisor->value,
            'tipoEstablecimiento' => "02",
            'nomEstablecimiento' => $nombreEmisor->value,
            'telefono' => "26385555",
            'correo' => "pumadonbosco@gmail.com",
            'codEstableMH' => "null",
            'codEstable' => "null",
            'codPuntoVentaMH' => "null",
            'codPuntoVenta' => "null"
        ];

        $documento = [
            'tipoDte' => $factura->tipo_documento,
            'codigoGeneracion' => $factura->codigoGeneracion,
            'selloRecibido' => $factura->selloRecibido,
            'numeroControl' => $factura->numeroControl,
            'fecEmi' => $formattedDate,
            'montoIva' => $factura->iva_fc,
            'codigoGeneracionR' => null,
            'nombre' =>  $cliente->nombre,
            'telefono' =>  $cliente->telContacto,
            'correo' =>  $cliente->email,
        ];

        if ($factura->tipo_documento == "03" || $factura->tipo_documento == "05" ) {
            $documento["tipoDocumento"] = "36";
            $documento["numDocumento"] = self::removeCharacterFromString("-",$cliente->nit);
        }else if ($factura->tipo_documento == "11") {
            $documento["tipoDocumento"] = "37";
            $documento["numDocumento"] = self::removeCharacterFromString("-",$cliente->dui);
        }else  {
            $documento["tipoDocumento"] = "13";
            $documento["numDocumento"] = self::removeCharacterFromString("-",$cliente->dui);
        }

        $motivo = [
            'tipoAnulacion' => 2,
            'motivoAnulacion' => "Rescindir de la operación realizada",
            'nombreResponsable' => "christopher monterroza",
            'tipDocResponsable' => "13",
            'numDocResponsable' => "010490346",
            'nombreSolicita' => $cliente->nombre,
            'tipDocSolicita' => "36",
            'numDocSolicita' => self::removeCharacterFromString("-",$cliente->nit),
        ];

        $service = new  FacturaElectronicaServiceService();
        $unifiedData = [
            'identificacion' => $identificacion,
            'emisor' => $emisor,
            'documento' => $documento,
            "motivo" => $motivo
        ];
        $response = $service->signDocument($unifiedData,$factura->id,$tipoDte,$factura->tipo_documento,0,2);
        return response()->json($response);
    }

     //CATALOGOS
     public function selectIncoTerms(Request $request){
        if (!$request->ajax()) return redirect('/');
        $incoterms = Incoterm::select('id','cod_intcoter','incoterms')->orderBy('cod_intcoter', 'asc')->get();
        return ['incoterms' => $incoterms];
    }

    public function selectPaises(Request $request){
        if (!$request->ajax()) return redirect('/');
        $paises = Pais::select('id','cod_pais','pais')->orderBy('cod_pais', 'asc')->get();
        return ['paises' => $paises];
    }

    public function selectRegimen(Request $request){
        if (!$request->ajax()) return redirect('/');
        $regimenes = RegimenExportacion::select('id','cod_regimen','regimen')->orderBy('cod_regimen', 'asc')->get();
        return ['regimenes' => $regimenes];
    }

    public function selectRecintoFiscal(Request $request){
        if (!$request->ajax()) return redirect('/');
        $recintos = RecintoFiscal::select('id','cod_rfiscal','recintofiscal')->orderBy('cod_rfiscal', 'asc')->get();
        return ['recintos' => $recintos];
    }

    public function selectTipoDeDocumentoDte(Request $request){
        if (!$request->ajax()) return redirect('/');
        $tiposDocumentoDte = TipoDocumentoDTE::select('id','codigo','tipo_documento')->orderBy('codigo', 'asc')->get();
        return ['tiposDocumentoDte' => $tiposDocumentoDte];
    }
    
}
