<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request; 
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use App\Ingreso;
use App\DetalleIngreso;
use App\ExistenciaProducto;

class IngresoController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
          if ($buscar==''){
            $ingresos = Ingreso::where('idEmpresa','=', Auth::user()->idEmpresa)->with('proveedor')->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $compras = Ingreso::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->with('proveedor')->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $ingresos->total(),
                'current_page' => $ingresos->currentPage(),
                'per_page'     => $ingresos->perPage(),
                'last_page'    => $ingresos->lastPage(),
                'from'         => $ingresos->firstItem(),
                'to'           => $ingresos->lastItem(),
            ],
            'ingresos' => $ingresos
        ];
    }


     public function indexExistencia(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
          if ($buscar==''){
            $existencias = ExistenciaProducto::join('articulos','in_exis_productos.idProducto','=','articulos.id')
            ->join('categorias','articulos.idcategoria','=','categorias.id')
            ->join('in_bodega','in_exis_productos.idBodega','=','in_bodega.id')
            ->select('articulos.codigo as codigoArticulo','articulos.nombre as nombreArticulo','categorias.nombre as nombreCategoria','in_exis_productos.existencia','in_bodega.nombre as nombreBodega')
             ->where('articulos.idEmpresa','=', Auth::user()->idEmpresa)
            ->orderBy('articulos.codigo', 'asc')->paginate(10);
        }
        else{
            $existencias = ExistenciaProducto::join('articulos','in_exis_productos.idProducto','=','articulos.id')
            ->join('categorias','articulos.idcategoria','=','categorias.id')
            ->join('in_bodega','in_exis_productos.idBodega','=','in_bodega.id')
            ->select('articulos.codigo as codigoArticulo','articulos.nombre as nombreArticulo','categorias.nombre as nombreCategoria','in_exis_productos.existencia','in_bodega.nombre as nombreBodega')
            ->where('articulos.idEmpresa','=', Auth::user()->idEmpresa)
            ->where("articulos.".$criterio, 'like', '%'. $buscar . '%')
            ->orderBy('articulos.codigo', 'asc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $existencias->total(),
                'current_page' => $existencias->currentPage(),
                'per_page'     => $existencias->perPage(),
                'last_page'    => $existencias->lastPage(),
                'from'         => $existencias->firstItem(),
                'to'           => $existencias->lastItem(),
            ],
            'existencias' => $existencias
        ];
    }


    public function listarPdf(){
        $idEmpresa = Auth::user()->idEmpresa;
        $ingresos = Ingreso::join('personas','ingresos.idproveedor','=','personas.id')
            ->join('users','ingresos.idusuario','=','users.id')
            ->select('ingresos.id','ingresos.tipo_comprobante','ingresos.serie_comprobante',
            'ingresos.num_comprobante','ingresos.fecha_hora','ingresos.impuesto','ingresos.total',
            'ingresos.estado','personas.nombre','users.usuario')
             ->where('ingresos.idEmpresa','=', $idEmpresa)
            ->orderBy('ingresos.id', 'asc')->get();
        //$cont=Ingreso::count();
        $cont=sizeof($ingresos);

        $pdf = \PDF::loadView('pdf.ingresospdf',['ingresos'=>$ingresos,'cont'=>$cont])->setPaper('a4', 'landscape');
        return $pdf->download('ingresos.pdf');
    }

    public function pdf(Request $request,$id){
        $ingreso = Ingreso::join('personas','ingresos.idproveedor','=','personas.id')
        ->join('users','ingresos.idusuario','=','users.id')
        ->select('ingresos.id','ingresos.tipo_comprobante','ingresos.serie_comprobante',
        'ingresos.num_comprobante','ingresos.created_at','ingresos.impuesto','ingresos.total',
        'ingresos.estado','personas.nombre','personas.tipo_documento','personas.num_documento',
        'personas.direccion','personas.email',
        'personas.telefono','users.usuario')
        ->where('ingresos.id','=',$id)
        ->orderBy('ingresos.id', 'desc')->take(1)->get();

        $detalles = DetalleIngreso::join('articulos','detalle_ingresos.idarticulo','=','articulos.id')
        ->select('detalle_ingresos.cantidad','detalle_ingresos.precio',
        'articulos.nombre as articulo')
        ->where('detalle_ingresos.idingreso','=',$id)
        ->orderBy('detalle_ingresos.id', 'desc')->get();

        $numingreso=Ingreso::select('num_comprobante')->where('id',$id)->get();

        $pdf = \PDF::loadView('pdf.ingreso',['ingreso'=>$ingreso,'detalles'=>$detalles]);
        return $pdf->download('ingreso-'.$numingreso[0]->num_comprobante.'.pdf');
    }

    public function obtenerCabecera(Request $request){
        if (!$request->ajax()) return redirect('/');

        $id = $request->id;
        $ingreso = Ingreso::join('personas','ingresos.idproveedor','=','personas.id')
        ->join('users','ingresos.idusuario','=','users.id')
        ->select('ingresos.id','ingresos.tipo_comprobante','ingresos.serie_comprobante',
        'ingresos.num_comprobante','ingresos.fecha_hora','ingresos.impuesto','ingresos.total',
        'ingresos.estado','personas.nombre','users.usuario')
        ->where('ingresos.id','=',$id)
        ->orderBy('ingresos.id', 'desc')->take(1)->get();
        
        return ['ingreso' => $ingreso];
    }

    public function obtenerDetalles(Request $request){
        if (!$request->ajax()) return redirect('/');

        $id = $request->id;
        $detalles = DetalleIngreso::join('articulos','detalle_ingresos.idarticulo','=','articulos.id')
        ->select('detalle_ingresos.cantidad','detalle_ingresos.precio','articulos.nombre as articulo')
        ->where('detalle_ingresos.idingreso','=',$id)
        ->orderBy('detalle_ingresos.id', 'desc')->get();
        
        return ['detalles' => $detalles];
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        $fecha1 = explode('/',$request->fechaOrden);
        $fecha1 = $fecha1[2].'-'.$fecha1[1].'-'.$fecha1[0];

        $fecha2 = explode('/',$request->fechaLiva);
        $fecha2 = $fecha2[2].'-'.$fecha2[1].'-'.$fecha2[0];

        $fecha3 = explode('/',$request->fechaVencimiento);
        $fecha3 = $fecha3[2].'-'.$fecha3[1].'-'.$fecha3[0];
         
        try{
            DB::beginTransaction();

           // $mytime= Carbon::now('America/Lima');

            $ingreso = new Ingreso();
            $ingreso->idEmpresa = $idEmpresa;
            $ingreso->idBodega = $request->idBodega;
            $ingreso->tipo_compra = $request->tipoCompra;
            $ingreso->clase_documento = $request->claseDocumento;
            $ingreso->tipo_documento = $request->tipoDocumento;
            $ingreso->num_documento = $request->numeroDocumento;
            $ingreso->numreso = $request->numeroResolucion;
            $ingreso->numseriedoc = $request->numeroSerie;
            $ingreso->numfunico = $request->numFunico;
            $ingreso->fecha_orden = $fecha1;
            $ingreso->fecha_liva = $fecha2;
            $ingreso->idProveedor = $request->idProveedor;
            $ingreso->condicion_compra = $request->condicionCompra;
            $ingreso->dias_credito = $request->diasCredito;
            $ingreso->fecha_vencimiento =  $fecha3;
            $ingreso->ver_libro = $request->verLiva;
            $ingreso->concepto = $request->detalleCompra;
            $ingreso->anulada =$request->anulada;
            $ingreso->contabilizada = $request->contabilizada;
            $ingreso->cext_locales = $request->subExcenta;
            $ingreso->csexcluidoesp =  $request->subExcluido;
            $ingreso->sub_total_oc = $request->subEGravada;
            $ingreso->iva_oc = $request->iva;
            $ingreso->ret_per1 =$request->retPer;
            $ingreso->total_oc = $request->total;
            $ingreso->userId = Auth::user()->id;
            $ingreso->num_orden = $request->numOrden;
            $ingreso->save();


            $detalles = $request->detalle;//Array de detalles
            //Recorro todos los elementos

            foreach($detalles as $ep=>$det)
            {
                $detalle = new DetalleIngreso();
                $detalle->id_ocompra = $ingreso->id;
                $detalle->idproducto = $det['idProducto'];
                $detalle->cantidad = $det['cantidad'];
                $detalle->precio_costo = $det['precio']; 
                $detalle->subtotal = $det['total'];
                $detalle->observaciones = $det['observaciones'];
                if ($det['tipo'] == 2){ //sin orden de compra , cuenta contable
                    $detalle->cuenta_contable = 1;
                }else{
                    $detalle->cuenta_contable = 0;
                }          
                $detalle->save();

                if ($det['tipo'] ==1 ){ //PRODUCTO
                    $existenciaProducto = ExistenciaProducto::where('idProducto','=',$det['idProducto'])->where('idBodega','=',$request->idBodega)->first();
                    if (is_null($existenciaProducto)) {
                        $existencia = new ExistenciaProducto();
                        $existencia->idProducto = $det['idProducto'];
                        $existencia->idBodega = $request->idBodega;
                        if ($request->tipoCompra == 3){
                            $existencia->existencia_transito = $det['cantidad'];
                        }else{
                            $existencia->existencia = $det['cantidad'];
                            $existencia->costo_promedio = $det['precio'];
                        }         
                        $existencia->save();     
                    }else{
                        $existenciaNueva = $existenciaProducto['existencia'] + $det['cantidad'];
                        $totalActual = $existenciaProducto['existencia'] * $existenciaProducto['costo_promedio'];
                        $totalNuevo = $totalActual + ($det['cantidad'] * $det['precio']);
                        $costoNuevo = $totalNuevo/$existenciaNueva;
                        $existencia = ExistenciaProducto::findOrFail($existenciaProducto['id']);
                        if ($request->tipoCompra == 3){
                            $existencia->existencia_transito = $det['cantidad'];
                        }else{
                            $existencia->existencia = $existenciaNueva;
                            $existencia->costo_promedio = number_format($costoNuevo,2);
                        }  
                        $existencia->save();
                    }
                }
            }
            DB::commit();
            return [
                'id' => $ingreso->id
            ];
        } catch (Exception $e){
            DB::rollBack();
        }
    }

    public function anularOrden(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        DB::beginTransaction();
        $orden = Ingreso::findOrFail($request->idOrden);
        $orden->sub_total_oc = 0;
        $orden->iva_oc = 0;
        $orden->total_oc = 0;
        $orden->saldo_oc = 0;
        $orden->anulada = 1;
        $orden->save();

        //REGRESANDO PRODUCTO AL INVENTARIO
        $detalleOrden = DetalleIngreso::where("id_ocompra","=",$orden->id)->get();
        foreach($detalleOrden as $ep=>$det)
         {  
            $existencia = ExistenciaProducto::select('id','existencia_transito')
            ->where("idProducto", '=' ,$det['idproducto'])
            ->where("idBodega", '=' ,$orden->idbodega)
            ->first();
            
            $objExistencia = ExistenciaProducto::findOrFail($existencia['id']);
            $objExistencia->existencia_transito  = 0;
            $objExistencia->save();
         }
         DB::commit();
    }

    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $ingreso = Ingreso::findOrFail($request->id);
        $ingreso->estado = 'Anulado';
        $ingreso->save();
    }
    public function selectNumResolucion(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = Ingreso::select('numreso')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numreso', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numreso', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function selectNumSerie(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = Ingreso::select('numseriedoc')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numseriedoc', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numseriedoc', 'asc')->get();
        return ['numeros' => $numeros];
    }
}
