<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Banco;
use App\CuentaBancaria;
use App\CuentaContable;
use App\OperacionBancaria;
use App\PartidaContableDetalle;
use Illuminate\Support\Facades\DB;

class OperacionBancariaController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        // $rolUsuario = Auth::user()->idrol;
        // $idEmpresa = Auth::user()->idEmpresa;
        if ($buscar==''){
            $bancos = Banco::orderBy('id', 'desc')->paginate(10);
        }
        else{
            $bancos = Banco::where($criterio, 'like', '%'. $buscar . '%')->orderBy('id', 'desc')->paginate(10);
        }
        
        

        return [
            'pagination' => [
                'total'        => $bancos->total(),
                'current_page' => $bancos->currentPage(),
                'per_page'     => $bancos->perPage(),
                'last_page'    => $bancos->lastPage(),
                'from'         => $bancos->firstItem(),
                'to'           => $bancos->lastItem(),
            ],
            'bancos' => $bancos
        ];
    }

    // public function selectCategoria(Request $request){
    //     if (!$request->ajax()) return redirect('/');
    //     $idEmpresa = Auth::user()->idEmpresa;
    //     $categorias = Categoria::where('condicion','=','1')->where('idEmpresa','=',$idEmpresa)
    //     ->select('id','nombre')->orderBy('nombre', 'asc')->get();
    //     return ['categorias' => $categorias];
    // }

    // public function listarPdf(){
    //     $idEmpresa = Auth::user()->idEmpresa;
    //     $categorias = Categoria::select('nombre','descripcion','condicion')->where('idEmpresa','=', $idEmpresa)->orderBy('nombre', 'asc')->get();
    //     $cont=Categoria::count();

    //     $pdf = \PDF::loadView('pdf.categoriaspdf',['categorias'=>$categorias,'cont'=>$cont])->setPaper('a4', 'portrait');
    //     return $pdf->download('categorias.pdf');
    // }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        // $rolUsuario = Auth::user()->idrol;
        // $idEmpresa = Auth::user()->idEmpresa;
        $banco = new Banco();
        $banco->cod_banco = $request->codigo;
        $banco->nombre_banco = $request->nombre;
        $banco->save();
    }
  

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $banco = banco::findOrFail($request->id);
        $banco->cod_banco = $request->codigo;
        $banco->nombre_banco = $request->nombre;
        $banco->save();
    }
    public function delete(Request $request){
        if (!$request->ajax()) return redirect('/');
        $banco = Banco::findOrFail($request->id);
        $banco->delete();
    }


    //=================CUENTAS BANCARIAS ================================================//
    public function indexCuentaBancaria(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        if ($buscar==''){
            $cuentas = CuentaBancaria::orderBy('id', 'desc')->with('banco')->with('cuentaContable')->paginate(10);
        }
        else{
            $cuentas = CuentaBancaria::where($criterio, 'like', '%'. $buscar . '%')->with('banco')->with('cuentaContable')->orderBy('id', 'desc')->paginate(10);
        }
        
        

        return [
            'pagination' => [
                'total'        => $cuentas->total(),
                'current_page' => $cuentas->currentPage(),
                'per_page'     => $cuentas->perPage(),
                'last_page'    => $cuentas->lastPage(),
                'from'         => $cuentas->firstItem(),
                'to'           => $cuentas->lastItem(),
            ],
            'cuentas' => $cuentas
        ];
    }

    public function storeCuentaBancaria(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $cuenta = new CuentaBancaria();
        $cuenta->idBanco = $request->idBanco;
        $cuenta->idCuentaContable = $request->idCuentaContable;
        $cuenta->num_cuenta_banco = $request->numeroCuenta;
        $cuenta->nombre_cuenta = $request->nombreCuenta;
        $cuenta->save();
    }

    public function updateCuentaBancaria(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $cuenta = CuentaBancaria::findOrFail($request->id);
        $cuenta->idBanco = $request->idBanco;
        $cuenta->idCuentaContable = $request->idCuentaContable;
        $cuenta->num_cuenta_banco = $request->numeroCuenta;
        $cuenta->nombre_cuenta = $request->nombreCuenta;
        $cuenta->save();
    }

    public function selectBanco(Request $request){
        if (!$request->ajax()) return redirect('/');
        $bancos = Banco::select('id','cod_banco','nombre_banco')
        ->orderBy('cod_banco', 'asc')->get();
        return ['bancos' => $bancos];
    }
    public function selectCuenta(Request $request){
        if (!$request->ajax()) return redirect('/');
        $cuentas = CuentaBancaria::join('banco','cuenta_banco.idBanco','=','banco.id')
        ->select(
            'cuenta_banco.id as value',
            DB::raw("CONCAT('[',banco.nombre_banco,'] ',cuenta_banco.nombre_cuenta, ' - ', cuenta_banco.num_cuenta_banco) as text"),
            'num_cuenta_banco'
            )
        ->orderBy('cuenta_banco.id', 'asc')->get();
        return ['cuentas' => $cuentas];
    }

    public function deleteCuentaBancaria(Request $request){
        if (!$request->ajax()) return redirect('/');
        $cuenta = CuentaBancaria::findOrFail($request->id);
        $cuenta->delete();
    }
    //===================================================================================//

    //======================OPERACIONES BANCARIAS=======================================//
    public function indexOperacionBancaria(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        if ($buscar==''){
            $operaciones = OperacionBancaria::orderBy('id', 'asc')->with("cuentaBancaria.banco")->with("cliente")->paginate(10);
        }
        else{
            $operaciones = OperacionBancaria::where($criterio, 'like', '%'. $buscar . '%')->with("cuentaBancaria")->with("cliente")->orderBy('id', 'asc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $operaciones->total(),
                'current_page' => $operaciones->currentPage(),
                'per_page'     => $operaciones->perPage(),
                'last_page'    => $operaciones->lastPage(),
                'from'         => $operaciones->firstItem(),
                'to'           => $operaciones->lastItem(),
            ],
            'operaciones' => $operaciones
        ];
    }
    public function storeOperacionBancaria(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        // $rolUsuario = Auth::user()->idrol;
        // $idEmpresa = Auth::user()->idEmpresa;

        try{
            DB::beginTransaction();
            $operacion = new OperacionBancaria();
            $operacion->idCuenta = $request->idCuenta;
            $operacion->id_cliente = $request->idCliente;
            $operacion->tipo_ingreso = $request->tipoIngreso;
            $operacion->tipo_movimiento = $request->tipoMovimiento;
            $operacion->num_movimiento = $request->numMovimiento;
            // $operacion->fecha_movimiento = $request->fechaMovimiento;
            $movimientosDebito = ["CH", "ND"];

// Validación de entrada
if (!isset($request->cantidadMov) || !is_numeric($request->cantidadMov)) {
    throw new \Exception("El valor de cantidadMov debe ser un número válido.");
}

$cantidadMov = number_format($request->cantidadMov, 2, '.', '');

// Lógica principal con el mapeo
if (in_array($request->tipoIngreso, $movimientosDebito) || in_array($request->tipoMovimiento, $movimientosDebito)) {
    $operacion->valor_debito = $cantidadMov;
} else {
    $operacion->valor_credito = $cantidadMov;
}
            $operacion->fecha_movimiento =  date("Y-m-d");
            $operacion->desc_movimiento = $request->descripcionMovimiento;
            
            $operacion->usuario = "-";
            $operacion->anulado = 0;
            $operacion->ordenDe = $request->ordenDe;
            $operacion->save();


            $detalles = $request->detalle;//Array de detalles
            //Recorro todos los elementos

            foreach($detalles as $ep=>$det)
            {
                $detalle = new PartidaContableDetalle();
                $detalle->id_partida = $operacion->id;
                $detalle->id_cuenta = $det['idCuenta'];
                $detalle->descripcion_linea = $det['concepto'];
                $detalle->tipo_referencia = $det['tipo'];
                $detalle->num_referencia_str = $det['numero'];
                $detalle->monto_debe = $det['debe'];
                $detalle->monto_haber = $det['haber'];            
                $detalle->save();
            }
            
            DB::commit();
            return 0;
        } catch (Exception $e){
            DB::rollBack();
            return 1;
        }
    }

    public function anularOperacion(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $operacion = OperacionBancaria::findOrFail($request->idOperacion);
        $operacion->anulado = 1;
        $operacion->save();
    }

    public function getCuentaContableByCuenta(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $cuenta = CuentaBancaria::findOrFail($request->idCuenta);
        $cuentaContable = CuentaContable::findOrFail($cuenta->idCuentaContable);
        return [
            'cuentaContable' => $cuentaContable
        ];

        
    }
    //=================================================================================//
}
