<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\RegistroOdontologico;
use App\CallCenterModel;
use App\Cliente;
use App\SalaPaciente;
use App\Ortodoncia;
use App\OrdenTrabajo;
use App\Odontograma;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;

class PacienteController extends Controller
{
    public function indexCallCenter(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $calls = CallCenterModel::where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $calls = CallCenterModel::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $calls->total(),
                'current_page' => $calls->currentPage(),
                'per_page'     => $calls->perPage(),
                'last_page'    => $calls->lastPage(),
                'from'         => $calls->firstItem(),
                'to'           => $calls->lastItem(),
            ],
            'calls' => $calls
        ];
    }

    public function storeCallCenter(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = new CallCenterModel();
        $call->idPaciente = $request->idPaciente;
        $call->idUser = Auth::user()->id;
        $call->comentario = $request->comentario;
        $call->save();
    }

    public function updateCallCenter(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = CallCenterModel::findOrFail($request->id);
        $call->comentario = $request->comentario;
        $call->save();
    }

    public function eliminarCallCenter(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $call = CallCenterModel::findOrFail($request->id);
        $call->delete();
    }

    public function getPaciente(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $paciente = Cliente::findOrFail($request->id);
        return [
            'paciente' => $paciente
        ];
    }

    public function indexRegistroOdonto(Request $request)
{
   if (!$request->ajax()) return redirect('/');

    $buscar = $request->buscar;
    $criterio = $request->criterio;
    
    if ($buscar==''){
        $registros = RegistroOdontologico::where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
    }
    else{
        $registros = RegistroOdontologico::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
    }
    
    return [
        'pagination' => [
            'total'        => $registros->total(),
            'current_page' => $registros->currentPage(),
            'per_page'     => $registros->perPage(),
            'last_page'    => $registros->lastPage(),
            'from'         => $registros->firstItem(),
            'to'           => $registros->lastItem(),
        ],
        'registros' => $registros
    ];
}
public function storeRegistroOdonto(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $registro = new RegistroOdontologico();
    $registro->idPaciente = $request->idPaciente;
    $registro->idUser = Auth::user()->id;
    $registro->comentario = $request->comentario;
    $registro->save();
}

public function updateRegistroOdonto(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $registro = RegistroOdontologico::findOrFail($request->id);
    $registro->comentario = $request->comentario;
    $registro->save();
}

public function eliminarRegistroOdonto(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $registro = RegistroOdontologico::findOrFail($request->id);
    $registro->delete();
}

public function indexSalas(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $salas = SalaPaciente::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{
            $salas = SalaPaciente::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        return [
            'pagination' => [
                'total'        => $salas->total(),
                'current_page' => $salas->currentPage(),
                'per_page'     => $salas->perPage(),
                'last_page'    => $salas->lastPage(),
                'from'         => $salas->firstItem(),
                'to'           => $salas->lastItem(),
            ],
            'salas' => $salas
        ];
    }


    
    public function storeSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = new SalaPaciente();
        $sala->nombre = $request->nombre;
        $sala->descripcion = $request->descripcion;
        $sala->idEmpresa = Auth::user()->idEmpresa;
        $sala->estado = "1";
        $sala->save();
    }

    public function updateSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = SalaPaciente::findOrFail($request->id);
        $sala->nombre = $request->nombre;
        $sala->descripcion = $request->descripcion;
        $sala->save();
    }
    public function desactivarSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = SalaPaciente::findOrFail($request->id);
        $sala->estado = 0;
        $sala->save();
    }

    public function activarSalas(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sala = SalaPaciente::findOrFail($request->id);
        $sala->estado = 1;
        $sala->save();
    }

     public function selectSalas(Request $request){
        if (!$request->ajax()) return redirect('/');
        $salas = SalaPaciente::select('id','nombre','descripcion')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('id', 'asc')->get();
        return ['salas' => $salas];
    }

    public function eliminarSalas(Request $request)
{
    if (!$request->ajax()) return redirect('/');
    $sala = SalaPaciente::findOrFail($request->id);
    $sala->delete();
}

//ORTODONCIA

public function indexOrtodoncia(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $paciente = $request->idPaciente;
        $fromMain = $request->fromMain;
        
        if ($buscar==''){
            if ($fromMain == 1){
                $ortodoncia = Ortodoncia::orderBy('created_at', 'ASC')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->with('ortodonciaUsuario')->paginate(10);
            }else{
                $ortodoncia = Ortodoncia::orderBy('created_at', 'ASC')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->where('anulada','=', 0)->with('ortodonciaUsuario')->paginate(10);
            }
            
        }
        else{
            if ($fromMain == 1){
                $ortodoncia = Ortodoncia::where($criterio, 'like', '%'. $buscar . '%')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->with('ortodonciaUsuario')->orderBy('id', 'desc')->paginate(10);
            }else{  
                $ortodoncia = Ortodoncia::where($criterio, 'like', '%'. $buscar . '%')->where('id_empresa','=', Auth::user()->idEmpresa)->where('id_paciente','=', $paciente)->where('anulada','=', 0)->with('ortodonciaUsuario')->orderBy('id', 'desc')->paginate(10);
            }
           
        }
        

        return [
            'pagination' => [
                'total'        => $ortodoncia->total(),
                'current_page' => $ortodoncia->currentPage(),
                'per_page'     => $ortodoncia->perPage(),
                'last_page'    => $ortodoncia->lastPage(),
                'from'         => $ortodoncia->firstItem(),
                'to'           => $ortodoncia->lastItem(),
            ],
            'ortodoncia' => $ortodoncia
        ];
    }
    public function storeOrtodoncia(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $orto = new Ortodoncia();

        $orto->id_empresa = Auth::user()->idEmpresa;
        $orto->id_paciente = $request->id_paciente;
        $orto->arniti_12_sup = $request->arniti_12_sup;
        $orto->arniti_12_inf = $request->arniti_12_inf;

        $orto->arniti_14_sup = $request->arniti_14_sup;
        $orto->arniti_14_inf = $request->arniti_14_inf;
        $orto->arniti_16_sup = $request->arniti_16_sup;
        $orto->arniti_16_inf = $request->arniti_16_inf;

        $orto->arniti_18_sup = $request->arniti_18_sup;
        $orto->arniti_18_inf = $request->arniti_18_inf;
        $orto->arace_18_sup = $request->arace_18_sup;
        $orto->arace_18_inf = $request->arace_18_inf;

        $orto->arace_18_ctsup = $request->arace_18_ctsup;
        $orto->arace_18_ctinf = $request->arace_18_ctinf;
        $orto->ar_16x_niti = $request->ar_16x_niti;
        $orto->ar_16x_acero = $request->ar_16x_acero;

        $orto->ar_16x_sup = $request->ar_16x_sup;
        $orto->ar_16x_inf = $request->ar_16x_inf;
        $orto->ar_18x_niti = $request->ar_18x_niti;

        $orto->ar_18x_acero = $request->ar_18x_acero;
        $orto->ar_18x_sup = $request->ar_18x_sup;
        $orto->ar_18x_inf = $request->ar_18x_inf;
        $orto->dista_cuadra1 = $request->dista_cuadra1;

        $orto->dista_cuadra2 = $request->dista_cuadra2;
        $orto->dista_cuadra3 = $request->dista_cuadra3;
        $orto->dista_cuadra4 = $request->dista_cuadra4;
        $orto->cie_ccadena_sup = $request->cie_ccadena_sup;

        $orto->cie_ccadena_inf = $request->cie_ccadena_inf;
        $orto->cadena_de = $request->cadena_de;
        $orto->resorte_entre = $request->resorte_entre;
        $orto->cad_trac_pie_ret = $request->cad_trac_pie_ret;

        $orto->tec_arcsobr_sup = $request->tec_arcsobr_sup;
        $orto->tec_arcsobr_inf = $request->tec_arcsobr_inf;
        $orto->ela_intermax_cla2 = $request->ela_intermax_cla2;
        $orto->ela_intermax_cla3 = $request->ela_intermax_cla3;

        $orto->ela_intermax_caja = $request->ela_intermax_caja;
        $orto->ela_intermax_trian = $request->ela_intermax_trian;
        $orto->ela_intermax_dboton = $request->ela_intermax_dboton;
        $orto->feru_comp_sup = $request->feru_comp_sup;

        $orto->feru_comp_inf = $request->feru_comp_inf;
        $orto->ferubloq_antsup = $request->ferubloq_antsup;
        $orto->ferubloq_antinf = $request->ferubloq_antinf;
        $orto->ferubloq_cuadr1 = $request->ferubloq_cuadr1;
    
        $orto->ferubloq_cuadr2 = $request->ferubloq_cuadr2;
        $orto->ferubloq_cuadr3 = $request->ferubloq_cuadr3;
        $orto->ferubloq_cuadr4 = $request->ferubloq_cuadr4;
        
        $orto->incobrak_antsup = $request->incobrak_antsup;
        $orto->incobrak_antinf = $request->incobrak_antinf;
        $orto->incobrak_postsup = $request->incobrak_postsup;
        $orto->incobrak_postinf = $request->incobrak_postinf;

        $orto->incobrak_bracket = $request->incobrak_bracket;
        $orto->bracket_despega = $request->bracket_despega;
        $orto->botones_en = $request->botones_en;
        $orto->nota_cita = $request->nota_cita;
        $orto->userId = Auth::user()->id;
        $orto->save();
    }

    public function anularOrtodoncia(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $ortodoncia = Ortodoncia::findOrFail($request->idOrtodoncia);
        $ortodoncia->anulada = 1;
        $ortodoncia->save();
    }

    //===============================================ORDEN DE TRABAJO =======================================================
    public function indexOrdenTrabajo(Request $request)
    {
       if (!$request->ajax()) return redirect('/');
    
        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $registros = RegistroOdontologico::where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $registros = RegistroOdontologico::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=',$request->idPaciente)->orderBy('id', 'desc')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $registros->total(),
                'current_page' => $registros->currentPage(),
                'per_page'     => $registros->perPage(),
                'last_page'    => $registros->lastPage(),
                'from'         => $registros->firstItem(),
                'to'           => $registros->lastItem(),
            ],
            'registros' => $registros
        ];
    }
    public function storeOrdenTrabajo(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $orden = new OrdenTrabajo();
        $orden->idPaciente = $request->idPaciente;
        $orden->idUser = Auth::user()->id;
        // $orden->num_otrabajo = $request->num_otrabajo;
        $orden->num_otrabajo = "OT01";
        $orden->fechaot = $request->fechaot;
        $orden->numero_contacto = $request->numero_contacto;
        $orden->descripcion = $request->descripcion;
        $orden->fechaentrega = $request->fechaentrega;
        $orden->color = $request->color;
        $orden->guia = $request->guia;
        $orden->observaciones = $request->observaciones;
        $orden->rodete = $request->rodete;
        $orden->metal = $request->metal;
        $orden->ceromero = $request->ceromero;
        $orden->prueba_dien = $request->prueba_dien;
        $orden->porcelana = $request->porcelana;
        $orden->ferula = $request->ferula;
        $orden->ppr_metal = $request->ppr_metal;
        $orden->terminado = $request->terminado;
        $orden->save();
    }
    
    public function updateOrdenTrabajo(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $registro = RegistroOdontologico::findOrFail($request->id);
        $registro->comentario = $request->comentario;
        $registro->save();
    }
    
    public function eliminarOrdenTrabajo(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $registro = RegistroOdontologico::findOrFail($request->id);
        $registro->delete();
    }


    //ODONTOGRAMA
    public function storeOdontograma(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $odonto = new Odontograma();
        $odonto->pac_id = $request->idPaciente;
        $odonto->otros = $request->idPaciente;
      
        $odonto->disponibleMX = 0;
        $odonto->necesarioMX = 0;
        $odonto->disponibleMD = 0;
        $odonto->necesarioMD = 0;
        $odonto->boltonMm = 0;
        $odonto->boltonMX = 0;
        $odonto->boltonMD = 0;
        $odonto->boltonnteriorMm = 0;
        $odonto->boltonnteriorMX = 0;
        $odonto->boltonnteriorMD = 0;
        
        $odonto->es_hta = $request->hta;
        $odonto->es_hepatitis = $request->hepa;
        $odonto->es_anemia = $request->anemia;
        $odonto->es_fumador = $request->fumador;
        $odonto->es_diabetes = $request->diabetes;
        $odonto->es_prenales = $request->renales;
        $odonto->es_ugastricas = $request->gastricas;
        $odonto->es_osteopo = $request->osteoporosis;
        $odonto->es_cancer = $request->cancer;
        $odonto->es_rquimote = $request->radiacion;
        $odonto->es_pcardia = $request->gastrcardiacosicas;
        $odonto->es_otros = $request->otros;
        $odonto->od_rellenos = $request->rellenos;
        $odonto->od_sellantes = $request->sellantes;
        $odonto->od_extracc = $request->extracciones;
        $odonto->od_limpiezas = $request->limpiezas;
        $odonto->od_endodon = $request->endodoncias;
        $odonto->od_implantes = $request->implantes;
        $odonto->od_prostodoncia = $request->prostodoncias;
        $odonto->ort_mdesvia = $request->mordidaDesviacion;
        $odonto->ort_mabierta = $request->mordidaAbierta;
        $odonto->ort_mprofu = $request->mordidaProfunda;
        $odonto->ort_mcruant = $request->morCruzadaAnterior;
        $odonto->ort_mcrupost = $request->morCruzadaPosterior;
        $odonto->ort_mtije = $request->mordidaTijera;
        $odonto->ort_apileve = $request->apiLeve;
        $odonto->ort_apimode = $request->apiModerado;

        $odonto->ort_apisevero = $request->apiSevero;
        $odonto->ort_perfil = $request->perfil;
        $odonto->ort_lmedias = $request->liMedia;
        $odonto->ort_overyet = $request->overyet;
        $odonto->ort_diastemas = $request->diastemas;
        $odonto->ort_ttrata = $request->tipoTratamiendo;
        $odonto->ort_duracion = $request->duracion;
        $odonto->ort_notas = $request->notas;
        $odonto->imgName = $request->idPaciente."_".date("mdyHis");
        $odonto->save();

        if ($request->imgBase64 != "empty") {
            self::saveFoto($request->imgBase64, $request->idPaciente."_".date("mdyHis"));
        }         
                    // 'blanqueamientos': this.blanqueamientos,
                
                    // 'tipoMordida': this.tipoMordida,
                    // otros text
    }
    public function saveFoto($base64Img,$name){
        $extension = explode('/', explode(':', substr($base64Img, 0, strpos($base64Img, ';')))[1])[1];   // .jpg .png .pdf

        $replace = substr($base64Img, 0, strpos($base64Img, ',')+1); 

        // find substring fro replace here eg: data:image/png;base64,

        $image = str_replace($replace, '', $base64Img); 
        $image = str_replace(' ', '+', $image); 
        $imageName = $name.'.'."jpg";
        Storage::disk('odontogramas')->put($imageName, base64_decode($image));
    }

    public function indexOdontograma(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        if ($buscar==''){
            $odontogramas = Odontograma::where('pac_id','=', $request->idPaciente)
            ->orderBy('created_at', 'ASC')->paginate(10);
        }
        else{
            $odontogramas = Odontograma::where($criterio, 'like', '%'. $buscar . '%')->where('pac_id','=', $request->idPaciente)->orderBy('created_at', 'ASC')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $odontogramas->total(),
                'current_page' => $odontogramas->currentPage(),
                'per_page'     => $odontogramas->perPage(),
                'last_page'    => $odontogramas->lastPage(),
                'from'         => $odontogramas->firstItem(),
                'to'           => $odontogramas->lastItem(),
            ],
            'odontogramas' => $odontogramas
        ];
    }

    public function getFotoBase64(Request $request){
        $file = "" ;
        $base64img = "";
        //return $request->idPaciente;
        if (Storage::disk('odontogramas')->exists($request->imgName)) {
           $file =  Storage::disk('odontogramas')->get($request->imgName);
          return  "data:image/jpeg;base64,".base64_encode($file);
        }else{
            return 'empty';
        }
    }
    
}

