<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;

use App\Proveedor;
use App\Persona;


class ProveedorController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        $buscar = $request->buscar;
        $criterio = $request->criterio;
         
        if ($buscar==''){
            $personas = Proveedor::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->with('cuentaContable')->paginate(10);
        }
        else{
            $personas = Proveedor::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->with('cuentaContable')->paginate(10);
        }
       
        

        return [
            'pagination' => [
                'total'        => $personas->total(),
                'current_page' => $personas->currentPage(),
                'per_page'     => $personas->perPage(),
                'last_page'    => $personas->lastPage(),
                'from'         => $personas->firstItem(),
                'to'           => $personas->lastItem(),
            ],
            'personas' => $personas
        ];
    }

    public function selectProveedor(Request $request){
        if (!$request->ajax()) return redirect('/');
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        $filtro = $request->filtro;
        $proveedores = Proveedor::join('personas','proveedores.id','=','personas.id')
        ->where('personas.nombre', 'like', '%'. $filtro . '%')
        ->where('proveedores.idEmpresa','=', $idEmpresa)
        //->orWhere('personas.num_documento', 'like', '%'. $filtro . '%')
        ->select('personas.id','personas.nombre','personas.num_documento')
        ->orderBy('personas.nombre', 'asc')->get();

        return ['proveedores' => $proveedores];
    }
    public function autoCompleteProveeodor(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        //$proveedores = Proveedor::select('nombre','nrc','nit','tipocontribuyente')
        $proveedores = Proveedor::select('id','nombre','nrc','nit','tipocontribuyente')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where(DB::raw("CONCAT(nrc,nit,nombre)"), 'like', '%'. $request->parametro . '%')
        ->distinct('id')
        ->orderBy('nrc', 'asc')->get();
        return ['proveedores' => $proveedores];
    }

    public function listarPdf(){
        $idEmpresa = Auth::user()->idEmpresa;
        $personas = Proveedor::join('personas','proveedores.id','=','personas.id')
            ->select('personas.id','personas.nombre','personas.tipo_documento',
            'personas.num_documento','personas.direccion','personas.telefono',
            'personas.email','proveedores.contacto','proveedores.telefono_contacto')
            ->where('proveedores.idEmpresa','=', $idEmpresa)
            ->orderBy('personas.nombre', 'asc')->get();        
        //$cont=Proveedor::count();
        $cont=sizeof($personas);

        $pdf = \PDF::loadView('pdf.proveedorespdf',['proveedores'=>$personas,'cont'=>$cont])->setPaper('a4', 'landscape');
        return $pdf->download('proveedores.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        try{
            DB::beginTransaction();

            $proveedor = new Proveedor();
            $proveedor->nombre = $request->nombre;
            $proveedor->direccion = $request->direccion;
            $proveedor->telefono = $request->telefono;
            $proveedor->email = $request->email;
            $proveedor->idEmpresa = $request->$idEmpresa;
            $proveedor->contacto = $request->contacto;
            $proveedor->dui = $request->dui;
            $proveedor->nit = $request->nit;
            $proveedor->nrc = $request->nrc;
            $proveedor->ctapagar = $request->idTermino;
            $proveedor->creditodisponible = $request->creditoDisponible;
            $proveedor->limitecredito = $request->limiteCredito;
            $proveedor->montodisponible = $request->montoDisponible;
            $proveedor->creditoutilizado = $request->creditoUtilizado;
            $proveedor->tipocontribuyente = $request->contribuyente;
            $proveedor->tipo_operacion = $request->tipoOperacion;
            $proveedor->clasificacion = $request->clasificacion;
            $proveedor->sector = $request->sector;
            $proveedor->tipo_costo_gasto = $request->tipoCostoGasto;
            if ($rolUsuario == $_ENV['ROOT']) {
                $proveedor->idEmpresa = $request->empresa;
            }else{
                $proveedor->idEmpresa = $idEmpresa;
            }
            $proveedor->save();

            DB::commit();

        } catch (Exception $e){
            DB::rollBack();
        }

        
        
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        
        try{
            DB::beginTransaction();

            //Buscar primero el proveedor a modificar
            $proveedor = Proveedor::findOrFail($request->id);
            $proveedor->nombre = $request->nombre;
            $proveedor->direccion = $request->direccion;
            $proveedor->telefono = $request->telefono;
            $proveedor->email = $request->email;
            $proveedor->contacto = $request->contacto;
            $proveedor->dui = $request->dui;
            $proveedor->nit = $request->nit;
            $proveedor->nrc = $request->nrc;
            $proveedor->ctapagar = $request->idTermino;
            $proveedor->creditodisponible = $request->creditoDisponible;
            $proveedor->limitecredito = $request->limiteCredito;
            $proveedor->montodisponible = $request->montoDisponible;
            $proveedor->creditoutilizado = $request->creditoUtilizado;
            $proveedor->tipocontribuyente = $request->contribuyente;
            $proveedor->tipo_operacion = $request->tipoOperacion;
            $proveedor->clasificacion = $request->clasificacion;
            $proveedor->sector = $request->sector;
            $proveedor->tipo_costo_gasto = $request->tipoCostoGasto;
            $proveedor->save();
            DB::commit();

        } catch (Exception $e){
            DB::rollBack();
        }

    }
}
