<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Radiografia;
use Illuminate\Support\Facades\Storage;

class RadiografiaController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        if ($buscar==''){
            $radiografias = Radiografia::where('idPaciente','=', $request->idPaciente)
            ->orderBy('created_at', 'ASC')->paginate(10);
        }
        else{
            $radiografias = Radiografia::where($criterio, 'like', '%'. $buscar . '%')->where('idPaciente','=', $request->idPaciente)->orderBy('created_at', 'ASC')->paginate(10);
        }
        
        return [
            'pagination' => [
                'total'        => $radiografias->total(),
                'current_page' => $radiografias->currentPage(),
                'per_page'     => $radiografias->perPage(),
                'last_page'    => $radiografias->lastPage(),
                'from'         => $radiografias->firstItem(),
                'to'           => $radiografias->lastItem(),
            ],
            'radiografias' => $radiografias
        ];
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $radiografia = new Radiografia();
        $imageName = "R".$request->idPaciente."_".date("YmdHis").'.'."jpg";
        $radiografia->idPaciente = $request->idPaciente;
        $radiografia->comentario = $request->comentario;
        $radiografia->imageName =$imageName;
        if ($radiografia->save()) {
            self::saveFoto($request->imgBase64,$imageName);
        }
    }
    public function saveFoto($base64Img,$imageName){
        $extension = explode('/', explode(':', substr($base64Img, 0, strpos($base64Img, ';')))[1])[1];   // .jpg .png .pdf

        $replace = substr($base64Img, 0, strpos($base64Img, ',')+1); 

        // find substring fro replace here eg: data:image/png;base64,
        $image = str_replace($replace, '', $base64Img); 
        $image = str_replace(' ', '+', $image); 
        Storage::disk('radiografias')->put($imageName, base64_decode($image));

    }
    public function getFotoBase64(Request $request){
        $file = "" ;
        $base64img = "";
        //return $request->idPaciente;
        if (Storage::disk('radiografias')->exists($request->imgName)) {
           $file =  Storage::disk('radiografias')->get($request->imgName);
          return  "data:image/jpeg;base64,".base64_encode($file);
        }else{
            return 'empty';
        }
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $radiografia = Radiografia::findOrFail($request->id);
        $radiografia->comentario = $request->comentario;
        if ($radiografia->save()) {
            self::saveFoto($request->imgBase64,$radiografia->imageName);
        }
    }
    public function destroy(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $radiografia = Radiografia::findOrFail($request->id);
       $nombre=  $radiografia->imageName;
        if ($radiografia->delete()){
            if (Storage::disk('radiografias')->exists($nombre)) {
                $file =  Storage::disk('radiografias')->delete($nombre); 
             }
        }
    }
}
