<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use App\SucursalCliente;
use App\Http\Controllers\Controller;

class SucursalClienteController extends Controller
{
     public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $sucursales = SucursalCliente::join('cliente','fc_sucxcliente.idCliente','=','cliente.id')
            			  ->select( DB::raw('cliente.nombre as nombreCliente'),'fc_sucxcliente.id','fc_sucxcliente.idCliente','fc_sucxcliente.codigo','fc_sucxcliente.nombre','fc_sucxcliente.direccion','fc_sucxcliente.idTipoPrecio','fc_sucxcliente.estado','fc_sucxcliente.created_at')
                          ->with('tipoPrecio')
            			  ->orderBy('fc_sucxcliente.id', 'desc')->where('fc_sucxcliente.idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{

            $sucursales = SucursalCliente::join('cliente','fc_sucxcliente.idCliente','=','cliente.id')
            			  ->select( DB::raw('cliente.nombre as nombreCliente'),'fc_sucxcliente.id','fc_sucxcliente.idCliente','fc_sucxcliente.codigo','fc_sucxcliente.nombre','fc_sucxcliente.direccion','fc_sucxcliente.idTipoPrecio','fc_sucxcliente.estado','fc_sucxcliente.created_at')
                          ->with('tipoPrecio')
            			  ->orderBy('fc_sucxcliente.id', 'desc')->where('fc_sucxcliente.idEmpresa','=', Auth::user()->idEmpresa);
            if ($criterio == "nombre") {
                $sucursales = $sucursales->where('fc_sucxcliente.'.$criterio, 'like', '%'. $buscar . '%')->where('fc_sucxcliente.idEmpresa','=', Auth::user()->idEmpresa)->orderBy('fc_sucxcliente.id', 'desc')->paginate(10);
            }else{
                $sucursales = $sucursales->where('cliente.nombre', 'like', '%'. $buscar . '%')->where('fc_sucxcliente.idEmpresa','=', Auth::user()->idEmpresa)->orderBy('fc_sucxcliente.id', 'desc')->paginate(10);
            }
            			 
        }
        

        return [
            'pagination' => [
                'total'        => $sucursales->total(),
                'current_page' => $sucursales->currentPage(),
                'per_page'     => $sucursales->perPage(),
                'last_page'    => $sucursales->lastPage(),
                'from'         => $sucursales->firstItem(),
                'to'           => $sucursales->lastItem(),
            ],
            'sucursales' => $sucursales
        ];
    }


    public function listarPdf(){
        $sucursales = SucursalCliente::join('cliente','fc_sucxcliente.idCliente','=','cliente.id')
            			  ->select( DB::raw('cliente.nombre as nombreCliente'),'fc_sucxcliente.id','fc_sucxcliente.idCliente','fc_sucxcliente.codigo','fc_sucxcliente.nombre','fc_sucxcliente.direccion','fc_sucxcliente.estado','fc_sucxcliente.created_at')
        				 ->where('fc_sucxcliente.idEmpresa','=', Auth::user()->idEmpresa)
        				 ->orderBy('fc_sucxcliente.id', 'desc')->get();        
        $cont=SucursalCliente::where('idEmpresa','=', Auth::user()->idEmpresa)->count();

        $pdf = \PDF::loadView('pdf.sucursalesClientespdf',['sucursales'=>$sucursales,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('sucursales.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sucursal = new SucursalCliente();
        $sucursal->codigo = $request->codigo;
        $sucursal->nombre = $request->nombre;
        $sucursal->direccion = $request->direccion;
        $sucursal->idTipoPrecio = $request->idTipoPrecio;
        $sucursal->idCliente = $request->idCliente;
        $sucursal->idEmpresa = Auth::user()->idEmpresa;
        $sucursal->estado = "1";
        $sucursal->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sucursal = SucursalCliente::findOrFail($request->id);
        $sucursal->codigo = $request->codigo;
        $sucursal->nombre = $request->nombre;
        $sucursal->direccion = $request->direccion;
        $sucursal->idTipoPrecio = $request->idTipoPrecio;
        $sucursal->idCliente = $request->idCliente;
        $sucursal->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sucursal = SucursalCliente::findOrFail($request->id);
        $sucursal->estado = 0;
        $sucursal->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $sucursal = SucursalCliente::findOrFail($request->id);
        $sucursal->estado = 1;
        $sucursal->save();
    }

     public function selectsSucursal(Request $request){
        if (!$request->ajax()) return redirect('/');
        $sucursales = SucursalCliente::select('id','codigo','nombre')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('codigo', 'asc')->get();
        return ['sucursales' => $sucursales];
    }

    public function selectSucursalCliente(Request $request){
        if (!$request->ajax()) return redirect('/');
        $idCliente = $request->idCliente;
        $sucursales = SucursalCliente::select('id','nombre')
        ->where('idCliente','=',$idCliente)
        ->orderBy('id', 'asc')->get();
        return ['sucursales' => $sucursales];
    }

    public function selectSucursalClienteSearch(Request $request){
        if (!$request->ajax()) return redirect('/');
        $idCliente = $request->idCliente;
        $sucursales = SucursalCliente::select('id as value','nombre as text')
        ->where('idCliente','=',$idCliente)
        ->orderBy('id', 'asc')->get();
        return ['sucursales' => $sucursales];
    }
}
