<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\TipoGasto;
use App\Http\Controllers\Controller;

class TipoGastoController extends Controller
{
    public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $gastos = TipoGasto::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->with('gastoCuentaContable')->paginate(10);
        }
        else{
            $gastos = TipoGasto::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->with('gastoCuentaContable')->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $gastos->total(),
                'current_page' => $gastos->currentPage(),
                'per_page'     => $gastos->perPage(),
                'last_page'    => $gastos->lastPage(),
                'from'         => $gastos->firstItem(),
                'to'           => $gastos->lastItem(),
            ],
            'gastos' => $gastos
        ];
    }


    public function listarPdf(){
        $gastos = TipoGasto::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('codigo', 'asc')->get();        
        $cont=TipoGasto::count();

        $pdf = \PDF::loadView('pdf.gastospdf',['gastos'=>$gastos,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('gastos.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $gasto = new TipoGasto();
        $gasto->codigo = $request->codigo;
        $gasto->nombre = $request->nombre;
        $gasto->ctacontable = $request->cuentaContable;
        $gasto->idEmpresa = Auth::user()->idEmpresa;
        $gasto->estado = "1";
        $gasto->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $gasto = TipoGasto::findOrFail($request->id);
        $gasto->codigo = $request->codigo;
        $gasto->nombre = $request->nombre;
        $gasto->ctacontable = $request->cuentaContable;
        $gasto->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $gasto = TipoGasto::findOrFail($request->id);
        $gasto->estado = 0;
        $gasto->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $gasto = TipoGasto::findOrFail($request->id);
        $gasto->estado = 1;
        $gasto->save();
    }

     public function selectGasto(Request $request){
        if (!$request->ajax()) return redirect('/');
        $gastos = TipoGasto::select('id','codigo','nombre')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('codigo', 'asc')->get();
        return ['gastos' => $gastos];
    }
}
