<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\TipoIngreso;
use App\Http\Controllers\Controller;

class TipoIngresoController extends Controller
{
     public function index(Request $request)
    {
       if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $ingresos = TipoIngreso::orderBy('id', 'desc')->where('idEmpresa','=', Auth::user()->idEmpresa)->paginate(10);
        }
        else{
            $ingresos = TipoIngreso::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $ingresos->total(),
                'current_page' => $ingresos->currentPage(),
                'per_page'     => $ingresos->perPage(),
                'last_page'    => $ingresos->lastPage(),
                'from'         => $ingresos->firstItem(),
                'to'           => $ingresos->lastItem(),
            ],
            'ingresos' => $ingresos
        ];
    }


    public function listarPdf(){
        $ingresos = TipoIngreso::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('codigo', 'asc')->get();        
        $cont=TipoIngreso::where('idEmpresa','=', Auth::user()->idEmpresa)->count();

        $pdf = \PDF::loadView('pdf.tipoIngresospdf',['ingresos'=>$ingresos,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('tipoIngresos.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $ingreso = new TipoIngreso();
        $ingreso->codigo = $request->codigo;
        $ingreso->nombre = $request->nombre;
        $ingreso->ctacontable = $request->cuentaContable;
        $ingreso->idEmpresa = Auth::user()->idEmpresa;
        $ingreso->estado = "1";
        $ingreso->save();
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $ingreso = TipoIngreso::findOrFail($request->id);
        $ingreso->codigo = $request->codigo;
        $ingreso->nombre = $request->nombre;
        $ingreso->ctacontable = $request->cuentaContable;
        $ingreso->save();
    }
    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $ingreso = TipoIngreso::findOrFail($request->id);
        $ingreso->estado = 0;
        $ingreso->save();
    }

    public function activar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $ingreso = TipoIngreso::findOrFail($request->id);
        $ingreso->estado = 1;
        $ingreso->save();
    }

     public function selectIngreso(Request $request){
        if (!$request->ajax()) return redirect('/');
        $ingresos = TipoIngreso::select('id','codigo','nombre')
        ->where('estado','=','1')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->orderBy('codigo', 'asc')->get();
        return ['ingresos' => $ingresos];
    }
}
