<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use App\VentaChica;

class VentaChicaController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        
        if ($buscar==''){
            $ventas = VentaChica::where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        else{
            $ventas = VentaChica::where($criterio, 'like', '%'. $buscar . '%')->where('idEmpresa','=', Auth::user()->idEmpresa)->orderBy('id', 'desc')->paginate(10);
        }
        

        return [
            'pagination' => [
                'total'        => $ventas->total(),
                'current_page' => $ventas->currentPage(),
                'per_page'     => $ventas->perPage(),
                'last_page'    => $ventas->lastPage(),
                'from'         => $ventas->firstItem(),
                'to'           => $ventas->lastItem(),
            ],
            'ventas' => $ventas
        ];
    }

   
    public function listarPdf(){
        $ventas = VentaChica::orderBy('id', 'asc')->where('idEmpresa','=', Auth::user()->idEmpresa)->get();
        $cont=VentaChica::where('idEmpresa','=', Auth::user()->idEmpresa)->count();        
        $pdf = \PDF::loadView('pdf.ventasccpdf',['ventas'=>$ventas,'cont'=>$cont])->setPaper('a4', 'portrait');
        return $pdf->download('VentaCajaChica.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $fecha1 = explode('/',$request->fechaDocumento);
        $fecha1 = $fecha1[2].'-'.$fecha1[1].'-'.$fecha1[0];

        $fecha2 = explode('/',$request->fechaIva);
        $fecha2 = $fecha2[2].'-'.$fecha2[1].'-'.$fecha2[0];

        $fecha3 = explode('/',$request->fechaVencimiento);
        $fecha3 = $fecha3[2].'-'.$fecha3[1].'-'.$fecha3[0];


        $venta = new VentaChica();
        $venta->idEmpresa = Auth::user()->idEmpresa;
        $venta->tipoventa = $request->idIngreso;
        $venta->idbodega = $request->idBodega;
        $venta->idsuccliente = $request->idSucCliente;
        $venta->clasedoc = $request->claseDocumento;
        $venta->tipodoc = $request->tipoDocumento;
        $venta->fechadoc = $fecha1;
        $venta->fechalibroiva = $fecha2;
        $venta->numdoc = $request->numeroDocumento;
        $venta->numreso = "0";
        $venta->numseriedoc = "0";
        $venta->numfunico = "0";
        $venta->numcomprobante = "0";
        $venta->nrcnit = "0";
        $venta->cliente = "0";
        $venta->tipocliente = $request->tipoCliente;
        $venta->detalleventa = $request->detalleventa;
        $venta->ver_liva = $request->verLiva;
        $venta->dias_credito = $request->diasCredito;
        $venta->fecha_vencimiento = $fecha3;
        $venta->observaciones_venta = $request->observaciones;
        $venta->numero_quedan = $request->numeroQuedan;
        $venta->subtotal = $request->subTotal;
        $venta->iva = $request->iva;
        $venta->ret = $request->ret;
        $venta->ret_renta_10 = $request->retRenta;
        $venta->fovial = $request->fovial;
        $venta->contrans = $request->contrans;
        $venta->cesc = $request->cesc;
        $venta->otros = $request->otros;
        $venta->total = $request->total;
        $venta->saldo_documento = $request->saldoDocumento;
        $venta->estado = 1;
        $venta->save();                            
    }

    public function update(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
         $fecha1 = explode('/',$request->fechaDocumento);
        $fecha1 = $fecha1[2].'-'.$fecha1[1].'-'.$fecha1[0];

        $fecha2 = explode('/',$request->fechaIva);
        $fecha2 = $fecha2[2].'-'.$fecha2[1].'-'.$fecha2[0];

        $fecha3 = explode('/',$request->fechaVencimiento);
        $fecha3 = $fecha3[2].'-'.$fecha3[1].'-'.$fecha3[0];
        $venta = VentaChica::findOrFail($request->id);
        $venta->tipoventa = $request->idIngreso;
        $venta->idbodega = $request->idBodega;
        $venta->clasedoc = $request->claseDocumento;
        $venta->idsuccliente = $request->idSucCliente;
        $venta->tipodoc = $request->tipoDocumento;
        $venta->fechadoc = $fecha1;
        $venta->fechalibroiva = $fecha2;
        $venta->numdoc = $request->numeroDocumento;
        $venta->numreso = "0";
        $venta->numseriedoc = "0";
        $venta->numfunico = "0";
        $venta->numcomprobante = "0";
        $venta->nrcnit = "0";
        $venta->cliente = "0";
        $venta->tipocliente = $request->tipoCliente;
        $venta->detalleventa = $request->detalleventa;
        $venta->ver_liva = $request->verLiva;
        $venta->dias_credito = $request->diasCredito;
        $venta->fecha_vencimiento = $fecha3;
        $venta->observaciones_venta = $request->observaciones;
        $venta->numero_quedan = $request->numeroQuedan;
        $venta->subtotal = $request->subTotal;
        $venta->iva = $request->iva;
        $venta->ret = $request->ret;
        $venta->ret_renta_10 = $request->retRenta;
        $venta->fovial = $request->fovial;
        $venta->contrans = $request->contrans;
        $venta->cesc = $request->cesc;
        $venta->otros = $request->otros;
        $venta->total = $request->total;
        $venta->saldo_documento = $request->saldoDocumento;
        $venta->save();
    }

    public function selectNumResolucion(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = VentaChica::select('numreso')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numreso', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numreso', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function selectNumSerie(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = VentaChica::select('numseriedoc')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numseriedoc', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numseriedoc', 'asc')->get();
        return ['numeros' => $numeros];
    }
}
