<?php

namespace App\Http\Controllers;

use App\Articulo;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Carbon\Carbon;
use App\Venta;
use App\DetalleVenta;
use App\BodegaEmpleado;
use App\ExistenciaProducto;
use Illuminate\Support\Facades\Storage;
use App\Http\Controllers\FacturaElectronicaController;
use App\Cliente;
use App\CuentaContable;
use App\Empresa;
use App\FacturaElectronicaLog;
use App\DocumentoRelacionado;
use App\Exports\CompraProveedorExport;
use App\Exports\VentaConsumidorFinal;
use App\Exports\VentaCreditoFiscalExport;
use Ramsey\Uuid\Uuid;
use App\Exports\VentaExport;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Log;

class VentaController extends Controller
{
    public function index(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $criterio = $request->criterio;
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        $bodegaEmpleado = $this->getBodegaEmpleadoSelf();
        $empresa = Empresa::findOrFail(Auth::user()->idEmpresa);
        $bodegas = array();
    
        foreach($bodegaEmpleado as $bod){
            array_push($bodegas,"".$bod->id_bodega);
        }
         if ($buscar==''){
                // $ventas = Venta::where('idEmpresa','=', $idEmpresa)
                // ->whereIn('idBodega', $bodegas)
                // ->orderBy('id', 'desc')
                // ->with('ventaPaciente')
                // ->with('bodega')->paginate(10);

                $ventas = Venta::join('cliente','fc_factura.idCliente','=','cliente.id')
                ->join('in_bodega','fc_factura.idBodega','=','in_bodega.id')
                ->join('personas','fc_factura.idVendedor','=','personas.id')
                ->join('dte_tipo_documentofc','fc_factura.tipo_documento','=','dte_tipo_documentofc.codigo')
                ->leftJoin('fc_sucxcliente','fc_factura.id_suc_cliente','=','fc_sucxcliente.id')
                ->select('fc_factura.id',
                'cliente.nombre as nombreCliente',
                'cliente.apellido as apellidoCliente',
                'cliente.codCliente',
                'cliente.id as idCliente',
                'fc_factura.num_documento',
                'fc_factura.tipo_documento',
                'dte_tipo_documentofc.tipo_documento as nombreTipoDocumento',
                'fc_factura.anulada',
                'fc_factura.tipoPago',
                'fc_factura.fecha_factura',
                'fc_factura.total_fc',
                'fc_factura.facturaNombre',
                'fc_factura.codigoGeneracion',
                'fc_factura.selloRecibido',
                'fc_factura.esPrefactura',
                'in_bodega.nombre as nombreBodega',
                'personas.nombre as nombreUsuario',
                'fc_sucxcliente.nombre as nombreSucursal'
                )
                ->whereIn('fc_factura.idBodega', $bodegas)
                ->orderBy('fc_factura.id', 'desc')
                ->paginate(10);
            }
            else{
                if ($criterio == 'num_documento'){
                    $criterio = 'fc_factura.codigoGeneracion';
                }else if ($criterio == 'nombrePaciente'){
                    $criterio = 'cliente.nombre';
                }else if ($criterio == 'apellidoPaciente'){
                    $criterio = 'cliente.apellido';
                }else if ($criterio == 'codigoPaciente'){
                    $criterio = 'cliente.codCliente';
                }else if ($criterio == 'date'){
                    $criterio = 'fc_factura.fecha_factura';
                }else if ($criterio == 'sucursal'){
                    $criterio = 'fc_sucxcliente.nombre';
                }else if ($criterio == 'tipoDocumento'){
                    $criterio = 'dte_tipo_documentofc.tipo_documento';
                }
                
                $ventas = Venta::join('cliente','fc_factura.idCliente','=','cliente.id')
                ->join('in_bodega','fc_factura.idBodega','=','in_bodega.id')
                ->join('personas','fc_factura.idVendedor','=','personas.id')
                ->join('dte_tipo_documentofc','fc_factura.tipo_documento','=','dte_tipo_documentofc.codigo')
                ->leftJoin('fc_sucxcliente','fc_factura.id_suc_cliente','=','fc_sucxcliente.id')
                ->select('fc_factura.id',
                'cliente.nombre as nombreCliente',
                'cliente.apellido as apellidoCliente',
                'cliente.codCliente',
                'cliente.id as idCliente',
                'fc_factura.num_documento',
                'fc_factura.tipo_documento',
                'dte_tipo_documentofc.tipo_documento as nombreTipoDocumento',
                'fc_factura.anulada',
                'fc_factura.tipoPago',
                'fc_factura.fecha_factura',
                'fc_factura.total_fc',
                'fc_factura.facturaNombre',
                'fc_factura.codigoGeneracion',
                'fc_factura.selloRecibido',
                'fc_factura.created_at',
                'fc_factura.esPrefactura',
                'in_bodega.nombre as nombreBodega',
                'personas.nombre as nombreUsuario',
                'fc_sucxcliente.nombre as nombreSucursal'
            )
                ->where($criterio, 'like', '%'. $buscar . '%')
                ->whereIn('fc_factura.idBodega', $bodegas)
                ->orderBy('fc_factura.id', 'desc')
                ->paginate(10);
            }
            // $ventaEjemplo = Venta::with('documentoRelacionado')->findOrFail(59);
        return [
            'pagination' => [
                'total'        => $ventas->total(),
                'current_page' => $ventas->currentPage(),
                'per_page'     => $ventas->perPage(),
                'last_page'    => $ventas->lastPage(),
                'from'         => $ventas->firstItem(),
                'to'           => $ventas->lastItem(),
            ],
            'ventas' => $ventas,
            'bodegaEmpleado' => $bodegaEmpleado,
            'empresa' => $empresa
            // 'ventaEjemplo' => $ventaEjemplo,
        ];
    }

    public function indexModalNotaCredito(Request $request)
    {
        if (!$request->ajax()) return redirect('/');

        $buscar = $request->buscar;
        $idCliente = $request->idCliente;
        $fechaDesde = $request->fechaDesde;
        $fechaHasta = $request->fechaHasta;
        $criterio = $request->criterio;
        $empresa = Empresa::findOrFail(Auth::user()->idEmpresa);
        $tipoDocumento = $request->tipoDocumento;
        $ventas = Venta::join('cliente','fc_factura.idCliente','=','cliente.id')
                ->join('in_bodega','fc_factura.idBodega','=','in_bodega.id')
                ->join('personas','fc_factura.idVendedor','=','personas.id')
                ->select('fc_factura.id',
                'cliente.nombre as nombreCliente',
                'cliente.apellido as apellidoCliente',
                'cliente.codCliente',
                'cliente.id as idCliente',
                'fc_factura.num_documento',
                'fc_factura.tipo_documento',
                'fc_factura.anulada',
                'fc_factura.tipoPago',
                'fc_factura.fecha_factura',
                'fc_factura.sub_total_fc',
                'fc_factura.total_fc',
                'fc_factura.facturaNombre',
                'fc_factura.codigoGeneracion',
                'fc_factura.selloRecibido',
                'in_bodega.nombre as nombreBodega',
                'personas.nombre as nombreUsuario'
                )
                ->where("fc_factura.tipo_documento", '=', $tipoDocumento)
                ->where("fc_factura.anulada", '=', '0')
                ->where("fc_factura.idCliente", '=', $idCliente)
                ->whereBetween('fc_factura.created_at', [$fechaDesde, $fechaHasta])
                ->whereNotNull('fc_factura.selloRecibido')
                ->orderBy('fc_factura.id', 'desc')
                ->paginate(10);
        
        return [
            'pagination' => [
                'total'        => $ventas->total(),
                'current_page' => $ventas->currentPage(),
                'per_page'     => $ventas->perPage(),
                'last_page'    => $ventas->lastPage(),
                'from'         => $ventas->firstItem(),
                'to'           => $ventas->lastItem(),
            ],
            'ventas' => $ventas
        ];
    }
    public function obtenerCabecera(Request $request){
        if (!$request->ajax()) return redirect('/');

        $id = $request->id;
        $venta = Venta::join('personas','ventas.idcliente','=','personas.id')
        ->join('users','ventas.idusuario','=','users.id')
        ->select('ventas.id','ventas.tipo_comprobante','ventas.serie_comprobante',
        'ventas.num_comprobante','ventas.fecha_hora','ventas.impuesto','ventas.total',
        'ventas.estado','personas.nombre','users.usuario')
        ->where('ventas.id','=',$id)
        ->orderBy('ventas.id', 'desc')->take(1)->get();
        
        return ['venta' => $venta];
    }
    public function obtenerDetalles(Request $request){
        if (!$request->ajax()) return redirect('/');

        $id = $request->id;
        $detalles = DetalleVenta::join('articulos','detalle_ventas.idarticulo','=','articulos.id')
        ->select('detalle_ventas.cantidad','detalle_ventas.precio','detalle_ventas.descuento',
        'articulos.nombre as articulo')
        ->where('detalle_ventas.idventa','=',$id)
        ->orderBy('detalle_ventas.id', 'desc')->get();
        
        return ['detalles' => $detalles];
    }
      
    public function listarPdf(){
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        $ventas = Venta::join('personas','ventas.idcliente','=','personas.id')
            ->join('users','ventas.idusuario','=','users.id')
            ->select('ventas.id','ventas.tipo_comprobante','ventas.serie_comprobante',
            'ventas.num_comprobante','ventas.fecha_hora','ventas.impuesto','ventas.total',
            'ventas.estado','personas.nombre','users.usuario')
            ->where('ventas.idEmpresa','=', $idEmpresa)
            ->orderBy('ventas.id', 'desc')->get();
        //$cont=Venta::count();
        $cont=sizeof($ventas);

        $pdf = \PDF::loadView('pdf.ventaspdf',['ventas'=>$ventas,'cont'=>$cont])->setPaper('a4', 'landscape');
        return $pdf->download('ventas.pdf');
    }

    public function pdf(Request $request,$id){
        $venta = Venta::join('personas','ventas.idcliente','=','personas.id')
        ->join('users','ventas.idusuario','=','users.id')
        ->select('ventas.id','ventas.tipo_comprobante','ventas.serie_comprobante',
        'ventas.num_comprobante','ventas.created_at','ventas.impuesto','ventas.total',
        'ventas.estado','personas.nombre','personas.tipo_documento','personas.num_documento',
        'personas.direccion','personas.email',
        'personas.telefono','users.usuario')
        ->where('ventas.id','=',$id)
        ->orderBy('ventas.id', 'desc')->take(1)->get();

        $detalles = DetalleVenta::join('articulos','detalle_ventas.idarticulo','=','articulos.id')
        ->select('detalle_ventas.cantidad','detalle_ventas.precio','detalle_ventas.descuento',
        'articulos.nombre as articulo')
        ->where('detalle_ventas.idventa','=',$id)
        ->orderBy('detalle_ventas.id', 'desc')->get();

        $numventa=Venta::select('num_comprobante')->where('id',$id)->get();

        $pdf = \PDF::loadView('pdf.venta',['venta'=>$venta,'detalles'=>$detalles]);
        return $pdf->download('venta-'.$numventa[0]->num_comprobante.'.pdf');
    }

    public function pdfTicket(Request $request,$id){
        $venta = Venta::join('personas','ventas.idcliente','=','personas.id')
        ->join('users','ventas.idusuario','=','users.id')
        ->select('ventas.id','ventas.tipo_comprobante','ventas.serie_comprobante',
        'ventas.num_comprobante','ventas.created_at','ventas.impuesto','ventas.total',
        'ventas.estado','personas.nombre','personas.tipo_documento','personas.num_documento',
        'personas.direccion','personas.email',
        'personas.telefono','users.usuario')
        ->where('ventas.id','=',$id)
        ->orderBy('ventas.id', 'desc')->take(1)->get();

        $detalles = DetalleVenta::join('articulos','detalle_ventas.idarticulo','=','articulos.id')
        ->select('detalle_ventas.cantidad','detalle_ventas.precio','detalle_ventas.descuento',
        'articulos.nombre as articulo')
        ->where('detalle_ventas.idventa','=',$id)
        ->orderBy('detalle_ventas.id', 'desc')->get();

        $numventa=Venta::select('num_comprobante')->where('id',$id)->get();

        $pdf = \PDF::loadView('pdf.ventaticket',['venta'=>$venta,'detalles'=>$detalles]);
        return $pdf->download('ventaTicket-'.$numventa[0]->num_comprobante.'.pdf');
    }

    public function store(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $rolUsuario = Auth::user()->idrol;
        $idEmpresa = Auth::user()->idEmpresa;
        $fecha1 = explode('/',$request->fechaFactura);
        $fecha1 = $fecha1[2].'-'.$fecha1[1].'-'.$fecha1[0];
        $fecha3 = explode('/',$request->fechaVencimiento);
        $fecha3 = $fecha3[2].'-'.$fecha3[1].'-'.$fecha3[0];
        $saldoDocumento = 0;
        if ($request->diasCredito > 0 && $request->condicionVenta == 1) {
            $saldoDocumento = $request->total;
        }
        try{
            // $ventaCount = Venta::where('num_documento','=',$request->numeroDocumento)
            // ->where('clase_documento','=',$request->claseDocumento)
            // ->where('numreso','=',$request->numeroResolucion)
            // ->where('numseriedoc','=',$request->numeroSerie)
            // ->count();

            // if($ventaCount > 0){
            //     return 100;
            // }
            DB::beginTransaction();
            if($request->idFacturaEditar > 0 ) {
                $venta = venta::findOrFail($request->idFacturaEditar);
            }else {
                $venta = new Venta();
            }
            $venta->esPrefactura = $request->esPrefactura;
            $venta->idEmpresa = $idEmpresa;
            $venta->idBodega = $request->idBodega;
            $venta->clase_documento = $request->claseDocumento;
            $venta->tipo_documento = $request->tipoDocumento;
            $venta->num_documento = $request->numeroDocumento;
            if ($request->numeroResolucion == ""){
                $venta->numreso = 0;
            }else{
                $venta->numreso = $request->numeroResolucion;
            }
            
            if ($request->numeroSerie == ""){
                $venta->numseriedoc = 0;
            }else{
                $venta->numseriedoc = $request->numeroSerie;
            }

            $venta->fecha_factura = $fecha1;
            $venta->idCliente = $request->idCliente;
            $venta->id_suc_cliente = $request->idSucCliente;
            // $venta->idVendedor = $request->idVendedor;
            $venta->idVendedor = Auth::user()->id;
            $venta->condicion_venta = $request->condicionVenta;
            $venta->dias_credito = $request->diasCredito;
            $venta->fecha_vencimiento =  $fecha3;
            $venta->observaciones = $request->detalleVenta;
            $venta->anulada =$request->anulada;
            $venta->contabilizada = $request->contabilizada;
            $venta->descuento_fc  = $request->descuento;
            $venta->vta_exentas =  $request->subExcenta;
            $venta->vta_nosuje = $request->subNoSujeta;
            $venta->vta_exporta = $request->subExportacion;
            $venta->sub_total_fc = $request->subGravada + ($venta->tipo_documento == "01" ? $request->iva : 0.0); 
            $venta->iva_fc = ($venta->tipo_documento == "01" ? 0.0 : $request->iva);
            $venta->ret13Iva = ($venta->tipo_documento == "07" ? $request->iva : 0.0); //COMPROBANTE DE RETENCION
            $venta->retencion_1 = $request->ret;
            $venta->percepcion_1 = $request->per;
            $venta->retencion_10 = $request->renta;
            $venta->total_fc = $request->total;
            $venta->saldo_documento = $saldoDocumento;
            $venta->tipoPago = $request->tipoPago;
            $venta->tipoPago2 = $request->tipoPago2;
            $venta->tipoPago3 = $request->tipoPago3;
            $venta->tipoPago4 = $request->tipoPagoCheque;
            $venta->tipoPago_cgratis = $request->tipoPagoGratis;
            $venta->tipoVenta = $request->tipoVenta;
            $venta->formapago_dte = $request->formaPagoDte;
            $venta->plazo_dte = $request->plazoDte;
            $venta->fovial = $request->fovial;
            $venta->cotrans = $request->cotrans;
            $venta->idActivo = $request->idActivo;
            $venta->idBanco = $request->idBanco;
            $venta->tipo_operacion = $request->tipoOperacion;
            $venta->clasificacion = $request->clasificacion;
            $venta->sector = $request->sector;
            $venta->tipo_costo_gasto = $request->tipoCostoGasto;
            $venta->pais = $request->pais;
            $venta->recintoFiscal = $request->recintoFiscal;
            $venta->regimenExportacion = $request->regimenExportacion;
            $venta->incoterm = $request->incoterm;
            $venta->seguro = $request->seguro;
            $venta->flete = $request->flete;
            if ($request->rutaCobro == ""){
                $venta->ruta_cobro = "0";
            }else{
                $venta->ruta_cobro = $request->rutaCobro;
            }
            if ( $request->condicionVenta == 2) { // crédito
                $cliente = Cliente::findOrFail($request->idCliente);
                $limite  = $cliente-> limitecredito;
                $creditoUtilizado  = $cliente-> creditoutilizado + $request->total;
                          
                $cliente-> limitecredito = $limite;
                $cliente-> creditoutilizado = $creditoUtilizado;
                $cliente-> montodisponible = $cliente-> limitecredito - $cliente-> creditoutilizado;
               
                $venta-> saldo_documento = $request->total;
                $cliente->save();
            }

            if ($venta->idCliente == 1611) { //CLIENTE VARIOS
                $venta->facturaNombre = $request->facturaNombre;
            }else{
                $venta->facturaNombre = "-";
            }
            $facturaElectronica = new  FacturaElectronicaController();
            $codigoGeneracion = strtoupper(Uuid::uuid4()->toString());
            $venta->codigoGeneracion = $codigoGeneracion;
            $venta->save();

            $venta->numeroControl = $facturaElectronica->createNumeroControl($venta->id,$venta->tipo_documento,$venta->idBodega);
            $venta->save();
            $detalles = $request->detalle;
            
            //VERIFICAMOS PRODUCTOS ELIMINADOS
            if ($request->idFacturaEditar > 0) {
                $detalleEliminados = $request -> detalleEliminados;
                foreach($detalleEliminados as $epe=>$detE)
                {  
                    if( $detE['idDetalle'] > 0) {
                        $detalleEliminar = DetalleVenta::findOrFail($detE['idDetalle']);
                        $detalleEliminar->delete();
                    }
                }
            }
            foreach($detalles as $ep=>$det)
            {   
                if($request->idFacturaEditar > 0 && $det['idDetalle'] > 0) {
                    $detalle = DetalleVenta::findOrFail($det['idDetalle']);
                }else{
                    $detalle = new DetalleVenta();
                }
               
                if ($request->esPrefactura == 0){
                    if ( $det['servicio_conta'] == 0) { //NO ES SERVICIO
                        $existencia = ExistenciaProducto::select('id','costo_promedio')
                        ->where("idProducto", '=' ,$det['idProducto'])
                        ->where("idBodega", '=' ,$request->idBodega)
                        ->first();
    
                        $objExistencia = ExistenciaProducto::findOrFail($existencia['id']);
                        $objExistencia->existencia  = $objExistencia->existencia - $det['cantidad'];
                        $objExistencia->save();
                        $detalle->costo_promedio = $existencia['costo_promedio']; 
                    }else{
                        $detalle->costo_promedio = 0.0;
                    }
                }else{
                    $detalle->costo_promedio = 0.0;
                }
                
                $detalle->idFactura = $venta->id;
                $detalle->idProducto = $det['idProducto'];
                if ($det['observaciones'] == ""){
                    $detalle->observaciones = "-";
                }else{
                    $detalle->observaciones = $det['observaciones'];
                }
                $detalle->cantidad = $det['cantidad'];
                $detalle->precio_venta = $det['precio'];
                $detalle->descuento_producto = $det['descuento'];
                $detalle->sub_total_producto = $det['total'];    
                $detalle->idBomba = $det['bomba'];   
                $detalle->efectivo = $det['efectivo'];   
                $detalle->ivaLinea = ($venta->tipo_documento == "01" ? 0.0 : $det['ivaLinea']); 
                $detalle->servicio_conta = $det['servicio_conta'];
                $subObjeto = $det['objDocumentoRelacionado'] ?? [];
                $attr1 = $subObjeto['codigoGeneracion'] ?? '-';
                $detalle->numeroDocumento = $attr1;
                $detalle->codigoPlu = $det['codPlu'] ?? '-';
                $detalle->save();
            }

            //DOCUMENTOS RELACIONADOS
            $documentos = $request->documentoRelacionados;
            if (count($documentos) > 0) {
                foreach($documentos as $ep=>$doc){ 
                    $documento = new DocumentoRelacionado();
                    $documento->idfactura = $venta->id;
                    $documento->tipo_documento = $doc['tipo_documento'];
                    if ($doc['id'] > 0) { //ELECTRONICO
                        $documento->tipo_generacion = 2;
                    }else{
                        $documento->tipo_generacion = 1;
                    }
                    $documento->numero_documento = $doc['codigoGeneracion'];
                    
                    $documento->valordocumento = ($venta->tipo_documento == "07" ? $doc['sub_total_fc'] : $doc['total_fc']); //COMPROBANTE DE RETENCION $doc['total_fc'];
                    $documento->fecha_emision = $doc['fecha_factura'];
                    $documento->save();
                }
            }

            $enContingencia = $request->enContingencia;
            // Storage::disk('jsonfacturas')->put("factura".$venta->id.".json", $jsonString);
            // $result = $facturaElectronica->buildFacturaJSON($venta->id,$enContingencia);
            if ($request->esPrefactura == 1){
                $venta->selloRecibido = $codigoGeneracion;
                $venta->save();
                $result = [
                    'original' => [
                        'selloRecibido' => $codigoGeneracion,
                        'codigoGeneracion' => $codigoGeneracion
                    ]
                ];
            }else{
                $result = $facturaElectronica->buildFacturaJSON($venta->id,$enContingencia);
                $responseData = $result->getData();
                // Log::info('Estructura de responseData:', (array)$responseData);
                $selloRecibido = $responseData->selloRecibido;
                if (!is_null($selloRecibido) && trim($selloRecibido) !== '') {
                    //procesado
                }else{
                    if ($request->idFacturaEditar > 0){ // ES PRE FACTURA EDITADA
                        $venta->esPrefactura = 1;
                        $venta->save();
                    }
                }
            }

            DB::commit();
            return [
                'id' => $venta->id,
                'tipoDoc' => $venta->tipo_documento,
                'jsonFacturaElectronica' => $result
            ];
        } catch (Exception $e){
            // DB::rollBack();
        }
    }

    public function desactivar(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $venta = Venta::findOrFail($request->id);
        $venta->estado = 'Anulado';
        $venta->save();
    }

    public function selectNumResolucion(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = Venta::select('numreso')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numreso', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numreso', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function selectNumSerie(Request $request){
        if (!$request->ajax()) return redirect('/');
     
        $numeros = Venta::select('numseriedoc')
        ->where('idEmpresa','=', Auth::user()->idEmpresa)
        ->where('numseriedoc', 'like', '%'. $request->parametro . '%')
        ->distinct()
        ->orderBy('numseriedoc', 'asc')->get();
        return ['numeros' => $numeros];
    }

    public function getBodegaEmpleado(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        $bodega = BodegaEmpleado::where("id_empleado", "=", Auth::user()->id)->first();
        return ['bodega' => $bodega];
    }

    public function getBodegaEmpleadoSelf()
    {
        $bodega = BodegaEmpleado::where("id_empleado", "=", Auth::user()->id)->get();
        return $bodega;
    }

    public function anularFactura(Request $request)
    {
        if (!$request->ajax()) return redirect('/');
        DB::beginTransaction();
        $venta = Venta::findOrFail($request->idFactura);
        if ($venta->esPrefactura != 1) {
             //OBTENEMOS LOS DATOS DEL CLIENTE
            $cliente = Cliente::findOrFail($venta->idCliente,);
            $log = FacturaElectronicaLog::where('idFactura', $venta->id)
            ->where('serviceType', 'envio')->firstOrFail();
            $facturaElectronica = new FacturaElectronicaController();
            $facturaElectronica->anularFactura($venta,$cliente);
        }
         $venta->sub_total_fc = 0;
         $venta->iva_fc = 0;
         $venta->total_fc = 0;
         $venta->saldo_documento = 0;
         $venta->anulada = 1;
         $venta->save();
 
         //REGRESANDO PRODUCTO AL INVENTARIO
         if ($venta->esPrefactura != 1) {
            $detalleVenta = DetalleVenta::where("idFactura","=",$venta->id)->get();
            foreach($detalleVenta as $ep=>$det)
            {  
                if ($det['servicio_conta'] == 0){
                    $existencia = ExistenciaProducto::select('id','costo_promedio')
                    ->where("idProducto", '=' ,$det['idProducto'])
                    ->where("idBodega", '=' ,$venta->idBodega)
                    ->first();
        
                    $objExistencia = ExistenciaProducto::findOrFail($existencia['id']);
                    $objExistencia->existencia  = $objExistencia->existencia + $det['cantidad'];
                    $objExistencia->save();
                }
            }
         }
         DB::commit();
    }

    public function exportVentaCSV(Request $request)
    {
        $fechaInicio = $request->fechaInicio;
        $fechaFin = $request->fechaFin;
        $startDate = $fechaInicio;
        $endDate = $fechaFin;
        $anulada = 1;
        $idBodega = 4;
        $tipoReporte = $request->tipoReporte;
        if ($tipoReporte == 1) { //ANULADAS
            return Excel::download(new VentaExport($startDate, $endDate, $anulada, $idBodega), 'facturasanuladas.csv');
        }else if ($tipoReporte == 2) { //CREDITO FISCAL
            return Excel::download(new VentaCreditoFiscalExport($startDate, $endDate, $anulada, $idBodega), 'facturascreditofiscal.csv');
        }else if ($tipoReporte == 3) { //CONSUMIDOR FINAL
            return Excel::download(new VentaConsumidorFinal($startDate, $endDate, $anulada, $idBodega), 'facturasconsumidorfinal.csv');
        }else{ //COMPRAS PROVEEDORES
            return Excel::download(new CompraProveedorExport($startDate, $endDate), 'compraproveedores.csv');
        }
    }

    public function getFactura(Request $request){
        $venta = Venta::findOrFail($request->id);
        $cliente =  Cliente::findOrFail($venta->idCliente);
        $detalleVenta  = DetalleVenta::where('idFactura','=',$venta->id)->get();
        $detalleVenta->each(function ($detalle) {
            if ($detalle->servicio_conta  == 1) {
                $detalle->producto = CuentaContable::find($detalle->idProducto);
            } else {
                $detalle->producto = Articulo::find($detalle->idProducto);
            }
        });
        return [
            'venta' => $venta,
            'cliente' => $cliente,
            'detalleVenta' => $detalleVenta
        ];
    }
}
