<?php

namespace App\Services;

use App\Venta;
use GuzzleHttp\Client;
use App\FacturaElectronicaLog;
use App\Http\Controllers\FacturaElectronicaController;
use App\Parameter;
use Carbon\Carbon;
use GuzzleHttp\Exception\RequestException;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;

class FacturaElectronicaServiceService
{
    protected $client;
    protected $baseUrl;
    protected $token;
    protected $ambiente;
    protected $tipoDte;
    protected $tipoDocumento;
    protected $version;
    protected $user;
    protected $pwd;
    protected $pwdPri;
    public function __construct($baseUrl = 'https://apitest.dtes.mh.gob.sv/')
    {
        $this->baseUrl = $baseUrl;

        $this->client = new Client([
            'base_uri' => $this->baseUrl,
            'timeout'  => 30,
            'verify' => false
        ]);
        $this->token = Parameter::where('name', 'token')->first();
        $this->ambiente = Parameter::where('name', 'ambiente')->first();
        $this->user = Parameter::where('name', 'user')->first();
        $this->pwd = Parameter::where('name', 'pwd')->first();
        $this->pwdPri = Parameter::where('name', 'pwdPrivado')->first();
    }

    public function authUser()
    {

        try {
            $response = $this->client->request('POST', 'seguridad/auth', [
                'headers' => [
                    'Content-Type' => 'application/x-www-form-urlencoded',
                    'User-Agent'   => 'SERCOM',
                ],
                'form_params' => [
                    'user' => $this->user->value,
                    'pwd'  => $this->pwd->value,
                ],
                'verify' => false
            ]);

            // Get the JSON response and decode it
            $jsonResponse = $response->getBody()->getContents();
            $decodedResponse = json_decode($jsonResponse, true);
            $log = new FacturaElectronicaLog();

            $log->serviceType = "LOGIN";
            $log->jsonResponse =  $jsonResponse;
            $log->save();

            Parameter::updateOrCreate(
                ['name' => 'token'], // Criteria for finding the record
                [
                    'name' => "token",
                    'value' => $decodedResponse['body']['token'],
                ]
            );
            return $decodedResponse;
        } catch (\Exception $e) {
            // Handle exceptions (e.g., log the error)
            // Log::error('Error: ' . $e->getMessage());
            throw new \Exception('Error: ' . $e->getMessage());
        }
    }

    public function signDocument(array $facturaJson, int $idFactura, String $tipoDte,String $tipoDocumento, int $enContingencia, $tipoFirma)
    {
        //TIPO FIRMA 1 - FIRMA DE DFACTURA
        //TIPO FIRMA 2 - FIRMA DOCUMENTO DE ANULACION
        $codigoGeneracion = $facturaJson['identificacion']['codigoGeneracion'];
        $this->tipoDte = $tipoDte;
        $this->tipoDocumento = $tipoDocumento;
        $clientFirmador =  new Client([
            'base_uri' => "http://localhost:8113/",
            'timeout'  => 30,
        ]);

        try {
            $response = $clientFirmador->request('POST', 'firmardocumento/', [
                'headers' => [
                    'Content-Type' => 'application/json',
                ],
                'json' => [
                    // Add your JSON payload here if needed
                    // Example: 'key' => 'value'
                    'nit' => $this->user->value,
                    'activo' => true,
                    'passwordPri' => $this->pwdPri->value,
                    'dteJson' => $facturaJson
                ],
            ]);

            // Get the JSON response and decode it
            $jsonResponse = $response->getBody()->getContents();
            $decodedResponse = json_decode($jsonResponse, true);
            $log = new FacturaElectronicaLog();
            $log->codigoGeneracion = $codigoGeneracion;
            $log->idFactura = $idFactura;
            $log->jsonResponse =  $jsonResponse;
            if ($enContingencia == 1) {
                return $decodedResponse;
            }
            if (self::isTokenExpired()) {
                self::authUser();
                $documentoFirmado = $decodedResponse["body"];
                if ($tipoFirma == 1) {
                    $log->serviceType = "FIRMA_FACTURA";
                    $log->save();
                    $decodedResponseSend = self::sendDocument($facturaJson,$documentoFirmado, $idFactura, $codigoGeneracion);
                } else {
                    $log->serviceType = "FIRMA_ANULACION";
                    $log->save();
                    $decodedResponseSend = self::anularDocumento($documentoFirmado, $idFactura, $codigoGeneracion);
                }
                return $decodedResponseSend;
            } else {
                $documentoFirmado = $decodedResponse["body"];
                if ($tipoFirma == 1) {
                    $log->serviceType = "FIRMA_FACTURA";
                    $log->save();
                    $decodedResponseSend = self::sendDocument($facturaJson,$documentoFirmado, $idFactura, $codigoGeneracion);
                } else {
                    $log->serviceType = "FIRMA_ANULACION";
                    $log->save();
                    $decodedResponseSend = self::anularDocumento($documentoFirmado, $idFactura, $codigoGeneracion);
                }
                return $decodedResponseSend;
            }
        } catch (\Exception $e) {
            // Handle exceptions (e.g., log the error)
            // Log::error('Error: ' . $e->getMessage());
            throw new \Exception('Error: ' . $e->getMessage());
            return 'Error: ' . $e->getMessage();
        }
    }

    public function sendDocument(array $facturaJson,String $documento, int $idFactura, String $codigoGeneracion)
    {
        try {
            $response = $this->client->request('POST', 'fesv/recepciondte', [
                'headers' => [
                    'Content-Type' => 'application/json',
                    'User-Agent' => 'SERCOM',
                    'Authorization' => $this->token->value,
                ],
                'json' => [
                    'ambiente' => $this->ambiente->value,
                    'version' => intval($this->tipoDte),
                    'idEnvio' => $idFactura,
                    'tipoDte' => $this->tipoDocumento,
                    'documento' => $documento,
                    'codigoGeneracion' => $codigoGeneracion
                ],
            ]);

            // Get the JSON response and decode it
            $jsonResponse = $response->getBody()->getContents();
            $decodedResponse = json_decode($jsonResponse, true);
            $log = new FacturaElectronicaLog();
            $log->idFactura = $idFactura;
            $log->codigoGeneracion = $codigoGeneracion;
            $log->serviceType = "ENVIO";
            $log->jsonResponse =  $jsonResponse;
            $log->save();
            $venta = Venta::with('ventaCliente.clienteGiro', 'ventaUsuario','formaPago','codicionVenta','documentoRelacionado','recintoFiscalRelacion','regimenRelacion','paisRelacion')->findOrFail($idFactura);
            $venta->selloRecibido = $decodedResponse["selloRecibido"];
            $venta->save();
            if (!is_null($decodedResponse["selloRecibido"]) && trim($decodedResponse["selloRecibido"]) !== '') {
                $facturaElectronicaController = new FacturaElectronicaController();
                $facturaElectronicaController->generatePDF($venta);
                // Convert the unified data to a JSON response
                $facturaJson['firmaElectronica'] = $documento;
                $facturaJson['selloRecibido'] = $venta->selloRecibido;
                $jsonContent = json_encode($facturaJson, JSON_PRETTY_PRINT | JSON_PRESERVE_ZERO_FRACTION);
                $nombreJson = $codigoGeneracion.".json";
                // if ($venta->tipo_documento == "03") { //CREDITO FISCAL
                //     $nombreJson = "ccf".$venta->id.".json";
                // }else{
                //     $nombreJson = "cf".$venta->id.".json";
                // }
                // Use the Storage facade to save the JSON content to a file
                Storage::disk('jsonfacturas')->put($nombreJson, $jsonContent);
                //ENVIAMOS EL CORREO 
                if (!empty($venta->ventaCliente->email)) {
                    self::sendEmailWithAttachments($venta->codigoGeneracion,$venta->ventaCliente);
                } 
            } 
            return $decodedResponse;
        } catch (RequestException $e) {
            if ($e->hasResponse()) {
                $errorResponse = $e->getResponse();
                $statusCode = $errorResponse->getStatusCode();
                $body = $errorResponse->getBody()->getContents();
                $reasonPhrase = $errorResponse->getReasonPhrase();
                $decodedResponse = json_decode($body, true);
                $log = new FacturaElectronicaLog();
                $log->idFactura = $idFactura;
                $log->codigoGeneracion =  $codigoGeneracion;
                $log->serviceType = "ENVIO";
                $log->jsonResponse = $body;
                $log->save();
                return $decodedResponse;
            } else {
                // Handle the case when no response was received
                echo "No response received for the request.\n";
            }
        }
    }

    public function anularDocumento(String $documento, int $idFactura, String $codigoGeneracion)
    {
        try {
            $response = $this->client->request('POST', 'fesv/anulardte', [
                'headers' => [
                    'Content-Type' => 'application/json',
                    'User-Agent' => 'SERCOM',
                    'Authorization' => $this->token->value,
                ],
                'json' => [
                    'ambiente' => $this->ambiente->value,
                    'idEnvio' => $idFactura,
                    'version' => 2,
                    'documento' => $documento
                ],
            ]);

            // Get the JSON response and decode it
            $jsonResponse = $response->getBody()->getContents();
            $decodedResponse = json_decode($jsonResponse, true);
            $log = new FacturaElectronicaLog();
            $log->idFactura = $idFactura;
            $log->codigoGeneracion = $codigoGeneracion;
            $log->serviceType = "ENVIO_ANULACION";
            $log->jsonResponse =  $jsonResponse;
            $log->save();
            return $decodedResponse;
        } catch (RequestException $e) {
            if ($e->hasResponse()) {
                $errorResponse = $e->getResponse();
                $statusCode = $errorResponse->getStatusCode();
                $body = $errorResponse->getBody()->getContents();
                $reasonPhrase = $errorResponse->getReasonPhrase();
                $decodedResponse = json_decode($body, true);
                $log = new FacturaElectronicaLog();
                $log->idFactura = $idFactura;
                $log->codigoGeneracion =  $codigoGeneracion;
                $log->serviceType = "ENVIO_ANULACION";
                $log->jsonResponse = $body;
                $log->save();
                return $decodedResponse;
            } else {
                // Handle the case when no response was received
                echo "No response received for the request.\n";
            }
        }
    }

    //UTILS
    function isTokenExpired()
    {
        // Convert date-time strings to Carbon objects for easy comparison
        $savedDateTime = Carbon::parse($this->token->updated_at);
        $currentDateTime = Carbon::now();

        // Calculate the difference in hours between two dates
        $hoursDifference = $currentDateTime->diffInHours($savedDateTime);

        // Check if the difference is greater than 24 hours
        return $hoursDifference > 24;
    }

    public function sendEmailWithAttachments($codigoGeneracion,$cliente)
    {
        // Your JSON content
        $jsonContent = Storage::disk('jsonfacturas')->get($codigoGeneracion.'.json');

        // PDF file path
        $pdfContent = Storage::disk('pdfFacturas')->get($codigoGeneracion.'.pdf');
        
        // Send email with attachments
        Mail::send([], [], function ($message) use ($jsonContent, $pdfContent,$codigoGeneracion,$cliente) {
            $message->to($cliente->email);
            $message->subject('Documento Tributario Electrónico');
            $message->from('dte@sistemasenlanube.com','dte@sistemasenlanube.com');

            // Attach JSON file
            $message->attachData($jsonContent, $codigoGeneracion.'.json', [
                'mime' => 'application/json',
            ]);

            // Attach PDF file
            $message->attachData($pdfContent, $codigoGeneracion.'.pdf', [
                'mime' => 'application/pdf',
            ]);

            $texto = "¡Gracias por elegir SERCOM EL SALVADOR!\n\n";
            $texto .= "Esperamos que su experiencia haya sido satisfactoria.\n";
            $texto.= "En este correo electrónico podrá encontrar los archivos adjuntos correspondientes a su documento electrónico en el cual se encuentra:\n\n";
            $texto.= "- Documento electrónico JSON.\n";
            $texto.= "- Representación gráfica del documento PDF.";
            $message->setBody($texto, 'text/plain');
        });

        return "Email sent successfully!";
    }
}
